@@ -20,6 +20,8 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
+#include <Lum/Model/Action.h>
+
 #include <Lum/AppInfo.h>
 #include <Lum/Object.h>
 
@@ -33,6 +35,8 @@ namespace Lum {
   private:
     AppInfo info;
 
+    Model::ActionRef illuminationAction;
+
   public:
     About();
     virtual ~About();
