@@ -144,10 +144,13 @@ namespace Lum {
       public:
         std::wstring     name;
         bool             isDir;
-        bool             isLink;
+        bool             isSoftLink;
+        bool             isHardLink;
         bool             isHidden;
 
         unsigned long    size;
+        unsigned long    blockSize;
+        unsigned long    linkId;
         SystemTime       timeModification;
 
         unsigned short   user;
