@@ -23,13 +23,14 @@
 #include <Lum/Private/ImportExport.h>
 
 #include <Lum/AppInfo.h>
-#include <Lum/Dialog.h>
+
+#include <Lum/Dlg/ActionDialog.h>
 
 #include <Lum/Model/Action.h>
 
 namespace Lum {
   namespace Dlg {
-    class LUMAPI About : public Dialog
+    class LUMAPI About : public ActionDialog
     {
     private:
       Lum::AppInfo info;
@@ -38,7 +39,8 @@ namespace Lum {
       About();
       ~About();
 
-      void PreInit();
+      Lum::Object* GetContent();
+      void GetActions(std::vector<Lum::Dlg::ActionInfo>& actions);
 
       void SetAppInfo(const AppInfo& info);
 
