@@ -30,13 +30,6 @@ namespace Lum {
 
   class LUMAPI Image : public Object
   {
-  public:
-    class Prefs : public Object::Prefs
-    {
-    public:
-      Prefs(const std::wstring& name);
-    };
-
   private:
     Images::ImageRef extImage;
     OS::ImageRef     osImage;
