@@ -30,7 +30,10 @@ namespace Lum {
   class LUMAPI LED : public ImageState
   {
   public:
-    LED();
+    LED(Base::Model* model=NULL);
+
+    static LED* Create();
+    static LED* Create(Base::Model* model);
   };
 }
 
