@@ -271,6 +271,7 @@ namespace Lum {
     void CalcSize();
     void Layout();
     bool HandleMouseEvent(const OS::MouseEvent& event);
+    void PrepareForBackground(OS::DrawInfo* draw);
     void Draw(int x, int y, size_t w, size_t h);
     void Hide();
     void Resync(Base::Model* model, const Base::ResyncMsg& msg);
@@ -323,8 +324,6 @@ namespace Lum {
     void Disable(bool disable);
     void Hide();
     void SetHotkey(wchar_t key);
-    void CalcLabelSize();
-    void CalcShortcutSize();
 
     void DrawImage(OS::DrawInfo* draw);
     void DrawLabel(OS::DrawInfo* draw);
@@ -345,6 +344,8 @@ namespace Lum {
     void SetShortcut(unsigned long qualifier, const std::wstring& key);
     void SetTextLabel(const std::wstring& label);
 
+    void CalcSize();
+
     void Resync(Base::Model* model, const Base::ResyncMsg& msg);
   };
 
@@ -362,7 +363,7 @@ namespace Lum {
     bool SetModel(Base::Model* model);
     virtual void OnSelection();
 
-    void CalcSize();
+    void PrepareForBackground(OS::DrawInfo* draw);
     void Draw(int x, int y, size_t w, size_t h);
   };
 
@@ -405,6 +406,7 @@ namespace Lum {
 
     void RegisterToParentWindow(OS::Window* parent);
     void CalcSize();
+    void PrepareForBackground(OS::DrawInfo* draw);
     void Draw(int x, int y, size_t w, size_t h);
 
     Menu* GetSubMenu() const;
@@ -424,9 +426,9 @@ namespace Lum {
     bool SetModel(Base::Model* model);
     void SetValue(size_t value);
 
-    virtual void OnSelection();
+    void OnSelection();
 
-    void CalcSize();
+    void PrepareForBackground(OS::DrawInfo* draw);
     void Draw(int x, int y, size_t w, size_t h);
   };
 
@@ -445,9 +447,9 @@ namespace Lum {
     bool SetModel(Base::Model* model);
     void SetValue(size_t value);
 
-    virtual void OnSelection();
+    void OnSelection();
 
-    void CalcSize();
+    void PrepareForBackground(OS::DrawInfo* draw);
     void Draw(int x, int y, size_t w, size_t h);
   };
 
@@ -465,6 +467,9 @@ namespace Lum {
     bool        popup;       //! Top menu of a popup menu
     bool        registered;
     Lum::Object *reference;  //! Optional control the window should be opened relative to
+    bool        manualPos;   //! If true, xPos and yPos hold the position of the menu
+    int         xPos;
+    int         yPos;
 
   public:
     SubMenu     *parentMenu; //! Parent menu
@@ -480,6 +485,7 @@ namespace Lum {
     MenuStrip* GetStrip();
     Menu* GetTop();
     Menu* GetMenuWithSelection();
+    Menu* GetMenuWithMouseSelection();
     void CloseAll();
     void EndAction();
     void Select(MenuEntry* entry);
@@ -487,6 +493,7 @@ namespace Lum {
     void SelectDown();
     void SelectLeft();
     void SelectRight();
+    void ActivateSelection();
     void SelectKey(wchar_t key);
 
   public:
@@ -498,6 +505,7 @@ namespace Lum {
 
     void SetParent(OS::Window* parent);
     void SetReference(Lum::Object* reference);
+    void SetPos(int x, int y);
 
     void CalcSize();
     void PreInit();
