@@ -39,6 +39,7 @@ namespace Lum {
         FillRef   fill[Display::fillCount];
         FrameRef  frame[Display::frameCount];
         ImageRef  image[Display::imageCount];
+        ImageRef  stockImage[stockImageCount];
 
         FrameRef  emptyFrame;
 
@@ -52,9 +53,19 @@ namespace Lum {
         Fill* GetFill(Display::FillIndex fill) const;
         Frame* GetFrame(Display::FrameIndex frame) const;
         Image* GetImage(Display::ImageIndex image) const;
+        Image* GetStockImage(StockImage image) const;
 
         void PushUniqueFill(OS::DrawInfo* draw, size_t index, size_t maximum);
         void PopUniqueFill(OS::DrawInfo* draw, size_t index, size_t maximum);
+
+        void GetMenuTypeActionPopupOffset(int& x, int& y) const;
+        int GetSubMenuHorizontalOffset() const;
+        bool RequestFingerFriendlyControls() const;
+
+        bool SubdialogsAlwaysFlexHoriz() const;
+        bool SubdialogsAlwaysFlexVert() const;
+
+        DialogActionPosition GetDialogActionPosition() const;
       };
 
       class DefaultTheme : public Theme
@@ -80,30 +91,18 @@ namespace Lum {
         std::wstring GetFixedFontName() const;
         size_t GetFixedFontSize() const;
 
-        void GetObjectSize(ObjectType object,
-                           size_t& width,
-                           size_t& height) const;
-
-        bool NeedsObjectParentDraw(ObjectType object) const;
-
-        void DrawObject(OS::DrawInfo* draw,
-                        ObjectType object,
-                        int oX, int oY, size_t oWidth, size_t oHeight,
-                        int x, int y, size_t width, size_t height);
-
         size_t GetSpaceHorizontal(Display::Space space) const;
         size_t GetSpaceVertical(Display::Space space) const;
         size_t GetMinimalButtonWidth() const;
         size_t GetFirstTabOffset() const;
         size_t GetLastTabOffset() const;
-        size_t GetEntryHorizTextOffset() const;
-        size_t GetEntryVertTextOffset() const;
         size_t GetDragStartSensitivity() const;
         size_t GetMouseClickHoldSensitivity() const;
 
         ScrollKnobMode GetScrollKnobMode() const;
         ToolbarPosition GetToolbarPosition() const;
         bool ShowButtonImages() const;
+
         MenuType GetMenuType() const;
       };
     }
