@@ -43,10 +43,13 @@ namespace Lum {
   namespace OS {
 
     class Display;
-    class KeyboardHandler;
     class Theme;
     class Window;
 
+    namespace Manager {
+      class KeyboardManager;
+    }
+
     /**
       Display represents the graphical display with all its global attributes.
     */
@@ -54,11 +57,11 @@ namespace Lum {
     {
 
     public:
-      static const size_t colorCount = 12; //! Number of predefined colors
+      static const size_t colorCount = 14; //! Number of predefined colors
       static const size_t fontCount  =  2; //! Number of predefined fonts
       static const size_t fillCount  = 42; //! Number of predefined fills
       static const size_t frameCount =  7; //! Number of predefined frames
-      static const size_t imageCount = 31; //! Number of predefined images
+      static const size_t imageCount = 29; //! Number of predefined images
 
     public:
       enum ColorIndex {
@@ -73,7 +76,9 @@ namespace Lum {
         blackColor                    =  8, // To be removed!
         whiteColor                    =  9, // To be removed!
         tabTextColor                  = 10, //! Font color for a textual tab rider
-        tabTextSelectColor            = 11  //! Font color of a selected textual tab rider
+        tabTextSelectColor            = 11, //! Font color of a selected textual tab rider
+        editTextColor                 = 12, //! Font color for edit fields and similar
+        editTextFillColor             = 13  //! Font color for selected texts in edit fields and similar
       };
 
       enum FillIndex {
@@ -131,6 +136,9 @@ namespace Lum {
         groupFrameIndex           =  6  //! Frame for grouping a number of elements
       };
 
+      /**
+        List of necessary theming pecific images for visualisation of controls.
+       */
       enum ImageIndex {
         hknobImageIndex            =  0, //! Image of a horizontal scrolling knob
         vknobImageIndex            =  1, //! Image of a vertical scrolling knob
@@ -154,15 +162,13 @@ namespace Lum {
         bottomSliderImageIndex     = 19, //! bottom (filled) part of the slider control
         menuDividerImageIndex      = 20, //! Divider between menu entry groups
         menuSubImageIndex          = 21, //! Image for signaling a sub menu
-        menuCheckImageIndex        = 22,
-        menuRadioImageIndex        = 23,
-        positiveImageIndex         = 24, //! Image for marking a positive action (e.g. for a button)
-        negativeImageIndex         = 25, //! Image for marking a negative action (e.g. for a button)
-        defaultImageIndex          = 26, //! Image for marking a default action (e.g. for a button)
-        helpImageIndex             = 27, //! Image for marking a help action (e.g. for a button)
-        closeImageIndex            = 28, //! Image for closing widget
-        hpaneHandleImageIndex      = 29, //! Grip object for the pane control
-        vpaneHandleImageIndex      = 30  //! Grip object for the pane control
+        menuCheckImageIndex        = 22, //! Image for checkmark in menues
+        menuRadioImageIndex        = 23, //! Image for radio button in menues
+        hpaneHandleImageIndex      = 24, //! Grip object for the pane control
+        vpaneHandleImageIndex      = 25, //! Grip object for the pane control
+        radioImageIndex            = 26, //! Radio control image
+        checkImageIndex            = 27, //! Checkbox image
+        ledImageIndex              = 28  //! LED image (for read-only signaling of boolean state)
       };
 
       enum FontScale {
@@ -273,12 +279,19 @@ namespace Lum {
         Method for font management.
       */
       //@{
+      /**
+        Get a default font of the given size, where
+        the size is handed to the function as percent points in relation
+        to the default font size.
+      */
+      virtual Font* GetFont(size_t scale=fontScaleNormal) const = 0;
+
       /**
         Get a font of the given type and the given size, where
         the size is handed to the function as percent points in relation
         to the default font size of the given font type.
       */
-      virtual Font* GetFont(FontType type=fontTypeProportional,
+      virtual Font* GetFont(FontType type,
                             size_t scale=fontScaleNormal) const = 0;
 
       /**
@@ -352,10 +365,10 @@ namespace Lum {
         exlipcitely be copied a selection is imediately active.
       */
       //@{
-      virtual bool RegisterSelection(Lum::Base::DnDObject* object, Window* window) = 0;
+      virtual bool RegisterSelection(Lum::Base::DataExchangeObject* object, Window* window) = 0;
       virtual void CancelSelection() = 0;
-      virtual ::Lum::Base::DnDObject* GetSelectionOwner() const = 0;
-      virtual bool QuerySelection(Window* window, Lum::Base::DnDObject* object) = 0;
+      virtual Lum::Base::DataExchangeObject* GetSelectionOwner() const = 0;
+      virtual bool QuerySelection(Window* window, Lum::Base::DataExchangeObject* object) = 0;
       //@}
 
       /**
@@ -504,7 +517,7 @@ namespace Lum {
         input methods.
       */
       //@{
-      virtual KeyboardHandler* GetKeyboardHandler() const = 0;
+      virtual Manager::KeyboardManager* GetKeyboardManager() const = 0;
       //@}
 
 
