@@ -40,11 +40,20 @@ namespace Lum {
     class LUMAPI Theme
     {
     public:
-      enum ObjectType {
-        // State widgets
-        objRadio,
-        objCheck,
-        objLED
+      static const size_t stockImageCount = 8; //! Number of predefined stock images
+
+      /**
+        List of (optional!) images for enriching visual controls.
+       */
+      enum StockImage {
+        positiveStockImage  =  0, //! Image for marking a positive action (e.g. for a button)
+        negativeStockImage  =  1, //! Image for marking a negative action (e.g. for a button)
+        defaultStockImage   =  2, //! Image for marking a default action (e.g. for a button)
+        helpStockImage      =  3, //! Image for marking a help action (e.g. for a button)
+        closeStockImage     =  4, //! Image for closing widget
+        addStockImage       =  5, //! Image for adding somethng
+        removeStockImage    =  6, //! Image for removing something
+        editStockImage      =  7  //! Image for editing something
       };
 
       enum ScrollKnobMode {
@@ -63,6 +72,11 @@ namespace Lum {
         menuTypeActionPopup
       };
 
+      enum DialogActionPosition {
+        actionPosHorizButtonRowBelow,
+        actionPosVertButtonRowRight
+      };
+
     public:
       Theme(Display *display);
       virtual ~Theme();
@@ -90,33 +104,12 @@ namespace Lum {
       virtual Fill* GetFill(Display::FillIndex fill) const = 0;
       virtual Frame* GetFrame(Display::FrameIndex frame) const = 0;
       virtual Image* GetImage(Display::ImageIndex image) const = 0;
+      virtual Image* GetStockImage(StockImage image) const = 0;
 
       virtual void PushUniqueFill(DrawInfo* draw, size_t index, size_t maximum) = 0;
       virtual void PopUniqueFill(DrawInfo* draw, size_t index, size_t maximum) = 0;
       //@}
 
-      /**
-        Return the size of an object with out content.
-      */
-      virtual void GetObjectSize(ObjectType object,
-                                 size_t& width,
-                                 size_t& height) const = 0;
-
-      /**
-        Returns true, if the objects needs the parent to fill
-        the content area first.
-       */
-      virtual bool NeedsObjectParentDraw(ObjectType object) const = 0;
-
-      /**
-        Draws the object itself, you have to draw childs
-        onto the area afterwards.
-       */
-      virtual void DrawObject(DrawInfo* draw,
-                              ObjectType object,
-                              int x, int y, size_t width, size_t height,
-                              int cX, int cY, size_t cWidth, size_t cHeight) = 0;
-
       /**
         @name Spaces
         Theme specific spacing.
@@ -128,8 +121,7 @@ namespace Lum {
       virtual size_t GetMinimalButtonWidth() const = 0;
       virtual size_t GetFirstTabOffset() const = 0;
       virtual size_t GetLastTabOffset() const = 0;
-      virtual size_t GetEntryHorizTextOffset() const = 0;
-      virtual size_t GetEntryVertTextOffset() const = 0;
+
       /**
         How many pixels must the mouse move before this is recognized as drag start?
       */
@@ -149,7 +141,16 @@ namespace Lum {
       virtual ScrollKnobMode GetScrollKnobMode() const = 0;
       virtual ToolbarPosition GetToolbarPosition() const = 0;
       virtual bool ShowButtonImages() const = 0;
+      virtual bool RequestFingerFriendlyControls() const = 0;
+
       virtual MenuType GetMenuType() const = 0;
+      virtual void GetMenuTypeActionPopupOffset(int& x, int& y) const = 0;
+      virtual int GetSubMenuHorizontalOffset() const = 0;
+
+      virtual bool SubdialogsAlwaysFlexHoriz() const = 0;
+      virtual bool SubdialogsAlwaysFlexVert() const = 0;
+
+      virtual DialogActionPosition GetDialogActionPosition() const =0;
       //@}
     };
   }
