@@ -24,6 +24,7 @@
 
 #include <Lum/OS/Win32/Display.h>
 #include <Lum/OS/Win32/Event.h>
+#include <Lum/OS/Win32/EventLoop.h>
 
 namespace Lum {
   namespace OS {
@@ -36,7 +37,6 @@ namespace Lum {
       private:
         Display     *display;
         bool        grab;
-        bool        exit;
 
         MouseEvent  oldButton;
         MouseEvent  lastButton;
@@ -83,19 +83,14 @@ namespace Lum {
 
         bool GetMousePos(int& x, int& y) const;
 
-        void EventLoop();
-
         void Enable();
         void Disable();
 
         bool SendBalloonMessage(const std::wstring& text,unsigned long timeout);
 
-        bool IsInEventLoop() const;
         bool IsDoubleClicked() const;
         bool IsTrippleClicked() const;
 
-        void Exit();
-
         LRESULT MessageHandler(HWND wnd,UINT msg, WPARAM wParam, LPARAM lParam);
       };
     }
