@@ -82,11 +82,6 @@ namespace Lum {
         ::GtkWidget     *layout;
         ::GtkAdjustment *adjustment;
         ::GtkWidget     *widget[styleMax];
-        ::GdkPixbuf     *positiveIcon;
-        ::GdkPixbuf     *negativeIcon;
-        ::GdkPixbuf     *defaultIcon;
-        ::GdkPixbuf     *helpIcon;
-        ::GdkPixbuf     *closeIcon;
 
       public:
         GtkWidgets();
@@ -99,7 +94,7 @@ namespace Lum {
 
       extern GtkWidgets *gtkWidgets;
 
-      class GtkImage : public ::Lum::OS::Image
+      class GtkImage : public OS::Image
       {
       private:
         Display::ImageIndex image;
@@ -114,7 +109,31 @@ namespace Lum {
         bool GetAlpha() const;
         unsigned long GetDrawCap() const;
 
-        void Draw(::Lum::OS::DrawInfo *draw, int x, int y, size_t w, size_t h);
+        void Draw(OS::DrawInfo *draw, int x, int y, size_t w, size_t h);
+      };
+
+      class GtkStockImage : public OS::Image
+      {
+      private:
+        const gchar *gtkStockId;
+        GtkIconSet  *gtkIconSet;
+        GdkPixbuf   *gdkPixbufNormal;
+        GdkPixbuf   *gdkPixbufDisabled;
+        GdkPixbuf   *gdkPixbufSelected;
+        GdkPixbuf   *gdkPixbufActivated;
+        size_t      width;
+        size_t      height;
+
+      public:
+        GtkStockImage(const gchar* gtkStockId);
+        ~GtkStockImage();
+
+        size_t GetWidth() const;
+        size_t GetHeight() const;
+        bool GetAlpha() const;
+        unsigned long GetDrawCap() const;
+
+        void Draw(OS::DrawInfo *draw, int x, int y, size_t w, size_t h);
       };
 
       class GtkFill : public Fill
@@ -125,7 +144,7 @@ namespace Lum {
       public:
         GtkFill(Display::FillIndex fill);
 
-        void Draw(::Lum::OS::DrawInfo* draw,
+        void Draw(OS::DrawInfo* draw,
                   int xOff, int yOff, size_t width, size_t height,
                   int x, int y, size_t w, size_t h);
       };
@@ -138,7 +157,7 @@ namespace Lum {
       public:
         GtkFrame(Display::FrameIndex frame);
 
-        void Draw(::Lum::OS::DrawInfo* draw, int x, int y, size_t w, size_t h);
+        void Draw(OS::DrawInfo* draw, int x, int y, size_t w, size_t h);
       };
 
       extern int GetWidgetStyleIntValue(GtkWidget *widget, const char* name);
@@ -146,7 +165,7 @@ namespace Lum {
       extern void GetWidgetStyleBorderValue(GtkWidget *widget, const char* name,
                                             int& left, int&right, int &top, int& bottom);
 
-      class GtkTheme : public ::Lum::OS::Base::Theme
+      class GtkTheme : public OS::Base::Theme
       {
       public:
         Color          helpBackgroundColor;
@@ -158,7 +177,7 @@ namespace Lum {
         ScrollKnobMode scrollKnobMode;
 
       public:
-        GtkTheme(::Lum::OS::Display *display);
+        GtkTheme(OS::Display *display);
         ~GtkTheme();
 
         bool HasFontSettings() const;
@@ -169,31 +188,25 @@ namespace Lum {
         std::wstring GetFixedFontName() const;
         size_t GetFixedFontSize() const;
 
-        void GetObjectSize(ObjectType object,
-                           size_t& width,
-                           size_t& height) const;
-
-        bool NeedsObjectParentDraw(ObjectType object) const;
-
-        void DrawObject(OS::DrawInfo* draw,
-                        ObjectType object,
-                        int x, int y, size_t width, size_t height,
-                        int cX, int cY, size_t cWidth, size_t cHeight);
-
         size_t GetSpaceHorizontal(OS::Display::Space space) const;
         size_t GetSpaceVertical(OS::Display::Space space) const;
         size_t GetMinimalButtonWidth() const;
         size_t GetFirstTabOffset() const;
         size_t GetLastTabOffset() const;
-        size_t GetEntryHorizTextOffset() const;
-        size_t GetEntryVertTextOffset() const;
         size_t GetDragStartSensitivity() const;
         size_t GetMouseClickHoldSensitivity() const;
 
         ScrollKnobMode GetScrollKnobMode() const;
         ToolbarPosition GetToolbarPosition() const;
         bool ShowButtonImages() const;
+        bool RequestFingerFriendlyControls() const;
+
         MenuType GetMenuType() const;
+        void GetMenuTypeActionPopupOffset(int& x, int& y) const;
+        int GetSubMenuHorizontalOffset() const;
+
+        bool SubdialogsAlwaysFlexHoriz() const;
+        DialogActionPosition GetDialogActionPosition() const;
       };
 
     }
