@@ -96,7 +96,7 @@ namespace Lum {
    - Drawing and refreshing
    - Keyboard Focus handling
   */
-  class LUMAPI Object : public Base::DnDObject
+  class LUMAPI Object : public Base::DataExchangeObject
   {
   public:
     enum Constraint
@@ -166,10 +166,12 @@ namespace Lum {
     OS::FillRef         background;    //! The object background. \see Object::SetBackground
     //@}
 
-    bool                horizontalFlex,verticalFlex;
-    bool                visible;       //! The object is currently visible
-    bool                firstDraw;     //! If true, this is the first drawing of the widget
-    bool                layouted;      //! The object has calculated its layout (can be reseted by move/resize)
+    bool                horizontalFlex; //! Object can change its size horizontically.
+    bool                verticalFlex;   //! Object can change its size vertically.
+    bool                visible;        //! The object is currently visible
+    bool                firstDraw;      //! If true, this is the first drawing of the widget
+    bool                layouted;       //! The object has calculated its layout (can be reseted by move/resize)
+    bool                clipFromParent; //! Parent objects must not draw into the area of this object (drawing of parent must be cliped)
 
   private:
     /**
@@ -231,6 +233,13 @@ namespace Lum {
     virtual void UnrequestFocus();
 
     virtual void SetRedrawOnMouseActive(bool redraw);
+    virtual void SetClipFromParent(bool clip);
+
+    virtual void DrawChildren(OS::DrawInfo* draw, int x, int y, size_t w, size_t h);
+    virtual void PreDrawChilds(OS::DrawInfo* draw);
+    virtual void PostDrawChilds(OS::DrawInfo* draw);
+    virtual void PreDrawChild(OS::DrawInfo* draw, Object* child);
+    virtual void PostDrawChild(OS::DrawInfo* draw, Object* child);
 
   public:
     virtual ~Object();
@@ -279,29 +288,29 @@ namespace Lum {
       Methods setting the minimal, current and maximum sizeof the object.
     */
     //@{
-    virtual Object& SetConstraint(Constraint constraint,
+    virtual Object* SetConstraint(Constraint constraint,
                                   Base::Size::Mode mode,
                                   size_t value,
                                   const OS::FontRef& font=::Lum::OS::FontRef());
 
-    virtual Object& SetWidth(Base::Size::Mode mode,
+    virtual Object* SetWidth(Base::Size::Mode mode,
                              size_t value,
                              const OS::FontRef& font=::Lum::OS::FontRef());
-    virtual Object& SetHeight(Base::Size::Mode mode,
+    virtual Object* SetHeight(Base::Size::Mode mode,
                               size_t value,
                               const OS::FontRef& font=::Lum::OS::FontRef());
 
-    virtual Object& SetMinWidth(Base::Size::Mode mode,
+    virtual Object* SetMinWidth(Base::Size::Mode mode,
                                 size_t value,
                                 const OS::FontRef& font=::Lum::OS::FontRef());
-    virtual Object& SetMinHeight(Base::Size::Mode mode,
+    virtual Object* SetMinHeight(Base::Size::Mode mode,
                                  size_t value,
                                  const OS::FontRef& font=::Lum::OS::FontRef());
 
-    virtual Object& SetMaxWidth(Base::Size::Mode mode,
+    virtual Object* SetMaxWidth(Base::Size::Mode mode,
                                 size_t value,
                                 const OS::FontRef& font=::Lum::OS::FontRef());
-    virtual Object& SetMaxHeight(Base::Size::Mode mode,
+    virtual Object* SetMaxHeight(Base::Size::Mode mode,
                                  size_t value,
                                  const OS::FontRef& font=::Lum::OS::FontRef());
     //@}
@@ -310,7 +319,7 @@ namespace Lum {
       @name Methods for manipulation object resize behaviour
     */
     //@{
-    virtual Object& SetFlex(bool horizontal, bool vertical);
+    virtual Object* SetFlex(bool horizontal, bool vertical);
     virtual bool HorizontalFlex() const;
     virtual bool VerticalFlex() const;
 
@@ -425,15 +434,13 @@ namespace Lum {
       @name Control background manipulation and drawing methods
     */
     //@{
-    virtual void SetBackground(OS::Fill* background);
-    virtual OS::Fill* GetBackground() const;
+    void SetBackground(OS::Fill* background);
+    OS::Fill* GetBackground() const;
+    bool    HasBackground() const;
 
-    virtual void DrawBackground(int x, int y, size_t w, size_t h);
-    virtual void DrawParentBackground(int x, int y, size_t w, size_t h);
-    virtual void DrawBackgroundWithDrawInfo(OS::DrawInfo* draw,
-                                            int x, int y, size_t w, size_t h);
-    virtual void DrawParentBackgroundWithDrawInfo(OS::DrawInfo* draw,
-                                                  int x, int y, size_t w, size_t h);
+    virtual void PrepareForBackground(OS::DrawInfo* draw);
+    void DrawBackground(OS::DrawInfo* draw,
+                        int x, int y, size_t w, size_t h);
     virtual OS::Color GetTextColor(OS::DrawInfo *draw) const;
     //@}
 
@@ -497,6 +504,10 @@ namespace Lum {
   protected:
     Model::AdjustmentRef hAdjustment;
     Model::AdjustmentRef vAdjustment;
+    Model::ActionRef     updateDimensionsAction;
+
+  protected:
+    virtual void UpdateDimensions() = 0;
 
   public:
     Scrollable();
@@ -508,21 +519,18 @@ namespace Lum {
     Model::Adjustment* GetHAdjustment() const;
     Model::Adjustment* GetVAdjustment() const;
 
-    virtual void CalculateDimension();
+    void Layout();
+    void Draw(int x, int y, size_t w, size_t h);
+
+    void Resync(Base::Model* model, const Base::ResyncMsg& msg);
   };
 
   /**
-    Objects that have child controls.
+    Objects that have child controls (only for tagging purposes, does not implement any additional
+    features).
   */
   class LUMAPI Group : public Object
   {
-  protected:
-    virtual void PreDrawChild(Object* child);
-    virtual void PostDrawChild(Object* child);
-
-  public:
-    void Draw(int x, int y, size_t w, size_t h);
-    void Hide();
   };
 
   /**
@@ -538,7 +546,7 @@ namespace Lum {
 
     bool VisitChildren(Visitor &visitor, bool onlyVisible);
 
-    virtual void Add(Object* object);
+    List* Add(Object* object);
   };
 
   /**
@@ -562,8 +570,6 @@ namespace Lum {
     void CalcSize();
 
     void Layout();
-    void Draw(int x, int y, size_t w, size_t h);
-    void Hide();
   };
 
   /**
@@ -587,8 +593,6 @@ namespace Lum {
     void CalcSize();
 
     void Layout();
-    void Draw(int x, int y, size_t w, size_t h);
-    void Hide();
   };
 }
 
