@@ -79,6 +79,13 @@ namespace Lum {
 
     bool HandleKeyEvent(const OS::KeyEvent& event);
     bool HandleMouseEvent(const OS::MouseEvent& event);
+
+    static Slider* Create(bool horizontalFlex=false, bool verticalFlex=false);
+    static Slider* Create(Base::Model* model,
+                          bool horizontalFlex=false, bool verticalFlex=false);
+    static Slider* Create(Base::Model* model,
+                          bool useScale,
+                          bool horizontalFlex=false, bool verticalFlex=false);
   };
 
   class LUMAPI VSlider : public Slider
@@ -96,6 +103,13 @@ namespace Lum {
 
     bool HandleKeyEvent(const OS::KeyEvent& event);
     bool HandleMouseEvent(const OS::MouseEvent& event);
+
+    static Slider* Create(bool horizontalFlex=false, bool verticalFlex=false);
+    static Slider* Create(Base::Model* model,
+                          bool horizontalFlex=false, bool verticalFlex=false);
+    static Slider* Create(Base::Model* model,
+                          bool useScale,
+                          bool horizontalFlex=false, bool verticalFlex=false);
   };
 }
 
