@@ -70,13 +70,13 @@ namespace Lum {
   class LUMAPI ImageState : public State
   {
   protected:
-    OS::Theme::ObjectType object;
+    OS::ImageRef image;
 
   protected:
     void DrawContent(int x, int y, size_t w, size_t h);
 
   public:
-    ImageState(OS::Theme::ObjectType objectType, bool active=true);
+    ImageState(OS::Image* image, bool active=true);
 
     void CalcSize();
   };
