@@ -98,7 +98,7 @@ namespace Lum {
       void Resync(Base::Model* model, const Base::ResyncMsg& msg);
     };
 
-    class Controller : public ::Lum::Controller
+    class Controller : public Lum::Controller
     {
     public:
       Controller();
@@ -224,12 +224,16 @@ namespace Lum {
 
     void SetCursorPos(size_t pos);
 
-    bool Drop(const Base::DnDType& type,
-              Base::DnDData& data,
-              Base::DnDObject::DnDAction action);
+    bool CanExecuteAction(const Base::Datatype& type,
+                          Base::DataExchangeObject::ExchangeAction action) const;
+    bool ExecuteAction(const Base::Datatype& type,
+                       Base::Data& data,
+                       Base::DataExchangeObject::ExchangeAction action);
+    bool PostprocessAction(Base::DataExchangeObject::ExchangeAction action);
 
     void CalcSize();
 
+    void PrepareForBackground(OS::DrawInfo* draw);
     void Draw(int x, int y, size_t w, size_t h);
     void RecatchedFocus();
     void CatchedFocus();
@@ -241,6 +245,11 @@ namespace Lum {
     bool HandleKeyEvent(const OS::KeyEvent& event);
 
     void Resync(Base::Model* model, const Base::ResyncMsg& msg);
+
+    static String* Create(bool horizontalFlex=false, bool verticalFlex=false);
+    static String* Create(Base::Model* model, bool horizontalFlex=false, bool verticalFlex=false);
+    static String* Create(Base::Model* model, size_t widthInChars,
+                          bool horizontalFlex=false, bool verticalFlex=false);
   };
 
   class LUMAPI InputHelperNumber : public String::InputHelper
