@@ -79,10 +79,8 @@ namespace Lum {
     Model::TableRef       model;                //! The table model
     Model::HeaderRef      header;               //! The header model
     Model::SelectionRef   selection;            //! The selection mode
-    Model::ActionRef      selectionAction;      //! Action to be triggered onselection change
     Model::ActionRef      doubleClickAction;    //! Action to be triggered on double click
     Model::ActionRef      mouseSelectionAction;
-    Model::ActionRef      updateScrollerAction;
 
     bool                  autoFitColumns;
     bool                  autoHSize;
@@ -107,14 +105,16 @@ namespace Lum {
     OS::FontRef           font;
 
   private:
+    void UpdateDimensions();
+
     bool GetCell(size_t mx, size_t my, size_t& x, size_t& y) const;
     void RecalcTableWidth();
     void FitColumn(size_t column);
-    void UpdateScroller();
     void CalculateRowHeight();
     void FitColumns();
     bool GetLineBounds(size_t line, int& y, size_t& height) const;
 
+    void FillWithBackground(OS::DrawInfo* draw, int xPos, int yPos, size_t width, size_t height);
     void DrawCell(OS::DrawInfo* draw,
                   int xPos, int yPos, size_t width, size_t height,
                   int x, int y,
@@ -138,7 +138,6 @@ namespace Lum {
     bool SetModel(Base::Model* model);
     void SetHeader(Model::Header *header);
     void SetSelection(Model::Selection* selection);
-    void SetSelectionAction(Model::Action* action);
     void SetMouseSelectionAction(Model::Action* action);
     void SetDoubleClickAction(Model::Action* action);
     void SetFormatProvider(FormatProvider* formatProvider);
@@ -146,7 +145,6 @@ namespace Lum {
     // Getter
     Model::Table* GetModel() const;
     Model::Selection* GetSelection() const;
-    Model::Action* GetSelectionAction() const;
     Model::Action* GetDoubleClickAction() const;
     Model::Action* GetMouseSelectionAction() const;
 
@@ -173,7 +171,6 @@ namespace Lum {
     void ActionPageDown();
     void ActionDoubleClick();
 
-    virtual void OnSelection();
     virtual void OnMouseSelection();
 
     bool HandleMouseEvent(const OS::MouseEvent& event);
