@@ -54,6 +54,9 @@ namespace Lum {
     void Layout();
 
     void Resync(Base::Model* model, const Base::ResyncMsg& msg);
+
+    static View* Create(bool horizontalFlex=false, bool verticalFlex=false);
+    static View* Create(Scrollable* object, bool horizontalFlex=false, bool verticalFlex=false);
   };
 }
 
