@@ -23,7 +23,6 @@
 
 #include <Lum/Label.h>
 #include <Lum/Panel.h>
-#include <Lum/Space.h>
 #include <Lum/Tab.h>
 #include <Lum/Text.h>
 
@@ -47,42 +46,38 @@ namespace Lum {
 
   void About::CalcSize()
   {
-    Label *label;
     Panel *vPanel,*topPanel;
     Tab   *tab;
     Text  *text;
 
-    tab=new Tab();
+    tab=Tab::Create(true,true);
 
-    vPanel=new VPanel();
+    vPanel=VPanel::Create(true,true);
 
     text=new Text(info.GetProgram()+L" "+info.GetVersion(),
                   OS::Font::normal,
                   Text::centered,
-                  OS::display->GetFont(OS::Display::fontTypeProportional,
-                                       OS::Display::fontScaleCaption1));
+                  OS::display->GetFont(OS::Display::fontScaleCaption1));
     text->SetFlex(true,false);
     vPanel->Add(text);
 
-    vPanel->Add(new VSpace());
+    vPanel->AddSpace();
 
     text=new Text(info.GetDescription(),
                   OS::Font::italic|OS::Font::bold,Text::centered);
     text->SetFlex(true,false);
     vPanel->Add(text);
 
-    vPanel->Add(new VSpace());
+    vPanel->AddSpace();
 
-    label=new Label();
-    label->SetFlex(true,false);
-    label->AddLabel(_l(L"ABOUT_AUTHOR",L"Author:"),new Text(info.GetAuthor()));
-    label->AddLabel(_l(L"ABOUT_CONTACT",L"Contact:"),new Text(info.GetContact()));
-    label->AddLabel(_l(L"ABOUT_COPYRIGHT",L"Copyright:"),new Text(info.GetCopyright()));
-    label->AddLabel(_l(L"ABOUT_LICENCE",L"License:"),new Text(info.GetLicense()));
-    vPanel->Add(label);
+    vPanel->Add(Label::Create(true,false)
+                ->AddLabel(_l(L"ABOUT_AUTHOR",L"Author:"),new Text(info.GetAuthor()))
+                ->AddLabel(_l(L"ABOUT_CONTACT",L"Contact:"),new Text(info.GetContact()))
+                ->AddLabel(_l(L"ABOUT_COPYRIGHT",L"Copyright:"),new Text(info.GetCopyright()))
+                ->AddLabel(_l(L"ABOUT_LICENCE",L"License:"),new Text(info.GetLicense())));
     tab->Add(_l(L"ABOUT_TAB_ABOUT",L"About"),vPanel);
 
-    vPanel=new VPanel();
+    vPanel=VPanel::Create();
 
     text=new Text();
     text->SetAlignment(Text::centered);
@@ -94,7 +89,7 @@ namespace Lum {
     vPanel->Add(text);
     tab->Add(_l(L"ABOUT_TAB_ILLUMINATION",L"Illumination"),vPanel);
 
-    topPanel=new VPanel();
+    topPanel=VPanel::Create(true,true);
     topPanel->Add(tab);
 
     container=topPanel;
