@@ -1018,6 +1018,11 @@ namespace Lum {
       }
     }
 
+    long operator-(const SystemTime& a, const SystemTime& b)
+    {
+      return a.time.tv_sec-b.time.tv_sec;
+    }
+
     Timer::Timer()
     : time(0,0),start(0,0)
     {
