@@ -824,10 +824,13 @@ namespace Lum {
 
         entry.name=data.cFileName;
         entry.isDir=data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY;
-        entry.isLink=false;
+        entry.isSoftLink=false;
+        entry.isHardLink=false;
         entry.isHidden=data.dwFileAttributes & FILE_ATTRIBUTE_HIDDEN;
 
         entry.size=data.nFileSizeLow;
+        entry.blockSize=0;
+        entry.linkId=0;
         entry.timeModification.SetTime(data.ftLastWriteTime);
 
         entry.user=0;
@@ -847,15 +850,17 @@ namespace Lum {
     class UnixDirScanner : public DirScanner
     {
     private:
-      bool   initialized;
-      DIR    *dir;
-      Status status;
+      bool        initialized;
+      DIR         *dir;
+      Status      status;
+      std::string frontPath;
 
     public:
       UnixDirScanner(const Path& path)
        : DirScanner(path),
          initialized(false),
-         dir(NULL)
+         dir(NULL),
+         frontPath(WStringToString(path.GetPath()))
       {
         // no code
       }
@@ -868,77 +873,97 @@ namespace Lum {
         }
         else if (!initialized) {
           initialized=true;
-          dir=opendir(Lum::Base::WStringToString(path.GetPath()).c_str());
+          dir=opendir(WStringToString(path.GetPath()).c_str());
 
           if (dir==NULL) {
             this->status.SetToCurrentErrno();
+            status=this->status;
             return false;
           }
         }
 
-        struct dirent *entr;
-        struct stat   info;
+        while (true) {
+          struct dirent *entr;
+          struct stat   info;
+          std::string   subPath(frontPath);
 
-        entr=readdir(dir);
+          entr=readdir(dir);
 
-        if (entr==NULL) {
-          // TODO: Error case?
-          closedir(dir);
-          return false;
-        }
+          if (entr==NULL) {
+            // TODO: Error case?
+            closedir(dir);
+            dir=NULL;
+            return false;
+          }
 
-        if (stat(entr->d_name,&info)!=0) {
-          closedir(dir);
-          return false;
-        }
+          subPath.append(1,'/');
+          subPath.append(entr->d_name);
 
-        entry.name=Base::StringToWString(entr->d_name);
-        entry.isDir=S_ISDIR(info.st_mode);
-        if (lstat(entr->d_name,&info)==0) {
-          entry.isLink=S_ISLNK(info.st_mode);
-        }
-        else {
-          entry.isLink=false;
-        }
-        entry.isHidden=entry.name.length()>0 && entry.name[0]==L'.';
+          if (lstat(subPath.c_str(),&info)==-1) {
+            // We skip this entry
+            continue;
+          }
 
-        entry.size=info.st_size;
-        entry.timeModification.SetTime(info.st_mtime);
+          entry.name=Base::StringToWString(entr->d_name);
+          entry.isSoftLink=S_ISLNK(info.st_mode);
+          entry.isHardLink=info.st_nlink!=1;
 
-        entry.user=0;
-        if (info.st_mode & S_IRUSR) {
-          entry.user|=rightRead;
-        }
-        if (info.st_mode & S_IWUSR) {
-          entry.user|=rightWrite;
-        }
-        if (info.st_mode & S_IXUSR) {
-          entry.user|=rightExecute;
-        }
+          if (entry.isHardLink) {
+            entry.linkId=info.st_ino;
+          }
+          else {
+            entry.linkId=0;
+          }
 
-        entry.group=0;
-        if (info.st_mode & S_IRGRP) {
-          entry.group|=rightRead;
-        }
-        if (info.st_mode & S_IWGRP) {
-          entry.group|=rightWrite;
-        }
-        if (info.st_mode & S_IXGRP) {
-          entry.group|=rightExecute;
-        }
+          if (entry.isSoftLink) {
+            if (stat(entr->d_name,&info)==-1) {
+              // We skip this entry
+              continue;
+            }
+          }
 
-        entry.other=0;
-        if (info.st_mode & S_IROTH) {
-          entry.other|=rightRead;
-        }
-        if (info.st_mode & S_IWOTH) {
-          entry.other|=rightWrite;
-        }
-        if (info.st_mode & S_IXOTH) {
-          entry.other|=rightExecute;
-        }
+          entry.isDir=S_ISDIR(info.st_mode);
+          entry.isHidden=entry.name.length()>0 && entry.name[0]==L'.';
 
-        return true;
+          entry.size=info.st_size;
+          entry.blockSize=info.st_blocks*512;
+          entry.timeModification.SetTime(info.st_mtime);
+
+          entry.user=0;
+          if (info.st_mode & S_IRUSR) {
+            entry.user|=rightRead;
+          }
+          if (info.st_mode & S_IWUSR) {
+            entry.user|=rightWrite;
+          }
+          if (info.st_mode & S_IXUSR) {
+            entry.user|=rightExecute;
+          }
+
+          entry.group=0;
+          if (info.st_mode & S_IRGRP) {
+            entry.group|=rightRead;
+          }
+          if (info.st_mode & S_IWGRP) {
+            entry.group|=rightWrite;
+          }
+          if (info.st_mode & S_IXGRP) {
+            entry.group|=rightExecute;
+          }
+
+          entry.other=0;
+          if (info.st_mode & S_IROTH) {
+            entry.other|=rightRead;
+          }
+          if (info.st_mode & S_IWOTH) {
+            entry.other|=rightWrite;
+          }
+          if (info.st_mode & S_IXOTH) {
+            entry.other|=rightExecute;
+          }
+
+          return true;
+        }
       }
     };
 #endif
