@@ -19,6 +19,8 @@
 
 #include <Lum/Blender.h>
 
+#include <Lum/OS/Theme.h>
+
 #include <Lum/Button.h>
 #include <Lum/Image.h>
 
@@ -53,13 +55,17 @@ namespace Lum {
   {
     assert(main!=NULL);
 
-    Image *image;
-
-    image=new Image();
-    image->SetImage(OS::display->GetImage(OS::Display::closeImageIndex));
     close->SetParent(this);
-    close->SetType(Button::typeImage);
-    close->SetLabel(image);
+
+    if (OS::display->GetTheme()->GetStockImage(OS::Theme::closeStockImage)!=NULL) {
+      Image *image;
+      image=new Image();
+      image->SetImage(OS::display->GetTheme()->GetStockImage(OS::Theme::closeStockImage));
+      close->SetLabel(image);
+    }
+    else {
+      close->SetText(L"X");
+    }
 
     if (isTop) {
       close->SetModel(blender->GetCloseTopAction());
@@ -111,9 +117,11 @@ namespace Lum {
   void BlenderEntry::Layout()
   {
     close->Move(x,y);
+    close->Layout();
     main->Resize(width,
                  height-close->GetOHeight()-OS::display->GetSpaceVertical(OS::Display::spaceInterObject));
     main->Move(x,y+close->GetOHeight()+OS::display->GetSpaceVertical(OS::Display::spaceInterObject));
+    main->Layout();
 
     Group::Layout();
   }
@@ -206,6 +214,7 @@ namespace Lum {
   {
     assert(object!=NULL);
 
+    delete main;
     object->SetParent(this);
     main=object;
   }
@@ -381,14 +390,17 @@ namespace Lum {
       yPos+=top->GetOHeight();
       h-=top->GetOHeight();
       top->MoveResize(x,y,width,top->GetOHeight());
+      top->Layout();
     }
 
     if (bottom!=NULL) {
       h-=bottom->GetOHeight();
       bottom->MoveResize(x,y+height-bottom->GetOHeight(),width,bottom->GetOHeight());
+      bottom->Layout();
     }
 
     main->MoveResize(x,yPos,width,h);
+    main->Layout();
 
     Group::Layout();
   }
