@@ -1,3 +1,4 @@
+
 /*
   This source is part of the Illumination library
   Copyright (C) 2004  Tim Teulings
@@ -34,7 +35,8 @@ namespace Lum {
 
   Button::Button()
   : object(NULL),font(OS::display->GetFont()),type(typeNormal),
-    pulse(false),shortCut('\0'),scAssigned(false)
+    pulse(false),shortCut('\0'),scAssigned(false),
+    triggering(false)
   {
     SetBackground(OS::display->GetFill(OS::Display::buttonBackgroundFillIndex));
 
@@ -42,30 +44,6 @@ namespace Lum {
     SetRedrawOnMouseActive(true);
   }
 
-  Button::Button(Object* object)
-  : object(NULL),font(OS::display->GetFont()),type(typeNormal),
-    pulse(false),shortCut('\0'),scAssigned(false)
-  {
-    SetBackground(OS::display->GetFill(OS::Display::buttonBackgroundFillIndex));
-
-    SetCanFocus(true);
-    SetRedrawOnMouseActive(true);
-
-    SetLabel(object);
-  }
-
-  Button::Button(const std::wstring& text)
-  : object(NULL),font(OS::display->GetFont()),type(typeNormal),
-    pulse(false),shortCut('\0'),scAssigned(false)
-  {
-    SetBackground(OS::display->GetFill(OS::Display::buttonBackgroundFillIndex));
-
-    SetCanFocus(true);
-    SetRedrawOnMouseActive(true);
-
-    SetText(text);
-  }
-
   Button::~Button()
   {
     delete object;
@@ -80,35 +58,41 @@ namespace Lum {
   {
     assert(inited && object!=NULL && object->HasBaseline());
 
-    return background->topBorder+object->GetBaseline()+OS::display->GetSpaceVertical(OS::Display::spaceObjectBorder);
+    return GetTopBorder()+object->GetBaseline();
   }
 
   /**
     Set a new font to be used by the button gadget.
   */
-  void Button::SetFont(OS::Font *font)
+  Button* Button::SetFont(OS::Font *font)
   {
     this->font=font;
+
+    return this;
   }
 
   /**
     Use this method if you do not want text displayed in the button but
     want to use some other Object as label
   */
-  void Button::SetLabel(Object* label)
+  Button* Button::SetLabel(Object* label)
   {
     delete this->object;
 
     this->object=label;
     this->object->SetParent(this);
+
+    return this;
   }
 
   /**
     Assign an additional image. Some buttons type will automatically assign an image.
   */
-  void Button::SetImage(OS::ImageRef& image)
+  Button* Button::SetImage(OS::Image* image)
   {
     this->image=image;
+
+    return this;
   }
 
   /**
@@ -120,7 +104,7 @@ namespace Lum {
 
     use '_' to mark the next character as key shortcut.
   */
-  void Button::SetText(const std::wstring& string)
+  Button* Button::SetText(const std::wstring& string)
   {
     Text   *text;
     size_t pos;
@@ -144,6 +128,8 @@ namespace Lum {
     delete object;
 
     object=text;
+
+    return this;
   }
 
   /**
@@ -151,9 +137,11 @@ namespace Lum {
 
     \see Type.
   */
-  void Button::SetType(Type type)
+  Button* Button::SetType(Type type)
   {
     this->type=type;
+
+    return this;
   }
 
   /**
@@ -162,9 +150,11 @@ namespace Lum {
 
     This is usefull for buttons in a scroller or similar.
   */
-  void Button::SetPulse(bool pulse)
+  Button* Button::SetPulse(bool pulse)
   {
     this->pulse=pulse;
+
+    return this;
   }
 
   bool Button::SetModel(Base::Model* model)
@@ -176,6 +166,16 @@ namespace Lum {
     return this->model.Valid();
   }
 
+  bool Button::ShowObject() const
+  {
+    return object!=NULL && !(image.Valid() && type==typeIcon);
+  }
+
+  bool Button::ShowImage() const
+  {
+    return image.Valid() && (OS::display->GetTheme()->ShowButtonImages() || type==typeIcon);
+  }
+
   void Button::CalcSize()
   {
     /* Let the frame calculate its size */
@@ -183,23 +183,40 @@ namespace Lum {
       case typeNormal:
       SetBackground(OS::display->GetFill(OS::Display::buttonBackgroundFillIndex));
       RequestFocus();
+      if (OS::display->GetTheme()->RequestFingerFriendlyControls()) {
+        SetMinWidth(Lum::Base::Size::mm,9);
+        SetMinHeight(Lum::Base::Size::mm,9);
+      }
       break;
     case typeDefault:
       SetBackground(OS::display->GetFill(OS::Display::defaultButtonBackgroundFillIndex));
-      image=OS::display->GetImage(OS::Display::defaultImageIndex);
       RequestFocus();
+      if (OS::display->GetTheme()->RequestFingerFriendlyControls()) {
+        SetMinWidth(Lum::Base::Size::mm,9);
+        SetMinHeight(Lum::Base::Size::mm,9);
+      }
       break;
     case typeCommit:
       SetBackground(OS::display->GetFill(OS::Display::positiveButtonBackgroundFillIndex));
-      image=OS::display->GetImage(OS::Display::positiveImageIndex);
       RequestFocus();
+      if (OS::display->GetTheme()->RequestFingerFriendlyControls()) {
+        SetMinWidth(Lum::Base::Size::mm,9);
+        SetMinHeight(Lum::Base::Size::mm,9);
+      }
       break;
     case typeCancel:
       SetBackground(OS::display->GetFill(OS::Display::negativeButtonBackgroundFillIndex));
-      image=OS::display->GetImage(OS::Display::negativeImageIndex);
+      RequestFocus();
+      if (OS::display->GetTheme()->RequestFingerFriendlyControls()) {
+        SetMinWidth(Lum::Base::Size::mm,9);
+        SetMinHeight(Lum::Base::Size::mm,9);
+      }
+      break;
+    case typeIcon:
+      SetBackground(OS::display->GetFill(OS::Display::buttonBackgroundFillIndex));
       RequestFocus();
       break;
-    case typeImage:
+    case typeAdditional:
       SetBackground(OS::display->GetFill(OS::Display::buttonBackgroundFillIndex));
       RequestFocus();
       break;
@@ -218,7 +235,7 @@ namespace Lum {
     minWidth=0;
     minHeight=0;
 
-    if (object!=NULL) {
+    if (ShowObject()) {
       /*
         Now we let the image calculate its bounds and simply add its size
         to the size of the button.
@@ -230,26 +247,16 @@ namespace Lum {
       minHeight=object->GetOMinHeight();
     }
 
-    if (image.Valid() && OS::display->GetTheme()->ShowButtonImages()) {
-
-      minWidth+=image->GetWidth()+OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject);
-      width+=image->GetWidth()+OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject);
+    if (ShowImage()) {
+      minWidth+=image->GetWidth();
+      width+=image->GetWidth();
       minHeight=std::max(minHeight,image->GetHeight());
       height=std::max(height,image->GetHeight());
     }
 
-    switch (type) {
-    case typeImage:
-      minWidth+=Base::GetSize(Base::Size::softUnitP,25)*2;
-      width+=Base::GetSize(Base::Size::softUnitP,25)*2;
-      minHeight+=Base::GetSize(Base::Size::softUnitP,25)*2;
-      height+=Base::GetSize(Base::Size::softUnitP,25)*2;
-      break;
-    case typeScroll:
-      // No space around object
-      break;
-    default:
-      break;
+    if (ShowObject() && ShowImage()) {
+      minWidth+=OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject);
+      width+=OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject);
     }
 
     maxWidth=30000;
@@ -274,10 +281,12 @@ namespace Lum {
     if (event.type==OS::MouseEvent::down && PointIsIn(event) && event.button==OS::MouseEvent::button1) {
       if (!model->IsStarted()) {
         /* We change our state to pressed and redisplay ourself */
+        triggering=true;
         model->Start();
 
         if (pulse) {
           model->Trigger();
+          triggering=false;
           OS::display->AddTimer(pulseSec,pulseMSec,model);
         }
 
@@ -297,13 +306,16 @@ namespace Lum {
         if (PointIsIn(event)) {
           if (pulse) {
             model->Cancel();
+            triggering=false;
           }
           else {
             model->Finish();
+            triggering=false;
           }
         }
         else {
           model->Cancel();
+          triggering=false;
         }
       }
 
@@ -315,12 +327,14 @@ namespace Lum {
     else if (event.type==OS::MouseEvent::move && event.IsGrabed()) {
       if (PointIsIn(event)) {
         if (!model->IsStarted()) {
+          triggering=true;
           model->Start();
         }
       }
       else {
         if (model->IsStarted()) {
           model->Cancel();
+          triggering=false;
         }
       }
 
@@ -334,6 +348,7 @@ namespace Lum {
   {
     if (event.type==OS::KeyEvent::down) {
       if (event.key==OS::keySpace && model->IsInactive()) {
+        triggering=true;
         model->Start();
 
         return true;
@@ -343,9 +358,11 @@ namespace Lum {
       if (event.key==OS::keySpace) {
         if (event.qualifier==0 && model->IsStarted()) {
           model->Finish();
+          triggering=false;
         }
         else if (event.qualifier!=0 && model->IsStarted()) {
           model->Cancel();
+          triggering=false;
         }
       }
     }
@@ -369,35 +386,44 @@ namespace Lum {
     width=this->width;
     height=this->height;
 
-    switch (type) {
-    case typeImage:
-      x+=Base::GetSize(Base::Size::softUnitP,25);
-      y+=Base::GetSize(Base::Size::softUnitP,25);
-      width-=2*Base::GetSize(Base::Size::softUnitP,25);
-      height-=2*Base::GetSize(Base::Size::softUnitP,25);
-      break;
-    case typeScroll:
-      break;
-    default:
-      break;
-    }
-
-    if (image.Valid() && OS::display->GetTheme()->ShowButtonImages()) {
+    if (ShowImage()) {
       imageX=x;
       imageY=y+(height-image->GetHeight())/2;
-      x+=image->GetWidth()+OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject);
-      width-=image->GetWidth()+OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject);
+      x+=image->GetWidth();
+      width-=image->GetWidth();
     }
 
-    if (object!=NULL) {
+    if (ShowObject()) {
+      if (ShowImage()) {
+        x+=OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject);
+        width-=OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject);
+      }
       object->Resize(width,height);
       object->Move(x+(width-object->GetOWidth()) / 2,
                  y+(height-object->GetOHeight()) / 2);
+      object->Layout();
     }
 
     Control::Layout();
   }
 
+  void Button::PrepareForBackground(OS::DrawInfo* draw)
+  {
+    if (model.Valid() && model->IsEnabled()) {
+      if (IsMouseActive() && !model->IsStarted()) {
+        draw->activated=true;
+      }
+      if (triggering && model->IsStarted()) {
+        draw->selected=true;
+      }
+    }
+    else {
+      draw->disabled=true;
+    }
+
+    draw->focused=HasFocus();
+  }
+
   void Button::Draw(int x, int y, size_t w, size_t h)
   {
     Control::Draw(x,y,w,h); /* We must call Draw of our superclass */
@@ -416,7 +442,7 @@ namespace Lum {
       if (IsMouseActive() && !model->IsStarted()) {
         draw->activated=true;
       }
-      if (model->IsStarted()) {
+      if (triggering && model->IsStarted()) {
         draw->selected=true;
       }
     }
@@ -426,15 +452,13 @@ namespace Lum {
 
     draw->focused=HasFocus();
 
-    DrawBackground(x,y,w,h);
-
     // Draw object
-    if (object!=NULL) {
+    if (ShowObject()) {
       object->Draw(x,y,w,h);
     }
 
     // Draw optional image
-    if (image.Valid() && OS::display->GetTheme()->ShowButtonImages()) {
+    if (ShowImage()) {
       image->Draw(draw,imageX,imageY,image->GetWidth(),image->GetHeight());
     }
 
@@ -503,50 +527,137 @@ namespace Lum {
     Control::Resync(model,msg);
   }
 
-  Button* Button::CreateOk(Model::Action* action)
+  Button* Button::Create(Object* object,
+                         Base::Model* action,
+                         bool horizontalFlex, bool verticalFlex)
   {
     Button *button;
 
     button=new Button();
-    button->SetType(Button::typeCommit);
-    button->SetText(_ld(dlgButtonOk));
+    button->SetFlex(horizontalFlex,verticalFlex);
     button->SetModel(action);
+    button->SetLabel(object);
 
     return button;
   }
 
-  Button* Button::CreateCancel(Model::Action* action)
+  Button* Button::Create(const std::wstring& text,
+                         bool horizontalFlex, bool verticalFlex)
   {
     Button *button;
 
     button=new Button();
-    button->SetType(Button::typeCancel);
-    button->SetText(_ld(dlgButtonCancel));
-    button->SetModel(action);
+    button->SetFlex(horizontalFlex,verticalFlex);
+    button->SetText(text);
 
     return button;
   }
 
-  Button* Button::CreateClose(Model::Action* action)
+  Button* Button::Create(const std::wstring& text,
+                         Base::Model* action,
+                         bool horizontalFlex, bool verticalFlex)
   {
     Button *button;
 
     button=new Button();
-    button->SetType(Button::typeCommit);
-    button->SetText(_ld(dlgButtonClose));
+    button->SetFlex(horizontalFlex,verticalFlex);
     button->SetModel(action);
+    button->SetText(text);
 
     return button;
   }
 
-  Button* Button::CreateHelp(Model::Action* action)
+  Button* Button::Create(const std::wstring& text,
+                         OS::Image* image,
+                         Base::Model* action,
+                         bool horizontalFlex, bool verticalFlex)
   {
     Button *button;
 
     button=new Button();
-    button->SetText(_ld(dlgButtonHelp));
+    button->SetFlex(horizontalFlex,verticalFlex);
     button->SetModel(action);
+    button->SetText(text);
+    button->SetImage(image);
 
     return button;
   }
+
+  Button* Button::Create(const std::wstring& text,
+                         Base::Model* action,
+                         Type type,
+                         bool horizontalFlex, bool verticalFlex)
+  {
+    Button *button;
+
+    button=new Button();
+    button->SetFlex(horizontalFlex,verticalFlex);
+    button->SetModel(action);
+    button->SetText(text);
+    button->SetType(type);
+
+    return button;
+  }
+
+  Button* Button::Create(const std::wstring& text,
+                         OS::Image* image,
+                         Base::Model* action,
+                         Type type,
+                         bool horizontalFlex, bool verticalFlex)
+  {
+    Button *button;
+
+    button=new Button();
+    button->SetFlex(horizontalFlex,verticalFlex);
+    button->SetModel(action);
+    button->SetText(text);
+    button->SetImage(image);
+    button->SetType(type);
+
+    return button;
+  }
+
+  Button* Button::CreateOk(Model::Action* action, bool horizontalFlex, bool verticalFlex)
+  {
+    return Create(_ld(dlgButtonOk),
+                  OS::display->GetTheme()->GetStockImage(OS::Theme::positiveStockImage),
+                  action,
+                  Button::typeCommit,
+                  horizontalFlex,verticalFlex);
+  }
+
+  Button* Button::CreateCancel(Model::Action* action, bool horizontalFlex, bool verticalFlex)
+  {
+    return Create(_ld(dlgButtonCancel),
+                  OS::display->GetTheme()->GetStockImage(OS::Theme::negativeStockImage),
+                  action,
+                  Button::typeCancel,
+                  horizontalFlex,verticalFlex);
+  }
+
+  Button* Button::CreateClose(Model::Action* action, bool horizontalFlex, bool verticalFlex)
+  {
+    return Create(_ld(dlgButtonClose),
+                  OS::display->GetTheme()->GetStockImage(OS::Theme::defaultStockImage),
+                  action,
+                  Button::typeDefault,
+                  horizontalFlex,verticalFlex);
+  }
+
+  Button* Button::CreateQuit(Model::Action* action, bool horizontalFlex, bool verticalFlex)
+  {
+    return Create(_ld(dlgButtonQuit),
+                  OS::display->GetTheme()->GetStockImage(OS::Theme::defaultStockImage),
+                  action,
+                  Button::typeDefault,
+                  horizontalFlex,verticalFlex);
+  }
+
+  Button* Button::CreateHelp(Model::Action* action, bool horizontalFlex, bool verticalFlex)
+  {
+    return Create(_ld(dlgButtonHelp),
+                  OS::display->GetTheme()->GetStockImage(OS::Theme::helpStockImage),
+                  action,
+                  horizontalFlex,verticalFlex);
+  }
 }
