@@ -74,6 +74,9 @@ namespace Lum {
       value->SetBackground(NULL);
     }
 
+    headerModel=new Model::HeaderImpl();
+    headerModel->AddColumn(L"",Lum::Base::Size::pixel,0);
+
     Observe(prevAction);
     Observe(nextAction);
   }
@@ -90,14 +93,11 @@ namespace Lum {
 
   size_t Combo::GetBaseline() const
   {
-    if (value!=NULL && value->HasBaseline()) {
+    assert(value!=NULL);
+
+    if (value->HasBaseline()) {
       // We also assume that is vertically flexible
-      if (background.Valid()) {
-        return background->topBorder+value->GetBaseline()+OS::display->GetSpaceVertical(OS::Display::spaceObjectBorder);
-      }
-      else {
-        return value->GetBaseline()+OS::display->GetSpaceVertical(OS::Display::spaceObjectBorder);
-      }
+      return GetTopBorder()+value->GetBaseline();
     }
     else {
       return 0;
@@ -116,15 +116,22 @@ namespace Lum {
 
   bool Combo::VisitChildren(Visitor &visitor, bool /*onlyVisible*/)
   {
-    if (value->CanFocus()) {
-      if (!visitor.Visit(value)) {
-        return false;
-      }
+    if (!visitor.Visit(value)) {
+      return false;
     }
 
     return true;
   }
 
+  /**
+    Call this method to assign the table model that will be used to display
+    all possible values in the pop window.
+  */
+  void Combo::SetHeaderModel(Model::Header* model)
+  {
+    headerModel=model;
+  }
+
   /**
     Call this method to assign the table model that will be used to display
     all possible values in the pop window.
@@ -226,15 +233,10 @@ namespace Lum {
     combo->table->SetMinHeight(Base::Size::stdCharHeight,1);
     combo->table->SetMaxWidth(Base::Size::workVRel,40);
     combo->table->SetMaxHeight(Base::Size::workVRel,40);
+    combo->table->GetTableView()->SetAutoFitColumns(true);
+    combo->table->SetHeaderModel(combo->headerModel);
     combo->table->SetModel(combo->tableModel);
     combo->table->SetSelection(combo->selection);
-    if (combo->tableModel.Valid() && combo->tableModel->GetColumns()==1) {
-      Model::HeaderRef headerModel=new Model::HeaderImpl();
-      headerModel->AddColumn(L"",Lum::Base::Size::pixel,0);
-
-      combo->table->SetHeaderModel(headerModel);
-      combo->table->GetTableView()->SetAutoFitColumns(true);
-    }
     combo->table->GetTableView()->SetAutoHSize(true);
     combo->table->GetTableView()->SetAutoVSize(true);
     RegisterCommitShortcut(combo->table->GetTableView(),
@@ -360,48 +362,37 @@ namespace Lum {
       }
     }
 
+    value->Layout();
+
     Control::Layout();
   }
 
+  void Combo::PrepareForBackground(OS::DrawInfo* draw)
+  {
+    draw->activated=IsMouseActive();
+    draw->focused=HasFocus();
+    draw->disabled=!(GetModel()!=NULL) || GetModel()->IsNull() || !GetModel()->IsEnabled();
+  }
+
   void Combo::Draw(int x, int y, size_t w, size_t h)
   {
     OS::DrawInfo *draw=GetDrawInfo();
 
-    if (IsMouseActive()) {
-      draw->activated=true;
-    }
-
-    draw->focused=HasFocus();
-
     Control::Draw(x,y,w,h);
 
-    draw->activated=false;
-    draw->focused=false;
-    draw->disabled=!(GetModel()!=NULL) || GetModel()->IsNull() || !GetModel()->IsEnabled();
-
     if (!OIntersect(x,y,w,h)) {
       return;
     }
 
     /* --- */
 
+    draw->activated=IsMouseActive();
+    draw->focused=HasFocus();
+    draw->disabled=!(GetModel()!=NULL) || GetModel()->IsNull() || !GetModel()->IsEnabled();
+
     if (editable) {
       OS::ImageRef image=OS::display->GetImage(OS::Display::comboEditButtonImageIndex);
 
-      value->Draw(x,y,w,h);
-
-      if (IsMouseActive()) {
-        draw->activated=true;
-      }
-
-      draw->focused=HasFocus();
-
-      draw->PushClipBegin(x,y,w,h);
-      draw->SubClipRegion(value->GetOX(),value->GetOY(),value->GetOWidth(),value->GetOHeight());
-      draw->PushClipEnd();
-      DrawBackground(x,y,w,h);
-      draw->PopClip();
-
       if (imageRight) {
         int    bx,by;
         size_t bw,bh;
@@ -427,20 +418,6 @@ namespace Lum {
       OS::ImageRef image=OS::display->GetImage(OS::Display::comboImageIndex);
       OS::ImageRef divider=OS::display->GetImage(OS::Display::comboDividerImageIndex);
 
-      if (IsMouseActive()) {
-        draw->activated=true;
-      }
-
-      draw->focused=HasFocus();
-
-      draw->PushClipBegin(x,y,w,h);
-      draw->SubClipRegion(value->GetOX(),value->GetOY(),value->GetOWidth(),value->GetOHeight());
-      draw->PushClipEnd();
-      DrawBackground(x,y,w,h);
-      draw->PopClip();
-
-      value->Draw(x,y,w,h);
-
       if (imageRight) {
         divider->Draw(draw,
                       this->x+width-
@@ -575,7 +552,7 @@ namespace Lum {
       size_t       x=1;
       std::wstring res;
 
-      while (res.empty() && x<=tableModel->GetColumns()) {
+      while (res.empty() && x<=headerModel->GetColumns()) {
         res=tableModel->GetString(x,row);
         x++;
       }
@@ -634,7 +611,7 @@ namespace Lum {
       size_t       x=1;
       std::wstring res;
 
-      while (res.empty() && x<=tableModel->GetColumns()) {
+      while (res.empty() && x<=headerModel->GetColumns()) {
         res=tableModel->GetString(x,row);
         x++;
       }
@@ -664,15 +641,22 @@ namespace Lum {
       std::wstring res;
 
       if (!this->model->IsNull()) {
-        selection->SelectCell(1,this->model->Get());
+        if (this->model->Get()<=tableModel->GetRows()) {
+          selection->SelectCell(1,this->model->Get());
 
-        while (res.empty() && x<=tableModel->GetColumns()) {
-          res=tableModel->GetString(x,this->model->Get());
-          x++;
+          while (res.empty() && x<=headerModel->GetColumns()) {
+            res=tableModel->GetString(x,this->model->Get());
+            x++;
+          }
+
+          dynamic_cast<Text*>(value)->SetText(res);
+        }
+        else {
+          this->model->SetNull();
+          dynamic_cast<Text*>(value)->SetText(L"");
         }
       }
 
-      dynamic_cast<Text*>(value)->SetText(res);
     }
     else if (model==this->model) {
       Redraw();
@@ -681,6 +665,18 @@ namespace Lum {
     Combo::Resync(model,msg);
   }
 
+  IndexCombo* IndexCombo::Create(Lum::Model::SizeT* index,
+                                 Lum::Model::Table* table,
+                                 bool horizontalFlex, bool verticalFlex)
+  {
+    IndexCombo *c=new IndexCombo();
+
+    c->SetFlex(horizontalFlex,verticalFlex);
+    c->SetModel(index);
+    c->SetTableModel(table);
+
+    return c;
+  }
 
   TextEditCombo::TextEditCombo()
   : Combo(new String,true),
@@ -723,7 +719,7 @@ namespace Lum {
       std::wstring res;
 
       x=1;
-      while (res.empty() && x<=tableModel->GetColumns()) {
+      while (res.empty() && x<=headerModel->GetColumns()) {
         res=tableModel->GetString(x,row);
         x++;
       }
