@@ -44,26 +44,14 @@ namespace Lum {
     Observe(nextMonth);
     Observe(nextYear);
 
-    prevYearButton=new Button();
-    prevYearButton->SetFlex(true,true);
-    prevYearButton->SetText(L"<<");
+    prevYearButton=Button::Create(L"<<",prevYear,true,true);
     prevYearButton->SetPulse(true);
-    prevYearButton->SetModel(prevYear);
-    prevMonthButton=new Button();
-    prevMonthButton->SetFlex(true,true);
-    prevMonthButton->SetText(L"<");
+    prevMonthButton=Button::Create(L"<",prevMonth,true,true);
     prevMonthButton->SetPulse(true);
-    prevMonthButton->SetModel(prevMonth);
-    nextMonthButton=new Button();
-    nextMonthButton->SetFlex(true,true);
-    nextMonthButton->SetText(L">");
+    nextMonthButton=Button::Create(L">",nextMonth,true,true);
     nextMonthButton->SetPulse(true);
-    nextMonthButton->SetModel(nextMonth);
-    nextYearButton=new Button();
-    nextYearButton->SetFlex(true,true);
-    nextYearButton->SetText(L">>");
+    nextYearButton=Button::Create(L">>",nextYear,true,true);
     nextYearButton->SetPulse(true);
-    nextYearButton->SetModel(nextYear);
 
     current=new Text(L"",OS::Font::normal,Text::centered);
     current->SetFlex(true,true);
@@ -283,27 +271,20 @@ namespace Lum {
 
         if (text!=(size_t)-1) {
           if (text==cS) {
+            draw->PushForeground(OS::Display::fillColor);
+            draw->FillRectangle(texts[text]->GetOX(),texts[text]->GetOY(),
+                                texts[text]->GetOWidth(),texts[text]->GetOHeight());
             draw->selected=true;
           }
           texts[text]->Draw(x,y,w,h);
-          draw->selected=false;
-        }
-        else {
-          DrawBackground(this->x+X*cw,this->y+(Y+1)*ch,cw,ch);
+
+          if (text==cS) {
+            draw->selected=false;
+            draw->PopForeground();
+          }
         }
       }
     }
-
-    /* TODO: Fill right, bottom border */
-    if (width>7*cw) {
-      DrawBackground(this->x+7*cw,this->y,width-7*cw,8*ch);
-    }
-    if (height>8*ch) {
-      DrawBackground(this->x,this->y+8*ch,7*cw,height-8*ch);
-    }
-    if (width>7*cw && height>8*ch) {
-      DrawBackground(this->x+7*cw,this->y+8*ch,width-7*cw,height-8*ch);
-    }
   }
 
   bool DatePicker::HandleMouseEvent(const OS::MouseEvent& event)
