@@ -106,10 +106,25 @@ namespace Lum {
 
   void Dialog::CalcSize()
   {
-    OS::Window  *window=GetWindow();
+    OS::Window *window=GetWindow();
 
     assert(main!=NULL);
 
+    if (GetWindow()->GetType()==OS::Window::typeDialog) {
+      bool hflex=main->HorizontalFlex();
+      bool vflex=main->VerticalFlex();
+
+      if (OS::display->GetTheme()->SubdialogsAlwaysFlexHoriz()) {
+        hflex=true;
+      }
+
+      if (OS::display->GetTheme()->SubdialogsAlwaysFlexVert()) {
+        vflex=true;
+      }
+
+      main->SetFlex(hflex,vflex);
+    }
+
     group=new WindowGroup();
     if (GetWindow()->GetType()==OS::Window::typeMain && !GetWindow()->IsFullScreen()) {
       group->SetBackground(OS::display->GetFill(OS::Display::dialogWindowBackgroundFillIndex));
@@ -179,6 +194,10 @@ namespace Lum {
     }
     else if (model==GetWindow()->GetMenuAction() && GetWindow()->GetMenuAction()->IsFinished()) {
       if (OS::display->GetTheme()->GetMenuType()==OS::Theme::menuTypeActionPopup && menuPopup!=NULL) {
+        int xOff, yOff;
+
+        OS::display->GetTheme()->GetMenuTypeActionPopupOffset(xOff,yOff);
+        menuPopup->SetPos(GetWindow()->GetX()+xOff,GetWindow()->GetY()+yOff);
         /* ignore */ menuPopup->Open();
       }
     }
