@@ -24,7 +24,6 @@
 #include <Lum/About.h>
 #include <Lum/ButtonRow.h>
 #include <Lum/Panel.h>
-#include <Lum/Space.h>
 
 namespace Lum {
   namespace Dlg {
@@ -40,27 +39,21 @@ namespace Lum {
       // no code
     }
 
-    void About::PreInit()
+    Lum::Object* About::GetContent()
     {
       Lum::About  *about;
-      ButtonRow   *row;
-      Panel       *vPanel;
 
-      vPanel=new VPanel();
 
       about=new Lum::About();
+      about->SetFlex(true,true);
       about->SetAppInfo(info);
-      vPanel->Add(about);
 
-      vPanel->Add(new VSpace());
+      return about;
+    }
 
-      row=ButtonRow::CreateOk(GetClosedAction());
-      row->SetFlex(true,false);
-      vPanel->Add(row);
-
-      SetMain(vPanel);
-
-      Dialog::PreInit();
+    void About::GetActions(std::vector<Lum::Dlg::ActionInfo>& actions)
+    {
+      Lum::Dlg::ActionDialog::CreateActionInfosClose(actions,GetClosedAction());
     }
 
     void About::SetAppInfo(const AppInfo& info)
