@@ -52,9 +52,6 @@ namespace Lum {
       }
 
       SetRedrawOnMouseActive(true);
-
-      this->label->SetFlex(true,true);
-      this->label->SetParent(this);
     }
 
     ~HeaderEntry()
@@ -72,6 +69,8 @@ namespace Lum {
 
     void CalcSize()
     {
+      label->SetFlex(true,true);
+      label->SetParent(this);
       label->CalcSize();
 
       minWidth=1;
@@ -89,31 +88,28 @@ namespace Lum {
                         y,
                         width-2*OS::display->GetSpaceHorizontal(OS::Display::spaceObjectBorder),
                         height);
+      label->Layout();
 
       Object::Layout();
     }
 
+    void PrepareForBackground(OS::DrawInfo* draw)
+    {
+      draw->activated=IsMouseActive();
+    }
+
     void Draw(int x, int y, size_t w, size_t h)
     {
-      OS::DrawInfo *draw=GetDrawInfo();
-
-      draw->activated=IsMouseActive();
 
       Object::Draw(x,y,w,h);
 
       if (!OIntersect(x,y,w,h)) {
-        draw->activated=false;
         return;
       }
 
-      /* --- */
+      OS::DrawInfo *draw=GetDrawInfo();
 
-      draw->PushClipBegin(x,y,w,h);
-      draw->SubClipRegion(label->GetOX(),label->GetOY(),
-                          label->GetOWidth(),label->GetOHeight());
-      draw->PushClipEnd();
-      DrawBackground(x,y,w,h);
-      draw->PopClip();
+      draw->activated=IsMouseActive();
 
       label->Draw(x,y,w,h);
 
@@ -132,10 +128,6 @@ namespace Lum {
     for (size_t x=0; x<objects.size(); x++) {
       delete objects[x];
     }
-
-    if (adjustment.Valid()) {
-      Forget(adjustment->GetTopModel());
-    }
   }
 
   bool Header::VisitChildren(Visitor &visitor, bool /*onlyVisible*/)
@@ -192,55 +184,23 @@ namespace Lum {
     Control::CalcSize();
   }
 
-  void Header::DrawHeader(OS::DrawInfo* draw, size_t startColumn)
+  void Header::Layout()
   {
-    if (model.Valid()) {
-      int    xPos;
-      size_t columnsWidth=0;
+    int xPos;
 
-      if (adjustment.Valid()) {
-        xPos=x-adjustment->GetTop()+1;
-      }
-      else {
-        xPos=x;
-      }
-
-      draw->PushClip(GetOX(),GetOY(),GetOWidth(),GetOHeight());
-
-      for (size_t count=0; count<objects.size(); count++) {
-        if (count>=startColumn &&
-            !(xPos>=x+(int)width) &&
-            !(xPos+(int)model->GetColumnWidth(count)-1<x)) {
-
-          if (model->IsSortColumn(count)) {
-            draw->selected=true;
-          }
-
-          objects[count]->MoveResize(xPos,y,model->GetColumnWidth(count),height);
-          objects[count]->Draw(x,y,width,height);
-
-          draw->selected=false;
-        }
-
-        xPos+=model->GetColumnWidth(count);
-        columnsWidth+=model->GetColumnWidth(count);
-      }
-
-      if (xPos<x+(int)width) {
-        OS::display->GetFill(OS::Display::columnEndBackgroundFillIndex)->Draw(draw,
-                                                                              xPos,y,x+width-xPos+1,height,
-                                                                              xPos,y,x+width-xPos+1,height);
-      }
-
-      draw->PopClip();
-
-      if (adjustment.Valid()) {
-        adjustment->SetDimension(width,columnsWidth);
-      }
+    if (adjustment.Valid()) {
+      xPos=x-adjustment->GetTop()+1;
     }
     else {
-      DrawBackground(x,y,width,height);
+      xPos=x;
     }
+
+    for (size_t count=0; count<objects.size(); count++) {
+      objects[count]->MoveResize(xPos,y,model->GetColumnWidth(count),height);
+      xPos+=model->GetColumnWidth(count);
+    }
+
+    Control::Layout();
   }
 
   bool Header::SetModel(Base::Model* model)
@@ -252,6 +212,16 @@ namespace Lum {
     return this->model.Valid();
   }
 
+  void Header::PreDrawChilds(OS::DrawInfo* draw)
+  {
+    draw->PushClip(GetOX(),GetOY(),GetOWidth(),GetOHeight());
+  }
+
+  void Header::PostDrawChilds(OS::DrawInfo* draw)
+  {
+    draw->PopClip();
+  }
+
   void Header::Draw(int x, int y, size_t w, size_t h)
   {
     Control::Draw(x,y,w,h);
@@ -262,7 +232,18 @@ namespace Lum {
 
     /* --- */
 
-    DrawHeader(GetDrawInfo(),0);
+    OS::DrawInfo *draw=GetDrawInfo();
+    size_t       columnsWidth=0;
+
+    PreDrawChilds(draw);
+    for (size_t count=0; count<objects.size(); count++) {
+      columnsWidth+=model->GetColumnWidth(count);
+    }
+    PostDrawChilds(draw);
+
+    if (adjustment.Valid()) {
+      adjustment->SetDimension(width,columnsWidth);
+    }
   }
 
   bool Header::HandleMouseEvent(const OS::MouseEvent& event)
@@ -307,21 +288,21 @@ namespace Lum {
 
   void Header::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
-    const Model::Header::ResizedColumn *resized;
-
-    if (!visible) {
-      return;
-    }
-
     if (model==this->model) {
-      if (IsVisible() && (resized=dynamic_cast<const Model::Header::ResizedColumn*>(&msg))!=NULL) {
-        DrawHeader(GetDrawInfo(),resized->column);
+      const Model::Header::ResizedColumn *resized;
+
+      if ((resized=dynamic_cast<const Model::Header::ResizedColumn*>(&msg))!=NULL) {
+        // TODO: Just do a Redraw over the to be redrawn area...
+        SetRelayout();
+        Redraw();
       }
       else {
+        SetRelayout();
         Redraw();
       }
     }
     else if (adjustment.Valid() && model==adjustment->GetTopModel()) {
+      SetRelayout();
       Redraw();
     }
   }
