@@ -26,15 +26,13 @@
 namespace Lum {
 
   Knob::Knob()
-  : knob(new Image),offset(0),corr(0),
+  : knob(new Image()),offset(0),corr(0),
     adjustment(NULL),
     incAction(new Model::Action),
     decAction(new Model::Action),
     vert(true),
     selected(false)
   {
-    SetBackground(OS::display->GetFill(OS::Display::backgroundFillIndex));
-
     SetCanFocus(true);
     SetRedrawOnMouseActive(true);
 
@@ -175,6 +173,7 @@ namespace Lum {
       if (offset>=0) {
         HandleMouseMovement(event.x,event.y);
       }
+
       if (selected) {
         selected=false;
         Redraw();
@@ -191,23 +190,18 @@ namespace Lum {
   bool Knob::HandleKeyEvent(const OS::KeyEvent& event)
   {
     if (event.type==OS::KeyEvent::down) {
-      if (vert && event.key==OS::keyUp) {
-        adjustment->DecTop();
+      if ((vert && event.key==OS::keyUp) ||
+          (!vert && event.key==OS::keyLeft)) {
+        decAction->Trigger();
+        return true;
       }
-      else if (vert && event.key==OS::keyDown) {
-        adjustment->IncTop();
+      else if ((vert && event.key==OS::keyDown) ||
+               (!vert && event.key==OS::keyRight)) {
+        incAction->Trigger();
+        return true;
       }
-      else if (!vert && event.key==OS::keyLeft) {
-        adjustment->DecTop();
-      }
-      else if (!vert && event.key==OS::keyRight) {
-        adjustment->IncTop();
-      }
-      else {
-        return false;
-      }
-      return true;
     }
+
     return false;
   }
 
@@ -221,65 +215,67 @@ namespace Lum {
 
     /* --- */
 
-    DrawBackground(x,y,w,h);
-
-    if (adjustment.Valid() && adjustment->IsValid() && adjustment->GetVisible()!=adjustment->GetTotal()) {
-      size_t       kSize,bSize;
-      int          kStart;
-      OS::DrawInfo *draw=GetDrawInfo();
-
-      if (IsMouseActive()) {
-        draw->activated=true;
-      }
-
-      if (vert) {
-        bSize=height;
-      }
-      else {
-        bSize=width;
-      }
-
-      if (adjustment->GetTotal()+corr==0) {
-        kSize=bSize;
-        kStart=0;
-      }
-      else {
-        kSize=Base::RoundDiv((bSize*adjustment->GetVisible()),adjustment->GetTotal()+corr);
-        kStart=Base::RoundDiv(bSize*(adjustment->GetTop()+corr-1),adjustment->GetTotal()+corr);
-      }
-
-      if (vert) {
-        if (kSize<knob->GetOMinWidth()) {
-          kSize=knob->GetOMinWidth();
-        }
-      }
-      else {
-        if (kSize<knob->GetOMinHeight()) {
-          kSize=knob->GetOMinHeight();
-        }
-      }
-
-      if (kSize>bSize) {
-        kSize=bSize;
-      }
-
-      if (kStart+kSize>bSize) {
-        kStart=bSize-kSize;
-      }
-
-      if (vert) {
-        knob->MoveResize(this->x,this->y+kStart,width,kSize);
-      }
-      else {
-        knob->MoveResize(this->x+kStart,this->y,kSize,height);
-      }
-
-      draw->selected=selected;
-      knob->Draw(knob->GetOX(),knob->GetOY(),knob->GetOWidth(),knob->GetOHeight());
-      draw->selected=false;
-
-      draw->activated=false;
+    if (!adjustment.Valid() ||
+        !adjustment->IsValid() ||
+        adjustment->GetVisible()==adjustment->GetTotal()) {
+      return;
     }
+
+    size_t       kSize,bSize;
+    int          kStart;
+    OS::DrawInfo *draw=GetDrawInfo();
+
+    if (IsMouseActive()) {
+      draw->activated=true;
+    }
+
+    if (vert) {
+      bSize=height;
+    }
+    else {
+      bSize=width;
+    }
+
+    if (adjustment->GetTotal()+corr==0) {
+      kSize=bSize;
+      kStart=0;
+    }
+    else {
+      kSize=Base::RoundDiv((bSize*adjustment->GetVisible()),adjustment->GetTotal()+corr);
+      kStart=Base::RoundDiv(bSize*(adjustment->GetTop()+corr-1),adjustment->GetTotal()+corr);
+    }
+
+    if (vert) {
+      if (kSize<knob->GetOMinWidth()) {
+        kSize=knob->GetOMinWidth();
+      }
+    }
+    else {
+      if (kSize<knob->GetOMinHeight()) {
+        kSize=knob->GetOMinHeight();
+      }
+    }
+
+    if (kSize>bSize) {
+      kSize=bSize;
+    }
+
+    if (kStart+kSize>bSize) {
+      kStart=bSize-kSize;
+    }
+
+    if (vert) {
+      knob->MoveResize(this->x,this->y+kStart,width,kSize);
+    }
+    else {
+      knob->MoveResize(this->x+kStart,this->y,kSize,height);
+    }
+
+    draw->selected=selected;
+    knob->Draw(knob->GetOX(),knob->GetOY(),knob->GetOWidth(),knob->GetOHeight());
+    draw->selected=false;
+
+    draw->activated=false;
   }
 
   void Knob::Hide()
@@ -314,7 +310,6 @@ namespace Lum {
     }
     else if (model==adjustment && visible && adjustment.Valid() && adjustment->IsEnabled()) {
       Redraw();
-      //Draw(this->x,this->y,this->width,this->height);
     }
   }
 }
