@@ -24,6 +24,7 @@
 #include <Lum/Base/Util.h>
 
 #include <Lum/OS/Driver.h>
+#include <Lum/OS/Theme.h>
 
 #include <Lum/Dialog.h>
 #include <Lum/Text.h>
@@ -729,25 +730,25 @@ namespace Lum {
     return false;
   }
 
+  void PullDownMenu::PrepareForBackground(OS::DrawInfo* draw)
+  {
+    draw->activated=IsMouseActive();
+  }
+
   void PullDownMenu::Draw(int x, int y, size_t w, size_t h)
   {
-    OS::DrawInfo *draw=GetDrawInfo();
-
-    draw->activated=IsMouseActive();
-
     Object::Draw(x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
-      draw->activated=false;
       return;
     }
 
     /* ---- */
 
-    DrawBackground(x,y,w,h);
+    OS::DrawInfo *draw=GetDrawInfo();
 
+    draw->activated=IsMouseActive();
     label->Draw(x,y,w,h);
-
     draw->activated=false;
   }
 
@@ -844,8 +845,15 @@ namespace Lum {
     }
   }
 
-  void MenuEntry::CalcLabelSize()
+  void MenuEntry::CalcSize()
   {
+    if (image.Valid()) {
+      w1=image->GetWidth();
+    }
+    else {
+      w1=0;
+    }
+
     if (label!=NULL) {
       if (!label->IsInited()) {
         label->SetParent(this);
@@ -855,10 +863,7 @@ namespace Lum {
       w2=label->GetOWidth();
       height=std::max(height,label->GetOHeight()+2*OS::display->GetSpaceVertical(OS::Display::spaceObjectBorder));
     }
-  }
 
-  void MenuEntry::CalcShortcutSize()
-  {
     if (shortcut!=NULL) {
       if (!shortcut->IsInited()) {
         shortcut->SetParent(this);
@@ -871,6 +876,17 @@ namespace Lum {
     else {
       w3=0;
     }
+
+    if (subImage.Valid()) {
+      w4=subImage->GetWidth();
+    }
+    else {
+      w4=0;
+    }
+
+    width=w1+w2+w3+w4;
+
+    Control::CalcSize();
   }
 
   void MenuEntry::DrawImage(OS::DrawInfo* draw)
@@ -1001,16 +1017,10 @@ namespace Lum {
     }
   }
 
-  void ActionItem::CalcSize()
+  void ActionItem::PrepareForBackground(OS::DrawInfo* draw)
   {
-    assert(label!=NULL);
-
-    width=w1+w2+w3+w4;
-
-    CalcLabelSize();
-    CalcShortcutSize();
-
-    MenuEntry::CalcSize();
+    draw->disabled=(!action.Valid() || !action->IsEnabled());
+    draw->selected=menuWindow->selected==this;
   }
 
   void ActionItem::Draw(int x, int y, size_t w, size_t h)
@@ -1025,14 +1035,8 @@ namespace Lum {
 
     OS::DrawInfo *draw=GetDrawInfo();
 
-    if (!action.Valid() || !action->IsEnabled()) {
-      draw->disabled=true;
-    }
-    if (menuWindow->selected==this) {
-      draw->selected=true;
-    }
-
-    DrawBackground(x,y,w,h);
+    draw->disabled=(!action.Valid() || !action->IsEnabled());
+    draw->selected=menuWindow->selected==this;
 
     DrawImage(draw);
     DrawLabel(draw);
@@ -1070,8 +1074,6 @@ namespace Lum {
     OS::ImageRef divider=OS::display->GetImage(OS::Display::menuDividerImageIndex);
     OS::DrawInfo *draw=GetDrawInfo();
 
-    DrawBackground(x,y,w,h);
-
     divider->Draw(draw,
                   this->x,this->y+(height-divider->GetHeight())/2,
                   this->width,divider->GetHeight());
@@ -1083,6 +1085,7 @@ namespace Lum {
   : subMenu(NULL)
   {
     selectable=true;
+    subImage=OS::display->GetImage(OS::Display::menuSubImageIndex);
   }
 
   SubMenu::~SubMenu()
@@ -1115,17 +1118,16 @@ namespace Lum {
 
   void SubMenu::CalcSize()
   {
-    CalcLabelSize();
-
-    subImage=OS::display->GetImage(OS::Display::menuSubImageIndex);
-    w4=subImage->GetWidth();
-
-    width=w1+w2+w3+w4;
     height=std::max(height,subImage->GetHeight()+2*OS::display->GetSpaceVertical(OS::Display::spaceObjectBorder));
 
     MenuEntry::CalcSize();
   }
 
+  void SubMenu::PrepareForBackground(OS::DrawInfo* draw)
+  {
+    draw->selected=menuWindow->selected==this;
+  }
+
   void SubMenu::Draw(int x, int y, size_t w, size_t h)
   {
     MenuEntry::Draw(x,y,w,h);
@@ -1142,8 +1144,6 @@ namespace Lum {
       draw->selected=true;
     }
 
-    DrawBackground(x,y,w,h);
-
     if (menuWindow->selected==this) {
       if (!subMenu->IsOpen() && subMenu->Open()) {
         menuWindow->child=subMenu;
@@ -1193,16 +1193,10 @@ namespace Lum {
     }
   }
 
-  void BoolItem::CalcSize()
+  void BoolItem::PrepareForBackground(OS::DrawInfo* draw)
   {
-    assert(label!=NULL);
-
-    CalcLabelSize();
-    CalcShortcutSize();
-
-    width=w1+w2+w3+w4;
-
-    MenuEntry::CalcSize();
+    draw->disabled=(!model.Valid() || !model->IsEnabled());
+    draw->selected=menuWindow->selected==this;
   }
 
   void BoolItem::Draw(int x, int y, size_t w, size_t h)
@@ -1217,15 +1211,7 @@ namespace Lum {
 
     OS::DrawInfo *draw=GetDrawInfo();
 
-    if (!model.Valid() || !model->IsEnabled()) {
-      draw->disabled=true;
-    }
-
-    /* Background */
-
-    draw->selected=(menuWindow->selected==this);
-    DrawBackground(x,y,w,h);
-    draw->selected=false;
+    draw->disabled=(!model.Valid() || !model->IsEnabled());
 
     /* Image */
 
@@ -1276,16 +1262,10 @@ namespace Lum {
     }
   }
 
-  void RadioItem::CalcSize()
+  void RadioItem::PrepareForBackground(OS::DrawInfo* draw)
   {
-    assert(label!=NULL);
-
-    CalcLabelSize();
-    CalcShortcutSize();
-
-    width=w1+w2+w3+w4;
-
-    MenuEntry::CalcSize();
+    draw->disabled=(!model.Valid() || !model->IsEnabled());
+    draw->selected=menuWindow->selected==this;
   }
 
   void RadioItem::Draw(int x, int y, size_t w, size_t h)
@@ -1304,12 +1284,6 @@ namespace Lum {
       draw->disabled=true;
     }
 
-    /* Background */
-
-    draw->selected=(menuWindow->selected==this);
-    DrawBackground(x,y,w,h);
-    draw->selected=false;
-
     /* Image */
 
     draw->selected=(model.Valid() && model->Get()==value);
@@ -1608,12 +1582,9 @@ namespace Lum {
 
   Menu::Menu()
   : window(OS::driver->CreateWindow()),parent(NULL),
-    popup(true),registered(false),reference(NULL),
+    popup(true),registered(false),reference(NULL),manualPos(false),xPos(0),yPos(0),
     parentMenu(NULL),child(NULL),selected(NULL),strip(NULL)
   {
-    //m.prefs:=prefs;   (* We set the prefs *)
-
-    window->SetType(OS::Window::typeMenu);
     window->SetMaster(this);
 
     top=new MenuBox();
@@ -1662,6 +1633,13 @@ namespace Lum {
     this->reference=reference;
   }
 
+  void Menu::SetPos(int x, int y)
+  {
+    manualPos=true;
+    xPos=x;
+    yPos=y;
+  }
+
   bool Menu::VisitChildren(Visitor &visitor, bool onlyVisible)
   {
     return top->VisitChildren(visitor,onlyVisible);
@@ -1704,23 +1682,7 @@ namespace Lum {
   */
   MenuStrip* Menu::GetStrip()
   {
-    Menu *current;
-
-    current=this;
-    while (current!=NULL) {
-      if (current->strip!=NULL) {
-        return current->strip;
-      }
-
-      if (current->parentMenu!=NULL) {
-        current=dynamic_cast<Menu*>(current->parentMenu->GetSubMenu());
-      }
-      else {
-        current=NULL;
-      }
-    }
-
-    return NULL;
+    return GetTop()->strip;
   }
 
   /**
@@ -1728,26 +1690,38 @@ namespace Lum {
   */
   Menu* Menu::GetTop()
   {
-    Menu *current;
+    Menu *menu=this;
 
-    current=this;
-    while (current!=NULL && current->parentMenu!=NULL) {
-      current=dynamic_cast<Menu*>(current->parentMenu->GetMenu());
-    }
-
-    return current;
-  }
-
-  Menu* Menu::GetMenuWithSelection()
-  {
-    Menu *menu;
-
-    menu=this;
     while (menu->parentMenu!=NULL) {
       menu=menu->parentMenu->menuWindow;
     }
 
-    while (menu!=NULL && menu->IsOpen() && menu->selected!=NULL && menu->selected->GetSubMenu()!=NULL && menu->selected->GetSubMenu()->selected!=NULL) {
+    return menu;
+  }
+
+  Menu* Menu::GetMenuWithSelection()
+  {
+    Menu *menu=GetTop();
+
+    while (menu!=NULL &&
+           menu->IsOpen() &&
+           menu->selected!=NULL &&
+           menu->selected->GetSubMenu()!=NULL &&
+           menu->selected->GetSubMenu()->selected!=NULL) {
+      menu=menu->selected->GetSubMenu();
+    }
+
+    return menu;
+  }
+
+  Menu* Menu::GetMenuWithMouseSelection()
+  {
+    Menu *menu=GetTop();
+
+    while (menu!=NULL &&
+           menu->IsOpen() &&
+           menu->selected!=NULL &&
+           !menu->CursorIsIn()) {
       menu=menu->selected->GetSubMenu();
     }
 
@@ -1761,9 +1735,20 @@ namespace Lum {
 
     CalcSize();
 
-    selected=NULL;
+    if (strip!=NULL) {
+      window->SetType(OS::Window::typeMenuDropdown);
+    }
+    else {
+      window->SetType(OS::Window::typeMenuPopup);
+    }
 
-    if (strip!=NULL && reference!=NULL) {
+
+    selected=NULL;
+    if (manualPos) {
+      x=xPos;
+      y=yPos;
+    }
+    else if (strip!=NULL && reference!=NULL) {
       help=reference->GetWindow();
       x=help->GetX();
       y=help->GetY();
@@ -1773,8 +1758,9 @@ namespace Lum {
       window->GrabKeyboard(true);
     }
     else if (parentMenu!=NULL) { /* submenu */
-      x=parentMenu->GetWindow()->GetX()+parentMenu->GetWindow()->GetWidth();
-      y=parentMenu->GetWindow()->GetY()+parentMenu->GetOY();
+      x=parentMenu->GetWindow()->GetX()+parentMenu->GetWindow()->GetWidth()+
+        OS::display->GetTheme()->GetSubMenuHorizontalOffset();
+      y=parentMenu->GetWindow()->GetY()+parentMenu->GetOY()-top->GetTopBorder();
     }
     else if (parent!=NULL) {
       OS::display->GetMousePos(rx,ry);
@@ -1830,20 +1816,19 @@ namespace Lum {
   void Menu::CloseAll()
   {
     Menu      *current;
-    MenuStrip *strip;
+    MenuStrip *strip=GetStrip();
 
     current=this;
     while (current!=NULL) {
       current->Close();
       if (current->parentMenu!=NULL) {
-        current=dynamic_cast<Menu*>(current->parentMenu->GetMenu());
+        current=current->parentMenu->menuWindow;
       }
       else {
         current=NULL;
       }
     }
 
-    strip=GetStrip();
     if (strip!=NULL) {
       strip->SetSelection(NULL);
     }
@@ -1967,9 +1952,10 @@ namespace Lum {
       }
     }
   }
+
   /**
     Select menu item with the given hotkey.
-*/
+  */
   void Menu::SelectKey(wchar_t key)
   {
     Menu *menu;
@@ -1993,6 +1979,24 @@ namespace Lum {
     }
   }
 
+  /**
+    Call OnSelection onthe current selection.
+  */
+  void Menu::ActivateSelection()
+  {
+    Menu      *menu;
+    MenuEntry *entry;
+
+    menu=GetMenuWithSelection();
+    entry=menu->selected;
+
+    if (entry->IsSelectable()) {
+      menu->EndAction();
+      entry->OnSelection();
+    }
+  }
+
+
   /* ---------------------------------- */
 
   bool Menu::HandleEvent(OS::Event* event)
@@ -2005,10 +2009,7 @@ namespace Lum {
 
     if ((mouse=dynamic_cast<OS::MouseEvent*>(event))!=NULL) {
       if (mouse->type==OS::MouseEvent::up && (mouse->button==OS::MouseEvent::button3 || mouse->button==OS::MouseEvent::button1)) {
-        menu=GetTop();
-        while (menu!=NULL && menu->IsOpen() && menu->selected!=NULL && !menu->CursorIsIn()) {
-          menu=menu->selected->GetSubMenu();
-        }
+        menu=GetMenuWithMouseSelection();
 
         if (menu!=NULL && menu->CursorIsIn()) {
           menu->Select(menu->GetSelected());
@@ -2034,10 +2035,7 @@ namespace Lum {
 
       }
       else if (mouse->type==OS::MouseEvent::down && (mouse->button==OS::MouseEvent::button3 || mouse->button==OS::MouseEvent::button1)) {
-        menu=GetTop();
-        while (menu!=NULL && menu->IsOpen() && menu->selected!=NULL && !menu->CursorIsIn()) {
-          menu=menu->selected->GetSubMenu();
-        }
+        menu=GetMenuWithMouseSelection();
 
         if (menu==NULL || !menu->CursorIsIn()) {
           EndAction();
@@ -2049,10 +2047,7 @@ namespace Lum {
           and then walk down the menu selection tree until we
           find a menu the mouse is currently over.
         */
-        menu=GetTop();
-        while (menu!=NULL && menu->IsOpen() && menu->selected!=NULL && !menu->CursorIsIn()) {
-          menu=menu->selected->GetSubMenu();
-        }
+        menu=GetMenuWithMouseSelection();
 
         if (menu!=NULL) { /* We found a submenu */
           if (menu->CursorIsIn()) {
@@ -2098,11 +2093,7 @@ namespace Lum {
         EndAction();
         break;
       case OS::keyReturn:
-        if (selected!=NULL) {
-          MenuEntry *tmp=selected; // EndAction my close the window and destroy variable "selected"
-          EndAction();
-          tmp->OnSelection();
-        }
+        ActivateSelection();
         break;
       case OS::keyUp:
         SelectUp();
@@ -2149,6 +2140,8 @@ namespace Lum {
       const OS::Window::RedrawMsg* redrawMsg=dynamic_cast<const OS::Window::RedrawMsg*>(&msg);
 
       top->Move(0,0);
+      top->Layout();
+      top->DrawBackground(window->GetDrawInfo(),redrawMsg->x,redrawMsg->y,redrawMsg->width,redrawMsg->height);
       top->Draw(redrawMsg->x,redrawMsg->y,redrawMsg->width,redrawMsg->height);
     }
     else if (model==window->GetPreInitAction() && window->GetPreInitAction()->IsFinished()) {
