@@ -393,6 +393,11 @@ namespace Lum {
         return this->image[image].Get();
       }
 
+      Image* Theme::GetStockImage(StockImage image) const
+      {
+        return stockImage[image].Get();
+      }
+
       void Theme::PushUniqueFill(OS::DrawInfo* draw, size_t index, size_t maximum)
       {
         if (index==maximum && index%3==2) {
@@ -410,6 +415,38 @@ namespace Lum {
         draw->PopForeground();
       }
 
+      void Theme::GetMenuTypeActionPopupOffset(int& x, int& y) const
+      {
+        x=0;
+        y=0;
+      }
+
+      int Theme::GetSubMenuHorizontalOffset() const
+      {
+        return 0;
+      }
+
+      bool Theme::RequestFingerFriendlyControls() const
+      {
+        return false;
+      }
+
+      bool Theme::SubdialogsAlwaysFlexHoriz() const
+      {
+        return false;
+      }
+
+      bool Theme::SubdialogsAlwaysFlexVert() const
+      {
+        return false;
+      }
+
+      Theme::DialogActionPosition Theme::GetDialogActionPosition() const
+      {
+        //return actionPosHorizButtonRowBelow;
+        return actionPosVertButtonRowRight;
+      }
+
       /**
         Class that creates internal default versions of theming images.
       */
@@ -1112,6 +1149,117 @@ namespace Lum {
           draw->DrawLine(x+width-1,y,x,y+height-1);
           draw->PopForeground();
           break;
+        case checkbox:
+          if (draw->disabled) {
+            draw->PushForeground(OS::Display::backgroundColor);
+          }
+          else {
+            draw->PushForeground(OS::Display::whiteColor);
+          }
+          draw->FillRectangle(x,y,width,height);
+          draw->PopForeground();
+
+          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShadowColor);
+          draw->DrawLine(x,y+height-1,x,y+1);
+          draw->DrawLine(x,y,x+width-1,y);
+          draw->PopForeground();
+
+          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shadowColor);
+          draw->DrawLine(x+1,y+height-2,x+1,y+2);
+          draw->DrawLine(x+1,y+1,x+width-2,y+1);
+          draw->PopForeground();
+
+          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shineColor);
+          draw->DrawLine(x+width-1,y+1,x+width-1,y+height-1);
+          draw->DrawLine(x+width-1,y+height-1,x,y+height-1);
+          draw->PopForeground();
+
+          draw->PushForeground(OS::Display::backgroundColor); /* TODO: */
+          draw->DrawLine(x+width-2,y+2,x+width-2,y+height-2);
+          draw->DrawLine(x+width-2,y+height-2,x+1,y+height-2);
+          draw->PopForeground();
+
+          if (draw->selected) {
+            if (draw->disabled) {
+              draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShadowColor);
+            }
+            else {
+              draw->PushForeground(OS::Display::blackColor);
+            }
+            draw->PushPen(2,OS::DrawInfo::penNormal);
+            draw->DrawLine(x+3,y+3,x+width-4,y+height-4);
+            draw->DrawLine(x+width-4,y+3,x+3,y+height-4);
+            draw->PopPen();
+            draw->PopForeground();
+          }
+          break;
+        case radio:
+          if (draw->disabled) {
+            draw->PushForeground(OS::Display::backgroundColor); /* TODO: */
+          }
+          else {
+            draw->PushForeground(OS::Display::whiteColor);
+          }
+          draw->FillArc(x,y,width,height,0,360*64);
+          draw->PopForeground();
+
+          draw->PushPen(2,OS::DrawInfo::penNormal);
+          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShadowColor);
+          draw->DrawArc(x,y,width,height,45*64,180*64);
+          draw->PopForeground();
+          draw->PopPen();
+
+          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shadowColor);
+          draw->DrawArc(x+1,y+1,width-2,height-2,45*64,180*64);
+          draw->PopForeground();
+
+          draw->PushPen(2,OS::DrawInfo::penNormal);
+          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shineColor);
+          draw->DrawArc(x,y,width,height,225*64,180*64);
+          draw->PopForeground();
+          draw->PopPen();
+
+          draw->PushForeground(OS::Display::backgroundColor); /* TODO: */
+          draw->DrawArc(x+1,y+1,width-2,height-2,225*64,180*64);
+          draw->PopForeground();
+
+          if (draw->selected) {
+            if (draw->disabled) {
+              draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShadowColor);
+            }
+            else {
+              draw->PushForeground(OS::Display::blackColor);
+            }
+            draw->FillArc(x+4,y+4,width-8,height-8,0,360*64);
+            draw->PopForeground();
+          }
+          break;
+        case led:
+          if (draw->selected) {
+            if (OS::display->GetColorMode()==OS::Display::colorModeMonochrome) {
+              draw->PushForeground(OS::Display::whiteColor);
+            }
+            else {
+              draw->PushForeground(OS::Display::fillColor);
+            }
+            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
+            draw->PopForeground();
+          }
+          else {
+            draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shadowColor);
+            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
+            draw->PopForeground();
+          }
+
+          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shineColor);
+          draw->DrawArc(x+width / 4,y+height / 4,
+                        width / 2,height / 2,60*64,90*64);
+          draw->PopForeground();
+
+          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shadowColor);
+          draw->DrawArc(x,y,width-1,height-1,0*64,360*64);
+          draw->PopForeground();
+          break;
         }
       }
 
@@ -1388,6 +1536,8 @@ namespace Lum {
           colorNames[Display::whiteColor]="white";
           colorNames[Display::tabTextColor]="black";
           colorNames[Display::tabTextSelectColor]="black";
+          colorNames[Display::editTextColor]="black";
+          colorNames[Display::editTextFillColor]="white";
 
           display->AllocateNamedColor("black",shineColor);
           display->AllocateNamedColor("black",halfShineColor);
@@ -1407,6 +1557,8 @@ namespace Lum {
           colorNames[Display::whiteColor]="white";
           colorNames[Display::tabTextColor]="black";
           colorNames[Display::tabTextSelectColor]="black";
+          colorNames[Display::editTextColor]="black";
+          colorNames[Display::editTextFillColor]="white";
 
           display->AllocateNamedColor("grey95",shineColor);
           display->AllocateNamedColor("grey82",halfShineColor);
@@ -1426,6 +1578,8 @@ namespace Lum {
           colorNames[Display::whiteColor]="white";
           colorNames[Display::tabTextColor]="black";
           colorNames[Display::tabTextSelectColor]="black";
+          colorNames[Display::editTextColor]="black";
+          colorNames[Display::editTextFillColor]="white";
 
           display->AllocateNamedColor("grey95",shineColor);
           display->AllocateNamedColor("grey82",halfShineColor);
@@ -1565,6 +1719,9 @@ namespace Lum {
         image[Display::menuSubImageIndex]=new InternalImage(InternalImage::arrowRight);
         image[Display::menuCheckImageIndex]=new InternalImage(InternalImage::checkbox);
         image[Display::menuRadioImageIndex]=new InternalImage(InternalImage::radio);
+        image[Display::radioImageIndex]=new InternalImage(InternalImage::radio);
+        image[Display::checkImageIndex]=new InternalImage(InternalImage::checkbox);
+        image[Display::ledImageIndex]=new InternalImage(InternalImage::led);
         // positive
         // default
         // help
@@ -1610,171 +1767,6 @@ namespace Lum {
         return DEFAULT_FIXED_FONT_SIZE;
       }
 
-      void DefaultTheme::GetObjectSize(ObjectType object,
-                                       size_t& width,
-                                       size_t& height) const
-      {
-        int         unit;
-        OS::FontRef font;
-
-        font=OS::display->GetFont();
-
-        unit=(font->height/2>0) ? font->height/2 : 1;
-
-        switch (object) {
-        case objCheck:
-          width=2*unit;
-          height=2*unit;
-          break;
-        case objRadio:
-          width=Lum::Base::RoundUpEven(7*unit / 4);
-          height=Lum::Base::RoundUpEven(7*unit / 4);
-          break;
-        case objLED:
-          width=2*unit;
-          height=2*unit;
-          break;
-        }
-      }
-
-      bool DefaultTheme::NeedsObjectParentDraw(ObjectType object) const
-      {
-        switch (object) {
-        case objRadio:
-          return true;
-          break;
-        case objCheck:
-          return false;
-          break;
-        case objLED:
-          return true;
-          break;
-        }
-
-        assert(false);
-      }
-
-      void DefaultTheme::DrawObject(DrawInfo* draw,
-                                    ObjectType object,
-                                    int x, int y, size_t width, size_t height,
-                                    int cX, int cY, size_t cWidth, size_t cHeight)
-      {
-        switch (object) {
-        case objCheck:
-          if (draw->disabled) {
-            draw->PushForeground(OS::Display::backgroundColor);
-          }
-          else {
-            draw->PushForeground(OS::Display::whiteColor);
-          }
-          draw->FillRectangle(x,y,width,height);
-          draw->PopForeground();
-
-          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShadowColor);
-          draw->DrawLine(x,y+height-1,x,y+1);
-          draw->DrawLine(x,y,x+width-1,y);
-          draw->PopForeground();
-
-          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shadowColor);
-          draw->DrawLine(x+1,y+height-2,x+1,y+2);
-          draw->DrawLine(x+1,y+1,x+width-2,y+1);
-          draw->PopForeground();
-
-          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shineColor);
-          draw->DrawLine(x+width-1,y+1,x+width-1,y+height-1);
-          draw->DrawLine(x+width-1,y+height-1,x,y+height-1);
-          draw->PopForeground();
-
-          draw->PushForeground(OS::Display::backgroundColor); /* TODO: */
-          draw->DrawLine(x+width-2,y+2,x+width-2,y+height-2);
-          draw->DrawLine(x+width-2,y+height-2,x+1,y+height-2);
-          draw->PopForeground();
-
-          if (draw->selected) {
-            if (draw->disabled) {
-              draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShadowColor);
-            }
-            else {
-              draw->PushForeground(OS::Display::blackColor);
-            }
-            draw->PushPen(2,OS::DrawInfo::penNormal);
-            draw->DrawLine(x+3,y+3,x+width-4,y+height-4);
-            draw->DrawLine(x+width-4,y+3,x+3,y+height-4);
-            draw->PopPen();
-            draw->PopForeground();
-          }
-          break;
-
-        case objRadio:
-          if (draw->disabled) {
-            draw->PushForeground(OS::Display::backgroundColor); /* TODO: */
-          }
-          else {
-            draw->PushForeground(OS::Display::whiteColor);
-          }
-          draw->FillArc(x,y,width,height,0,360*64);
-          draw->PopForeground();
-
-          draw->PushPen(2,OS::DrawInfo::penNormal);
-          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShadowColor);
-          draw->DrawArc(x,y,width,height,45*64,180*64);
-          draw->PopForeground();
-          draw->PopPen();
-
-          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shadowColor);
-          draw->DrawArc(x+1,y+1,width-2,height-2,45*64,180*64);
-          draw->PopForeground();
-
-          draw->PushPen(2,OS::DrawInfo::penNormal);
-          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shineColor);
-          draw->DrawArc(x,y,width,height,225*64,180*64);
-          draw->PopForeground();
-          draw->PopPen();
-
-          draw->PushForeground(OS::Display::backgroundColor); /* TODO: */
-          draw->DrawArc(x+1,y+1,width-2,height-2,225*64,180*64);
-          draw->PopForeground();
-
-          if (draw->selected) {
-            if (draw->disabled) {
-              draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShadowColor);
-            }
-            else {
-              draw->PushForeground(OS::Display::blackColor);
-            }
-            draw->FillArc(x+4,y+4,width-8,height-8,0,360*64);
-            draw->PopForeground();
-          }
-          break;
-        case objLED:
-          if (draw->selected) {
-            if (OS::display->GetColorMode()==OS::Display::colorModeMonochrome) {
-              draw->PushForeground(OS::Display::whiteColor);
-            }
-            else {
-              draw->PushForeground(OS::Display::fillColor);
-            }
-            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
-            draw->PopForeground();
-          }
-          else {
-            draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shadowColor);
-            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
-            draw->PopForeground();
-          }
-
-          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shineColor);
-          draw->DrawArc(x+width / 4,y+height / 4,
-                        width / 2,height / 2,60*64,90*64);
-          draw->PopForeground();
-
-          draw->PushForeground(dynamic_cast<DefaultTheme*>(display->GetTheme())->shadowColor);
-          draw->DrawArc(x,y,width-1,height-1,0*64,360*64);
-          draw->PopForeground();
-          break;
-        }
-      }
-
       size_t DefaultTheme::GetSpaceHorizontal(Display::Space space) const
       {
         switch (space) {
@@ -1848,16 +1840,6 @@ namespace Lum {
         return OS::display->GetSpaceHorizontal(OS::Display::spaceInterObject);
       }
 
-      size_t DefaultTheme::GetEntryHorizTextOffset() const
-      {
-        return 2;
-      }
-
-      size_t DefaultTheme::GetEntryVertTextOffset() const
-      {
-        return 2;
-      }
-
       OS::Theme::ScrollKnobMode DefaultTheme::GetScrollKnobMode() const
       {
         return OS::Theme::scrollKnobModeDouble;
