@@ -163,7 +163,7 @@ namespace Lum {
 
           assert(s!=NULL);
           assert(s->fontScaled!=NULL);
-          
+
           cairo_set_scaled_font(cairo,s->fontScaled);
         }
       }
@@ -198,9 +198,8 @@ namespace Lum {
           fontStack.top().font->StringExtent(tmp,extent,fontStack.top().style);
 
           DrawLine(x,y,x+extent.width-extent.left-extent.right,y);
+          cairo_stroke(cairo);
         }
-
-        cairo_stroke(cairo);
       }
 
       void DrawInfo::DrawFillString(int x, int y, const std::wstring& text,
@@ -216,7 +215,6 @@ namespace Lum {
         cairo_move_to(cairo,x,y);
         // TODO
         cairo_show_text(cairo,b.c_str());
-        cairo_stroke(cairo);
       }
 
       void DrawInfo::PushForeground(OS::Color color)
@@ -287,6 +285,8 @@ namespace Lum {
 
       void DrawInfo::PushDash(const char* dashList, size_t len, DashMode /*mode*/)
       {
+	assert(dashList!=NULL && len>0);
+
         dashPushCount++;
 
         cairo_save(cairo);
@@ -367,7 +367,6 @@ namespace Lum {
         cairo_set_line_width(cairo,1);
         cairo_arc(cairo,x+0.5,y+0.5,1.0,0,360*(M_PI/180.0));
         cairo_fill(cairo);
-        cairo_stroke(cairo);
         cairo_restore(cairo);
       }
 
@@ -394,7 +393,6 @@ namespace Lum {
         cairo_set_source_rgb(cairo,r,g,b);
         cairo_arc(cairo,x+0.5,y+0.5,1.0,0,360*(M_PI/180.0));
         cairo_fill(cairo);
-        cairo_stroke(cairo);
         cairo_restore(cairo);
       }
 
@@ -411,7 +409,9 @@ namespace Lum {
           return;
         }
 
-        cairo_rectangle(cairo,x+0.5,y+0.5,width-1,height-1);
+        double penSize=cairo_get_line_width(cairo);
+
+        cairo_rectangle(cairo,x+penSize/2,y+penSize/2,width-penSize,height-penSize);
         cairo_stroke(cairo);
       }
 
@@ -423,7 +423,6 @@ namespace Lum {
 
         cairo_rectangle(cairo,x,y,width,height);
         cairo_fill(cairo);
-        cairo_stroke(cairo);
       }
 
       void DrawInfo::DrawArc(int x, int y, int width, int height, int angle1, int angle2)
@@ -456,7 +455,6 @@ namespace Lum {
         }
         cairo_close_path(cairo);
         cairo_fill(cairo);
-        cairo_stroke(cairo);
         cairo_restore(cairo);
       }
 
@@ -467,9 +465,8 @@ namespace Lum {
         for (size_t x=1; x< count; x++) {
           cairo_line_to(cairo,points[x].x,points[x].y);
         }
-        cairo_fill(cairo);
         cairo_close_path(cairo);
-        cairo_stroke(cairo);
+        cairo_fill(cairo);
       }
 
       void DrawInfo::CopyArea(int sX, int sY, int width, int height, int dX, int dY)
@@ -478,7 +475,6 @@ namespace Lum {
         cairo_set_source_surface(cairo,surface,dX-sX,dY-sY);
         cairo_rectangle(cairo,dX,dY,width,height);
         cairo_fill(cairo);
-        cairo_stroke(cairo);
         cairo_restore(cairo);
       }
 
@@ -501,7 +497,6 @@ namespace Lum {
         cairo_set_source_surface(cairo,draw->surface,dX-sX,dY-sY);
         cairo_rectangle(cairo,dX,dY,width,height);
         cairo_fill(cairo);
-        cairo_stroke(cairo);
         cairo_restore(cairo);
       }
 
@@ -520,7 +515,6 @@ namespace Lum {
         cairo_set_source_surface(draw->cairo,surface,dX-sX,dY-sY);
         cairo_rectangle(draw->cairo,dX,dY,width,height);
         cairo_fill(draw->cairo);
-        cairo_stroke(draw->cairo);
         cairo_restore(draw->cairo);
       }
 
