@@ -41,10 +41,14 @@ namespace Lum {
         case Display::treeExpanderImageIndex:
         case Display::hscaleKnobImageIndex:
         case Display::vscaleKnobImageIndex:
-        case Display::closeImageIndex:
         case Display::menuSubImageIndex:
+        case Display::menuCheckImageIndex:
+        case Display::menuRadioImageIndex:
         case Display::hpaneHandleImageIndex:
         case Display::vpaneHandleImageIndex:
+        case Display::radioImageIndex:
+        case Display::checkImageIndex:
+        case Display::ledImageIndex:
           width=1;
           height=1;
           break;
@@ -58,10 +62,6 @@ namespace Lum {
         case Display::topSliderImageIndex:
         case Display::bottomSliderImageIndex:
         case Display::menuDividerImageIndex:
-        case Display::positiveImageIndex:
-        case Display::negativeImageIndex:
-        case Display::defaultImageIndex:
-        case Display::helpImageIndex:
           width=0;
           height=0;
           break;
@@ -98,6 +98,7 @@ namespace Lum {
 
             switch (image) {
             case Display::menuCheckImageIndex:
+            case Display::checkImageIndex:
               draw->PushForeground(Display::textColor);
               if (draw->selected) {
                 draw->DrawString(i,j,L"X");
@@ -108,6 +109,7 @@ namespace Lum {
               draw->PopForeground();
               break;
             case Display::menuRadioImageIndex:
+            case Display::radioImageIndex:
               draw->PushForeground(Display::textColor);
               if (draw->selected) {
                 draw->DrawString(i,j,L"*");
@@ -117,6 +119,14 @@ namespace Lum {
               }
               draw->PopForeground();
               break;
+            case Display::ledImageIndex:
+              if (draw->selected) {
+                draw->DrawString(i,j,L"*");
+              }
+              else {
+                draw->DrawString(i,j,L"o");
+              }
+              break;
             case Display::hknobImageIndex:
             case Display::vknobImageIndex:
             case Display::hpaneHandleImageIndex:
@@ -197,15 +207,83 @@ namespace Lum {
             case Display::topSliderImageIndex:
               draw->DrawString(i,j,L" ");
               break;
-            case Display::closeImageIndex:
-              draw->DrawString(i,j,L"X");
-              break;
             case Display::comboDividerImageIndex:
             case Display::menuDividerImageIndex:
-            case Display::positiveImageIndex:
-            case Display::negativeImageIndex:
-            case Display::defaultImageIndex:
-            case Display::helpImageIndex:
+              break;
+            }
+          }
+        }
+
+        draw->PopForeground();
+        draw->PopFont();
+      }
+
+      TextStockImage::TextStockImage(Theme::StockImage image)
+        : image(image)
+      {
+        switch (image) {
+        case Theme::closeStockImage:
+        case Theme::addStockImage:
+        case Theme::removeStockImage:
+        case Theme::editStockImage:
+          width=1;
+          height=1;
+          break;
+        case Theme::positiveStockImage:
+        case Theme::negativeStockImage:
+        case Theme::defaultStockImage:
+        case Theme::helpStockImage:
+          width=0;
+          height=0;
+          break;
+        }
+      }
+
+      size_t TextStockImage::GetWidth() const
+      {
+        return width;
+      }
+
+      size_t TextStockImage::GetHeight() const
+      {
+        return height;
+      }
+
+      bool TextStockImage::GetAlpha() const
+      {
+        return true;
+      }
+
+      unsigned long TextStockImage::GetDrawCap() const
+      {
+        return 0;//drawsDisabled|drawsFocused|drawsActivated;
+      }
+
+      void TextStockImage::Draw(OS::DrawInfo *draw, int x, int y, size_t w, size_t h)
+      {
+        draw->PushFont(OS::display->GetFont());
+        draw->PushForeground(Display::textColor);
+
+        for (int i=x; i<x+w; ++i) {
+          for (int j=y; j<y+h; ++j) {
+
+            switch (image) {
+            case Theme::closeStockImage:
+              draw->DrawString(i,j,L"X");
+              break;
+            case Theme::addStockImage:
+              draw->DrawString(i,j,L"+");
+              break;
+            case Theme::removeStockImage:
+              draw->DrawString(i,j,L"-");
+              break;
+            case Theme::editStockImage:
+              draw->DrawString(i,j,L"v");
+              break;
+            case Theme::positiveStockImage:
+            case Theme::negativeStockImage:
+            case Theme::defaultStockImage:
+            case Theme::helpStockImage:
               break;
             }
           }
@@ -676,6 +754,8 @@ namespace Lum {
           colorNames[Display::whiteColor]="white";
           colorNames[Display::tabTextColor]="black";
           colorNames[Display::tabTextSelectColor]="black";
+          colorNames[Display::editTextColor]="black";
+          colorNames[Display::editTextFillColor]="white";
         }
         else {
           colorNames[Display::backgroundColor]="white";
@@ -690,6 +770,8 @@ namespace Lum {
           colorNames[Display::whiteColor]="white";
           colorNames[Display::tabTextColor]="black";
           colorNames[Display::tabTextSelectColor]="black";
+          colorNames[Display::editTextColor]="black";
+          colorNames[Display::editTextFillColor]="white";
         }
 
         for (size_t i=0; i<Display::colorCount; i++) {
@@ -791,9 +873,14 @@ namespace Lum {
         image[Display::menuSubImageIndex]=new TextImage(Display::menuSubImageIndex);
         image[Display::menuCheckImageIndex]=new TextImage(Display::menuCheckImageIndex);
         image[Display::menuRadioImageIndex]=new TextImage(Display::menuRadioImageIndex);
-        image[Display::closeImageIndex]=new TextImage(Display::closeImageIndex);
         image[Display::hpaneHandleImageIndex]=new TextImage(Display::hpaneHandleImageIndex);
         image[Display::vpaneHandleImageIndex]=new TextImage(Display::vpaneHandleImageIndex);
+        image[Display::radioImageIndex]=new TextImage(Display::radioImageIndex);
+        image[Display::checkImageIndex]=new TextImage(Display::checkImageIndex);
+        image[Display::ledImageIndex]=new TextImage(Display::ledImageIndex);
+
+        stockImage[closeStockImage]=new TextStockImage(closeStockImage);
+
       }
 
       Theme::~Theme()
@@ -833,80 +920,6 @@ namespace Lum {
         return 1;
       }
 
-      void Theme::GetObjectSize(ObjectType object,
-                                size_t& width,
-                                size_t& height) const
-      {
-        switch (object) {
-        case objRadio:
-        case objCheck:
-        case objLED:
-          width=1;
-          height=1;
-          break;
-        }
-      }
-
-      bool Theme::NeedsObjectParentDraw(ObjectType object) const
-      {
-        switch (object) {
-        case objRadio:
-        case objCheck:
-        case objLED:
-          return false;
-          break;
-        }
-
-        return false;
-      }
-
-      void Theme::DrawObject(DrawInfo* draw,
-                             ObjectType object,
-                             int x, int y, size_t width, size_t height,
-                             int cX, int cY, size_t cWidth, size_t cHeight)
-      {
-        draw->PushFont(OS::display->GetFont());
-        draw->PushForeground(Display::textColor);
-
-        for (int i=x; i<x+width; ++i) {
-          for (int j=y; j<y+height; ++j) {
-            switch (object) {
-            case objCheck:
-              draw->PushForeground(Display::textColor);
-              if (draw->selected) {
-                draw->DrawString(i,j,L"X");
-              }
-              else {
-                draw->DrawString(i,j,L"o");
-              }
-              draw->PopForeground();
-              break;
-            case objRadio:
-              draw->PushForeground(Display::textColor);
-              if (draw->selected) {
-                draw->DrawString(i,j,L"*");
-              }
-              else {
-                draw->DrawString(i,j,L"o");
-              }
-              draw->PopForeground();
-              break;
-            case objLED:
-              if (draw->selected) {
-                draw->DrawString(i,j,L"*");
-              }
-              else {
-                draw->DrawString(i,j,L"o");
-              }
-              break;
-            }
-          }
-        }
-
-        draw->PopForeground();
-        draw->PopFont();
-      }
-
       size_t Theme::GetSpaceHorizontal(Display::Space space) const
       {
         switch (space) {
@@ -980,16 +993,6 @@ namespace Lum {
         return 0;
       }
 
-      size_t Theme::GetEntryHorizTextOffset() const
-      {
-        return 0;
-      }
-
-      size_t Theme::GetEntryVertTextOffset() const
-      {
-        return 0;
-      }
-
       OS::Theme::ScrollKnobMode Theme::GetScrollKnobMode() const
       {
         return OS::Theme::scrollKnobModeDouble;
