@@ -149,6 +149,18 @@ namespace Lum {
       // no code
     }
 
+#if defined(__WIN32__) || defined(WIN32)
+    HANDLE Thread::GetHandle() const
+    {
+      return handle;
+    }
+#elif defined(ILLUMINATION_THREAD_PTHREAD)
+    pthread_t Thread::GetHandle() const
+    {
+      return handle;
+    }
+#endif
+
     Mutex::Mutex()
     {
 #if defined(__WIN32__) || defined(WIN32)
