@@ -315,7 +315,7 @@ namespace Lum {
         }
       }
 
-      bool Display::RegisterSelection(Lum::Base::DnDObject* object, OS::Window* /*window*/)
+      bool Display::RegisterSelection(Lum::Base::DataExchangeObject* object, OS::Window* /*window*/)
       {
         if (selectObject!=object) {
           if (selectObject!=NULL) {
@@ -336,12 +336,12 @@ namespace Lum {
         selectObject=NULL;
       }
 
-      Lum::Base::DnDObject* Display::GetSelectionOwner() const
+      Lum::Base::DataExchangeObject* Display::GetSelectionOwner() const
       {
         return selectObject;
       }
 
-      bool Display::QuerySelection(OS::Window* /*window*/, Lum::Base::DnDObject* /*object*/)
+      bool Display::QuerySelection(OS::Window* /*window*/, Lum::Base::DataExchangeObject* /*object*/)
       {
         return false;
         // TODO
