@@ -80,10 +80,6 @@
   #include <Lum/OS/X11/GtkTheme.h>
 #endif
 
-#if defined(HAVE_LIB_HILDON)
-  #include <Lum/OS/X11/HildonKeyboardHandler.h>
-#endif
-
 #if defined(CAIRO_HAS_XLIB_SURFACE)
   #include <Lum/OS/Cairo/Display.h>
 #endif
@@ -121,6 +117,8 @@ namespace Lum {
         "_NET_WM_WINDOW_TYPE_DESKTOP",
         "_NET_WM_WINDOW_TYPE_TOOLBAR",
         "_NET_WM_WINDOW_TYPE_MENU",
+        "_NET_WM_WINDOW_TYPE_POPUP_MENU",
+        "_NET_WM_WINDOW_TYPE_DROPDOWN_MENU",
         "_NET_WM_WINDOW_TYPE_UTILITY",
         "_NET_WM_WINDOW_TYPE_SPLASH",
         "_NET_WM_WINDOW_TYPE_DIALOG",
@@ -133,6 +131,8 @@ namespace Lum {
         "_NET_SYSTEM_TRAY_OPCODE",
         "_NET_SYSTEM_TRAY_MESSAGE_DATA",
         "_NET_WORKAREA",
+        "_NET_FRAME_EXTENTS",
+        "_NET_WM_WINDOW_OPACITY",
         "UTF8_STRING",
         "_XEMBED",
         "_XEMBED_INFO",
@@ -912,6 +912,7 @@ namespace Lum {
 
         if (visual==NULL) {
           std::cerr << "Cannot get visual!" << std::endl;
+          XCloseDisplay(display);
           return false;
         }
 
@@ -1223,9 +1224,6 @@ namespace Lum {
         }
 #endif
 
-#if defined(HAVE_LIB_HILDON)
-        keyboardHandler=new HildonKeyboardHandler();
-#endif
         return true;
       }
 
@@ -1316,7 +1314,7 @@ namespace Lum {
        }
 
 
-      bool Display::RegisterSelection(Lum::Base::DnDObject* object,
+      bool Display::RegisterSelection(Lum::Base::DataExchangeObject* object,
                                       OS::Window* window)
       {
         if (selectObject!=object) {
@@ -1343,13 +1341,13 @@ namespace Lum {
         selectObject=NULL;
       }
 
-      Lum::Base::DnDObject* Display::GetSelectionOwner() const
+      Lum::Base::DataExchangeObject* Display::GetSelectionOwner() const
       {
         return selectObject;
       }
 
       bool Display::QuerySelection(OS::Window* window,
-                                   Lum::Base::DnDObject* object)
+                                   Lum::Base::DataExchangeObject* object)
       {
         XConvertSelection(display,
                           XA_PRIMARY,
@@ -1401,7 +1399,8 @@ namespace Lum {
 
         if (!WaitTimedForX11Event(appWindow,SelectionNotify,event) ||
             event.xselection.property==None) {
-          // This is not necesarly an error! Not all application might support this!
+          // This is not necesarily an error! Not all applications support this! We fall back
+          // to XA_STRING in this case.
         }
         else {
           if (XGetWindowProperty(display,
@@ -1482,7 +1481,7 @@ namespace Lum {
           result=Lum::Base::UTF8ToWString((char*)data);
         }
         else {
-          std::cerr << "Unsoported clipboard content of type " << retType << " and format " << retFormat << "!"<< std::endl;
+          std::cerr << "Unsupported clipboard content of type " << retType << " and format " << retFormat << "!"<< std::endl;
           result=L"";
         }
 
@@ -1528,7 +1527,7 @@ namespace Lum {
         int                retFormat;
         unsigned long      itemsReturn,bytesLeft;
         unsigned char      *data;
-        Lum::Base::DnDData dndData;
+        Lum::Base::Data dndData;
 
         if (event.property==None) {
           return;
@@ -1565,20 +1564,24 @@ namespace Lum {
 
         if (event.property==atoms[selectionAtom]) {
           if (querySelectObject!=NULL) {
-            if (!querySelectObject->Drop(Lum::Base::DnDType(::Lum::Base::DnDType::text,Lum::Base::DnDType::plain),
-                                         dndData,Lum::Base::DnDObject::dndInsert)) {
+            if (!querySelectObject->ExecuteAction(Lum::Base::Datatype(Lum::Base::Datatype::text,
+                                                                      Lum::Base::Datatype::plain),
+                                                  dndData,
+                                                  Lum::Base::DataExchangeObject::actionInsert)) {
               Beep();
             }
             querySelectObject=NULL;
           }
         }
         else if (event.property==atoms[clipBufferAtom]) {
-          Lum::Base::DnDObject *queryObject;
+          Lum::Base::DataExchangeObject *queryObject;
 
           queryObject=queryClipboardObject;
           if (queryObject!=NULL) {
-            if (!queryObject->Drop(Lum::Base::DnDType(Lum::Base::DnDType::text,Lum::Base::DnDType::plain),
-                                   dndData,Lum::Base::DnDObject::dndInsert)) {
+            if (!queryObject->ExecuteAction(Lum::Base::Datatype(Lum::Base::Datatype::text,
+                                                                Lum::Base::Datatype::plain),
+                                            dndData,
+                                            Lum::Base::DataExchangeObject::actionInsert)) {
               Beep();
             }
             queryObject=NULL;
@@ -1597,11 +1600,9 @@ namespace Lum {
       */
       void Display::HandleXSelectionRequest(const XSelectionRequestEvent& event)
       {
-        Lum::Base::DnDData data;
-        XEvent             selNotify;
-        //Atom               retType;
-        //unsigned long      retFormat,itemsReturn,bytesLeft;
-        std::string        text;
+        Lum::Base::Data data;
+        XEvent          selNotify;
+        std::string     text;
 
         selNotify.xselection.type=SelectionNotify;
         selNotify.xselection.display=event.display;
@@ -1611,10 +1612,16 @@ namespace Lum {
         selNotify.xselection.property=event.property;
         selNotify.xselection.time=event.time;
 
-        if (event.target==XA_STRING || event.target==atoms[compoundTextAtom]) {
+        if (event.target==XA_STRING ||
+            event.target==atoms[compoundTextAtom] ||
+            event.target==atoms[utf8StringAtom] ||
+            event.target==atoms[targetsAtom]) {
           if (event.selection==XA_PRIMARY) {
             if (selectObject!=NULL) {
-              if (selectObject->GetDataOfType(Lum::Base::DnDType(Lum::Base::DnDType::text,Lum::Base::DnDType::plain),data)) {
+              if (selectObject->ExecuteAction(Lum::Base::Datatype(Lum::Base::Datatype::text,
+                                                                  Lum::Base::Datatype::plain),
+                                              data,
+                                              Lum::Base::DataExchangeObject::actionCopy)) {
                 text=Lum::Base::WStringToString(data.GetText());
               }
             }
@@ -1626,13 +1633,39 @@ namespace Lum {
           }
 
           if (!text.empty()) {
-            XChangeProperty(event.display,event.requestor,event.property,
-                            XA_STRING,8,
-                            PropModeReplace,(const unsigned char*)text.c_str(),
-                            text.length());
+            if (event.target==XA_STRING || event.target==atoms[compoundTextAtom]) {
+              XChangeProperty(event.display,event.requestor,event.property,
+                              XA_STRING,8,
+                              PropModeReplace,(const unsigned char*)text.c_str(),
+                              text.length());
+            }
+            else if (event.target==atoms[targetsAtom]) {
+              Atom formats[3];
+
+              formats[0]=atoms[utf8StringAtom];
+              formats[1]=atoms[compoundTextAtom];
+              formats[2]=XA_STRING;
+
+              XChangeProperty(event.display,event.requestor,event.property,
+                              XA_ATOM,sizeof(Atom)*8,
+                              PropModeReplace,(const unsigned char*)formats,
+                              sizeof(formats)/sizeof(Atom));
+            }
+            else {
+              std::string tmp=Lum::Base::WStringToUTF8(clipboard);
+              XChangeProperty(event.display,event.requestor,event.property,
+                              atoms[utf8StringAtom],8,
+                              PropModeReplace,(const unsigned char*)tmp.c_str(),
+                              tmp.length());
+            }
+          }
+          else {
+            std::cerr << "Clipboard is empty" << std::endl;
+            selNotify.xselection.property=None;
           }
         }
         else {
+          std::cerr << "Unsupported datatype " <<  event.target << " for clipboard request from other application" << std::endl;
           selNotify.xselection.property=None;
         }
 
@@ -1720,7 +1753,7 @@ namespace Lum {
       }
 
       /**
-        Waits for certain events to happen:
+        Waits for the given event to happen:
         *A X11 event occurs
         *Timer run out
 
@@ -1762,12 +1795,67 @@ namespace Lum {
           ret=select(x11Fd+1,&set,NULL,NULL,&timeout);
 
           if (ret>0) {
-            if (XCheckTypedWindowEvent(display,appWindow,eventType,&event)==True) {
+            if (XCheckTypedWindowEvent(display,window,eventType,&event)==True) {
               return true;
             }
           }
         }
+      }
 
+      /**
+        Waits for certain events to happen:
+        *A X11 event occurs
+        *Timer run out
+
+        Returns TRUE, if the wait exceeds the given timeout, else FALSE, if Wait
+        returned because an X11 event is available.
+
+        File descriptors getting available will be handled internaly. In this
+        case a notification will be send and the wait will be restarted.
+      */
+      bool Display::WaitTimedForPropertyChange(::Window window, Atom property, XEvent& event) const
+      {
+        Lum::Base::SystemTime timer,max;
+
+        XFlush(display);
+
+        timer.SetTime(3,0);
+
+        max.Add(timer);
+
+        while (true) {
+          fd_set                set;
+          Lum::Base::SystemTime now,tmp;
+          timeval               timeout;
+          int                   ret,x11Fd;
+
+          x11Fd=ConnectionNumber(display);
+
+          FD_ZERO(&set);
+          FD_SET(x11Fd,&set);
+
+          if (now>=max) {
+            std::cout << "Timeout!" << std::endl;
+            return false;
+          }
+          tmp=max;
+          tmp.Sub(now);
+
+          tmp.GetTimeval(timeout);
+
+          ret=select(x11Fd+1,&set,NULL,NULL,&timeout);
+
+          if (ret>0) {
+            std::cout << "Wakeup!" << std::endl;
+            if (XCheckTypedWindowEvent(display,window,PropertyNotify,&event)==True) {
+              std::cout << "Got event " << event.type << " " << event.xproperty.atom << std::endl;
+              if (event.type==PropertyNotify && event.xproperty.atom==property) {
+                std::cout << "Match!" << std::endl;
+                return true;
+              }
+            }
+          }
+        }
       }
 
       bool Display::GetMousePos(int& x, int& y) const
