@@ -30,6 +30,7 @@
 
 #include <X11/X.h>
 #include <X11/extensions/shape.h>
+#include <X11/Xatom.h>
 
 #include <Lum/Base/String.h>
 
@@ -221,8 +222,8 @@ namespace Lum {
 
       bool Window::HandleEvent(OS::Event* event)
       {
-        if (display->GetKeyboardHandler()!=NULL &&
-            display->GetKeyboardHandler()->HandleEvent(this,event)) {
+        if (display->GetKeyboardManager()!=NULL &&
+            display->GetKeyboardManager()->HandleEvent(this,event)) {
           return true;
         }
 
@@ -418,7 +419,7 @@ data        : POINTER TO ARRAY OF LONGINT;*/
             }
             return true;
           }
-          else if ((Atom)x11Event.xclient.data.l[0]==display->atoms[Display::netWMContextCustomAtom]) {
+          else if ((Atom)x11Event.xclient.message_type==XInternAtom(display->display,"_MB_GRAB_TRANSFER",true)) {
 #ifdef HAVE_LIB_HILDON
             TriggerMenu();
 #endif
@@ -484,23 +485,23 @@ data        : POINTER TO ARRAY OF LONGINT;*/
             draw->PushClipBegin(x11Event.xgraphicsexpose.x,x11Event.xgraphicsexpose.y,
                                 x11Event.xgraphicsexpose.width,x11Event.xgraphicsexpose.height);
 
-            x1=x11Event.xexpose.x;
-            y1=x11Event.xexpose.y;
-            x2=x11Event.xexpose.x+x11Event.xexpose.width-1;
-            y2=x11Event.xexpose.y+x11Event.xexpose.height-1;
+            x1=x11Event.xgraphicsexpose.x;
+            y1=x11Event.xgraphicsexpose.y;
+            x2=x11Event.xgraphicsexpose.x+x11Event.xgraphicsexpose.width-1;
+            y2=x11Event.xgraphicsexpose.y+x11Event.xgraphicsexpose.height-1;
 
             while (XCheckTypedWindowEvent(display->display,window,GraphicsExpose,&x11Event)==True) {
-              if (x11Event.xexpose.x<x1) {
-                x1=x11Event.xexpose.x;
+              if (x11Event.xgraphicsexpose.x<x1) {
+                x1=x11Event.xgraphicsexpose.x;
               }
-              if (x11Event.xexpose.y<y1) {
-                y1=x11Event.xexpose.y;
+              if (x11Event.xgraphicsexpose.y<y1) {
+                y1=x11Event.xgraphicsexpose.y;
               }
-              if (x11Event.xexpose.x+x11Event.xexpose.width-1>x2) {
-                x2=x11Event.xexpose.x+x11Event.xexpose.width-1;
+              if (x11Event.xgraphicsexpose.x+x11Event.xgraphicsexpose.width-1>x2) {
+                x2=x11Event.xgraphicsexpose.x+x11Event.xgraphicsexpose.width-1;
               }
-              if (x11Event.xexpose.y+x11Event.xexpose.height-1>y2) {
-                y2=x11Event.xexpose.y+x11Event.xexpose.height-1;
+              if (x11Event.xgraphicsexpose.y+x11Event.xgraphicsexpose.height-1>y2) {
+                y2=x11Event.xgraphicsexpose.y+x11Event.xgraphicsexpose.height-1;
               }
 
               draw->AddClipRegion(x11Event.xgraphicsexpose.x,x11Event.xgraphicsexpose.y,
@@ -550,15 +551,15 @@ data        : POINTER TO ARRAY OF LONGINT;*/
           }
           return true;
         case FocusIn:
-          if (display->GetKeyboardHandler()!=NULL) {
-            display->GetKeyboardHandler()->OnWindowFocusIn(this);
+          if (display->GetKeyboardManager()!=NULL) {
+            display->GetKeyboardManager()->OnWindowFocusIn(this);
           }
           OnFocusIn();
           return true;
         case FocusOut:
           OnFocusOut();
-          if (display->GetKeyboardHandler()!=NULL) {
-            display->GetKeyboardHandler()->OnWindowFocusOut(this);
+          if (display->GetKeyboardManager()!=NULL) {
+            display->GetKeyboardManager()->OnWindowFocusOut(this);
           }
           return true;
         case EnterNotify:
@@ -600,7 +601,10 @@ data        : POINTER TO ARRAY OF LONGINT;*/
           break;
 #ifdef HAVE_LIB_HILDON
         case KeyPress:
-          if (dynamic_cast<KeyEvent*>(event)->key==keyF6) {
+          if (dynamic_cast<KeyEvent*>(event)->key==keyF4) {
+            TriggerMenu();
+          }
+          else if (dynamic_cast<KeyEvent*>(event)->key==keyF6) {
             if (IsFullScreen()) {
               LeaveFullScreen();
             }
@@ -609,6 +613,9 @@ data        : POINTER TO ARRAY OF LONGINT;*/
             }
           }
 #endif
+        case PropertyNotify:
+          //std::cout << "Property " << x11Event.xproperty.atom << " has changed!" << std::endl;
+          break;
         default:
           break;
         }
@@ -682,6 +689,19 @@ data        : POINTER TO ARRAY OF LONGINT;*/
         //XUngrabKeyboard(display->display,CurrentTime);
       }
 
+/*
+      static Bool property_notify_predicate (::Display *xdisplay __attribute__((unused)),
+                                             XEvent  *event,
+                                             XPointer window_id)
+      {
+        if (event->xany.type == PropertyNotify &&
+            //event->xproperty.window==*(&::Window)window_id &&
+            event->xproperty.atom == 233)
+          return True;
+        else
+          return False;
+      }*/
+
       bool Window::Open(bool activate)
       {
         XSizeHints           *sHints;
@@ -714,11 +734,13 @@ data        : POINTER TO ARRAY OF LONGINT;*/
         attr.event_mask=KeyPressMask | KeyReleaseMask |  ExposureMask |
                         StructureNotifyMask | ButtonPressMask |  ButtonReleaseMask |
                         ButtonMotionMask | PointerMotionMask | FocusChangeMask |
-                        EnterWindowMask | LeaveWindowMask;
+                        EnterWindowMask | LeaveWindowMask | PropertyChangeMask;
 
         switch (type) {
         case typeTooltip:
         case typeMenu:
+        case typeMenuDropdown:
+        case typeMenuPopup:
         case typePopup:
         case typeSplash:
           attr.override_redirect=True;
@@ -801,14 +823,102 @@ data        : POINTER TO ARRAY OF LONGINT;*/
 
         SetTitle(GetTitle());
 
+        switch (type) {
+        case typeToolbar:
+          display->SetAtomProperty(this,
+                                   display->atoms[Display::netWMWindowTypeAtom],
+                                   display->atoms[Display::netWMWindowTypeToolbarAtom]);
+          break;
+        case typeMenu:
+          display->SetAtomProperty(this,
+                                   display->atoms[Display::netWMWindowTypeAtom],
+                                   display->atoms[Display::netWMWindowTypeMenuAtom]);
+          break;
+        case typeMenuPopup:
+          display->SetAtomProperty(this,
+                                   display->atoms[Display::netWMWindowTypeAtom],
+                                   display->atoms[Display::netWMWindowTypeMenuPopupAtom]);
+          break;
+        case typeMenuDropdown:
+          display->SetAtomProperty(this,
+                                   display->atoms[Display::netWMWindowTypeAtom],
+                                   display->atoms[Display::netWMWindowTypeMenuDropdownAtom]);
+          break;
+        case typeTooltip:
+        case typePopup:
+          display->SetAtomProperty(this,
+                                   display->atoms[Display::netWMWindowTypeAtom],
+                                   display->atoms[Display::netWMWindowTypeMenuPopupAtom]);
+          break;
+        case typeUtility:
+          display->SetAtomProperty(this,
+                                   display->atoms[Display::netWMWindowTypeAtom],
+                                   display->atoms[Display::netWMWindowTypeUtilityAtom]);
+          break;
+        case typeSplash:
+          display->SetAtomProperty(this,
+                                   display->atoms[Display::netWMWindowTypeAtom],
+                                   display->atoms[Display::netWMWindowTypeSplashAtom]);
+          break;
+        case typeDialog:
+          display->SetAtomProperty(this,
+                                   display->atoms[Display::netWMWindowTypeAtom],
+                                   display->atoms[Display::netWMWindowTypeDialogAtom]);
+          break;
+        case typeMain:
+          display->SetAtomProperty(this,
+                                   display->atoms[Display::netWMWindowTypeAtom],
+                                   display->atoms[Display::netWMWindowTypeNormalAtom]);
+          break;
+        case typeAuto:
+          assert(false);
+          break;
+        }
+
+        /*
+        if (type==typeMain && XInternAtom(display->display,"_NET_REQUEST_FRAME_EXTENTS",true)!=None) {
+          Atom           type;
+          int            format;
+          unsigned long  items;
+          unsigned long  bytes;
+          unsigned char* data;
+
+          ::XEvent event;
+
+          memset(&event,0,sizeof(event));
+          event.xclient.type=ClientMessage;
+          event.xclient.message_type=XInternAtom(display->display,"_NET_REQUEST_FRAME_EXTENTS",false);
+          event.xclient.display=display->display;
+          event.xclient.window=window;
+          event.xclient.format=32;
+          if (XSendEvent(display->display,XRootWindow(display->display,display->scrNum),
+                         false,(SubstructureRedirectMask | SubstructureNotifyMask),&event)!=0) {
+            XIfEvent(display->display,&event,property_notify_predicate,0);
+            if (XGetWindowProperty(display->display,window,
+                                   display->atoms[Display::netFrameExtents],
+                                   0,sizeof(unsigned long)*4,
+                                   false,
+                                   XA_CARDINAL,
+                                   &type,&format,&items,&bytes,&data)==Success) {
+              if (type==XA_CARDINAL && format==32 && items==4 && data!=NULL) {
+                unsigned long* frame=(unsigned long*)data;
+
+                std::cout << "Frame: " << frame[0]<< " " << frame[1] << " " << frame[2] << " " << frame[3] << std::endl;
+              }
+            }
+          }
+        }*/
+
         sHints->min_width=minWidth;
-        sHints->max_width=maxWidth;
         sHints->min_height=minHeight;
+        sHints->width=width;
+        sHints->height=height;
+        sHints->max_width=maxWidth;
         sHints->max_height=maxHeight;
-        sHints->flags=PMinSize | PMaxSize;
+        sHints->flags=PMinSize | PSize| PMaxSize;
 
         if (xManPos || yManPos) {
-          sHints->flags=sHints->flags | USPosition;
+          sHints->flags|=PPosition;
           sHints->x=x;
           sHints->y=y;
         }
@@ -869,47 +979,8 @@ data        : POINTER TO ARRAY OF LONGINT;*/
                                      getpid());
 
 #endif
-        switch (type) {
-        case typeToolbar:
-          display->SetAtomProperty(this,
-                                   display->atoms[Display::netWMWindowTypeAtom],
-                                   display->atoms[Display::netWMWindowTypeToolbarAtom]);
-          break;
-        case typeMenu:
-          display->SetAtomProperty(this,
-                                   display->atoms[Display::netWMWindowTypeAtom],
-                                   display->atoms[Display::netWMWindowTypeMenuAtom]);
-          break;
-        case typeTooltip:
-        case typePopup:
-          display->SetAtomProperty(this,
-                                   display->atoms[Display::netWMWindowTypeAtom],
-                                   display->atoms[Display::netWMWindowTypeMenuAtom]);
-          break;
-        case typeUtility:
-          display->SetAtomProperty(this,
-                                   display->atoms[Display::netWMWindowTypeAtom],
-                                   display->atoms[Display::netWMWindowTypeUtilityAtom]);
-          break;
-        case typeSplash:
-          display->SetAtomProperty(this,
-                                   display->atoms[Display::netWMWindowTypeAtom],
-                                   display->atoms[Display::netWMWindowTypeSplashAtom]);
-          break;
-        case typeDialog:
-          display->SetAtomProperty(this,
-                                   display->atoms[Display::netWMWindowTypeAtom],
-                                   display->atoms[Display::netWMWindowTypeDialogAtom]);
-          break;
-        case typeMain:
-          display->SetAtomProperty(this,
-                                   display->atoms[Display::netWMWindowTypeAtom],
-                                   display->atoms[Display::netWMWindowTypeNormalAtom]);
-          break;
-        case typeAuto:
-          assert(false);
-          break;
-        }
+
+        SetOpacity(GetOpacity());
 
         draw=driver->CreateDrawInfo(this);
 
@@ -926,16 +997,6 @@ data        : POINTER TO ARRAY OF LONGINT;*/
           GrabMouse(false);
           GrabKeyboard(false);
 
-          /*
-
-          if (parent!=NULL) {
-            XSetInputFocus(display->display,dynamic_cast<Window*>(parent)->window,RevertToParent,CurrentTime);
-          }
-          else {
-            XSetInputFocus(display->display,PointerRoot,RevertToParent,CurrentTime);
-          }
-          */
-
           if (IsShown()) {
             XUnmapWindow(display->display,window);
 
@@ -949,8 +1010,8 @@ data        : POINTER TO ARRAY OF LONGINT;*/
             draw->OnUnmap();
           }
 
-          if (display->GetKeyboardHandler()!=NULL) {
-            display->GetKeyboardHandler()->OnWindowFocusOut(this);
+          if (display->GetKeyboardManager()!=NULL) {
+            display->GetKeyboardManager()->OnWindowFocusOut(this);
           }
         }
 
@@ -1205,6 +1266,35 @@ data        : POINTER TO ARRAY OF LONGINT;*/
       {
         return fullScreen;
       }
+
+      bool Window::SetOpacity(double opacity)
+      {
+        if (opacity<0) {
+          opacity=0;
+        }
+        else if (opacity>1) {
+          opacity=1;
+        }
+
+        Lum::OS::Base::Window::SetOpacity(opacity);
+
+        if (window!=0) {
+          unsigned long value=(unsigned long)(opacity*0xffffffff);
+
+          if (value==0xffffffff) {
+            XDeleteProperty(display->display,window,display->atoms[Display::netWMWindowOpacity]);
+          }
+          else {
+            XChangeProperty(display->display,window,
+                            display->atoms[Display::netWMWindowOpacity],
+                            XA_CARDINAL, 32,
+                            PropModeReplace,
+                            (const unsigned char*)&value,1);
+          }
+        }
+
+        return true;
+      }
     }
   }
 }
