@@ -95,6 +95,7 @@ namespace Lum {
     visible(false),
     firstDraw(true),
     layouted(false),
+    clipFromParent(false),
     hasFocus(false),
     canFocus(false),
     showFocus(false),
@@ -331,100 +332,100 @@ namespace Lum {
   /**
     Set a size constraint for the object.
    */
-  Object& Object::SetConstraint(Constraint constraint,
+  Object* Object::SetConstraint(Constraint constraint,
                                 Base::Size::Mode mode,
                                 size_t value,
                                 const OS::FontRef& font)
   {
     constraints.push_back(ConstraintSize(constraint,Base::Size(mode,value,font)));
 
-    return *this;
+    return this;
   }
 
   /**
     Set the width of the object.
     "Mode" defines the interpretation of the given value.
   */
-  Object& Object::SetWidth(Base::Size::Mode mode,
+  Object* Object::SetWidth(Base::Size::Mode mode,
                            size_t value,
                            const OS::FontRef& font)
   {
     SetConstraint(constraintWidth,mode,value,font);
 
-    return *this;
+    return this;
   }
 
   /**
     Set the height of the object.
     "Mode" defines the interpretation of the given value.
   */
-  Object& Object::SetHeight(Base::Size::Mode mode,
+  Object* Object::SetHeight(Base::Size::Mode mode,
                             size_t value,
                             const OS::FontRef& font)
   {
     SetConstraint(constraintHeight,mode,value,font);
 
-    return *this;
+    return this;
   }
 
   /**
     Set the minimal width of the object.
     "Mode" defines the interpretation of the given value.
   */
-  Object& Object::SetMinWidth(Base::Size::Mode mode,
+  Object* Object::SetMinWidth(Base::Size::Mode mode,
                               size_t value,
                               const OS::FontRef& font)
   {
     SetConstraint(constraintMinWidth,mode,value,font);
 
-    return *this;
+    return this;
   }
 
   /**
     Set the minimal height of the object.
     "Mode" defines the interpretation of the given value.
   */
-  Object& Object::SetMinHeight(Base::Size::Mode mode,
+  Object* Object::SetMinHeight(Base::Size::Mode mode,
                                size_t value,
                                const OS::FontRef& font)
   {
     SetConstraint(constraintMinHeight,mode,value,font);
 
-    return *this;
+    return this;
   }
 
   /**
     Set the maximum width of the object.
     "Mode" defines the interpretation of the given value.
   */
-  Object& Object::SetMaxWidth(Base::Size::Mode mode,
+  Object* Object::SetMaxWidth(Base::Size::Mode mode,
                               size_t value,
                               const OS::FontRef& font)
   {
     SetConstraint(constraintMaxWidth,mode,value,font);
 
-    return *this;
+    return this;
   }
 
   /**
     Set the maximum height of the object.
     "Mode" defines the interpretation of the given value.
   */
-  Object& Object::SetMaxHeight(Base::Size::Mode mode,
+  Object* Object::SetMaxHeight(Base::Size::Mode mode,
                                size_t value,
                                const OS::FontRef& font)
   {
     SetConstraint(constraintMaxHeight,mode,value,font);
 
-    return *this;
+    return this;
   }
 
-  Object& Object::SetFlex(bool horizontal, bool vertical)
+  Object* Object::SetFlex(bool horizontal, bool vertical)
   {
     horizontalFlex=horizontal;
     verticalFlex=vertical;
 
-    return *this;
+    return this;
   }
 
   bool Object::HorizontalFlex() const
@@ -437,20 +438,6 @@ namespace Lum {
     return verticalFlex;
   }
 
-  /**
-    Set the background object for the object. Behaviour for complex/group
-    objects is undefined.
-  */
-  void Object::SetBackground(OS::Fill* background)
-  {
-    this->background=background;
-  }
-
-  OS::Fill* Object::GetBackground() const
-  {
-    return background.Get();
-  }
-
   /**
     Assign a label to the object. Some object may support some
     tricky operations with their label. A checkbox f.e. may delegate its
@@ -1199,54 +1186,6 @@ namespace Lum {
   */
   void Object::CalcSize()
   {
-    for(std::list<ConstraintSize>::const_iterator iter=constraints.begin();
-        iter!=constraints.end();
-        ++iter) {
-      switch (iter->constraint) {
-        case constraintMinWidth:
-          minWidth=Base::RoundRange(iter->size.GetSize(),width,maxWidth);
-          break;
-        case constraintMinHeight:
-          minHeight=Base::RoundRange(iter->size.GetSize(),height,maxHeight);
-          break;
-        default:
-          break;
-      }
-    }
-
-    for(std::list<ConstraintSize>::const_iterator iter=constraints.begin();
-        iter!=constraints.end();
-        ++iter) {
-      switch (iter->constraint) {
-        case constraintMaxWidth:
-          maxWidth=Base::RoundRange(minWidth,width,iter->size.GetSize());
-          break;
-        case constraintMaxHeight:
-          maxHeight=Base::RoundRange(minHeight,height,iter->size.GetSize());
-          break;
-        default:
-          break;
-      }
-    }
-
-    width=Base::RoundRange(width,minWidth,maxWidth);
-    height=Base::RoundRange(height,minHeight,maxHeight);
-
-    for(std::list<ConstraintSize>::const_iterator iter=constraints.begin();
-        iter!=constraints.end();
-        ++iter) {
-      switch (iter->constraint) {
-        case constraintWidth:
-          width=Base::RoundRange(minWidth,iter->size.GetSize(),maxWidth);
-          break;
-        case constraintHeight:
-          height=Base::RoundRange(minHeight,iter->size.GetSize(),maxHeight);
-          break;
-        default:
-          break;
-      }
-    }
-
     oWidth=width;
     oHeight=height;
 
@@ -1267,6 +1206,67 @@ namespace Lum {
       oMaxHeight+=background->topBorder+background->bottomBorder;
     }
 
+    for(std::list<ConstraintSize>::const_iterator iter=constraints.begin();
+        iter!=constraints.end();
+        ++iter) {
+      switch (iter->constraint) {
+        case constraintMinWidth:
+          oMinWidth=Base::RoundRange(iter->size.GetSize(),oWidth,oMaxWidth);
+          break;
+        case constraintMinHeight:
+          oMinHeight=Base::RoundRange(iter->size.GetSize(),oHeight,oMaxHeight);
+          break;
+        default:
+          break;
+      }
+    }
+
+    for(std::list<ConstraintSize>::const_iterator iter=constraints.begin();
+        iter!=constraints.end();
+        ++iter) {
+      switch (iter->constraint) {
+        case constraintMaxWidth:
+          oMaxWidth=Base::RoundRange(iter->size.GetSize(),oMinWidth,oMaxWidth);
+          break;
+        case constraintMaxHeight:
+          oMaxHeight=Base::RoundRange(iter->size.GetSize(),oMinHeight,oMaxHeight);
+          break;
+        default:
+          break;
+      }
+    }
+
+    oWidth=Base::RoundRange(oWidth,oMinWidth,oMaxWidth);
+    oHeight=Base::RoundRange(oHeight,oMinHeight,oMaxHeight);
+
+
+    for(std::list<ConstraintSize>::const_iterator iter=constraints.begin();
+        iter!=constraints.end();
+        ++iter) {
+      switch (iter->constraint) {
+        case constraintWidth:
+          oWidth=Base::RoundRange(iter->size.GetSize(),oMinWidth,oMaxWidth);
+          break;
+        case constraintHeight:
+          oHeight=Base::RoundRange(iter->size.GetSize(),oMinHeight,oMaxHeight);
+          break;
+        default:
+          break;
+      }
+    }
+
+    if (background.Valid()) {
+      width=oWidth-background->leftBorder-background->rightBorder;
+      height=oHeight-background->topBorder-background->bottomBorder;
+
+      minWidth=oMinWidth-background->leftBorder-background->rightBorder;
+      minHeight=oMinHeight-background->topBorder-background->bottomBorder;
+
+      maxWidth=oMaxWidth-background->leftBorder-background->rightBorder;
+      maxHeight=oMaxHeight-background->topBorder-background->bottomBorder;
+    }
+
+
     if (!horizontalFlex) {
       oMinWidth=oWidth;
       minWidth=width;
@@ -1389,7 +1389,24 @@ namespace Lum {
 
   void Object::Layout()
   {
+    struct LayoutVisitor : public Visitor
+    {
+      bool Visit(Lum::Object* object)
+      {
+        if (!object->IsLayouted()) {
+          object->Layout();
+        }
+
+        object->VisitChildren(*this,true);
+
+        return true;
+      }
+    };
+
+    LayoutVisitor layoutVisitor;
+
     layouted=true;
+    layoutVisitor.Visit(this);
   }
 
   /**
@@ -1541,96 +1558,179 @@ namespace Lum {
   }
 
   /**
-    This is just a convenience function. Draws the given rectangle using the
-    current background of the object.
+    Set the background object for the object. Behaviour for complex/group
+    objects is undefined.
   */
-  void Object::DrawBackground(int x, int y, size_t w, size_t h)
+  void Object::SetBackground(OS::Fill* background)
   {
-    DrawBackgroundWithDrawInfo(GetDrawInfo(),x,y,w,h);
+    this->background=background;
   }
 
   /**
-    This is just a convenience function. Draws the given rectangle using the
-    current background of the parent object.
-  */
-  void Object::DrawParentBackground(int x, int y, size_t w, size_t h)
+    Returns the fill that is responsible for drawing the background.
+   */
+  OS::Fill* Object::GetBackground() const
   {
-    DrawParentBackgroundWithDrawInfo(GetDrawInfo(),x,y,w,h);
+    return background.Get();
   }
 
   /**
-    This is just a convenience function. Draws the given rectangle using the
-    current background of the object.
-  */
-  void Object::DrawBackgroundWithDrawInfo(OS::DrawInfo* draw,
-                                          int x, int y, size_t w, size_t h)
+    Returns true if this objects has a Fill for drawing backgrounds assigned.
+   */
+  bool Object::HasBackground() const
   {
-    // We clip the drawing rectangle to the bound of the object since we cannot draw
-    // anything outside the object!
-    if (!OIntersect(x,y,w,h)) {
+    return background.Valid();
+  }
+
+
+  void Object::PrepareForBackground(OS::DrawInfo* draw)
+  {
+    // no code
+  }
+
+  void Object::DrawBackground(OS::DrawInfo* draw,
+                              int x, int y, size_t w, size_t h)
+  {
+    struct ClipVisitor : public Visitor
+    {
+      OS::DrawInfo *draw;
+      bool         cliped;
+      int          x,y;
+      size_t       w,h;
+
+      bool Visit(Object* object)
+      {
+        if (object->clipFromParent) {
+          if (!cliped) {
+            draw->PushClipBegin(x,y,w,h);
+            cliped=true;
+          }
+
+          if (!object->HasBackground() || !object->GetBackground()->transparentBorder) {
+            draw->SubClipRegion(object->GetOX(),object->GetOY(),
+                                object->GetOWidth(),object->GetOHeight());
+          }
+          else {
+            OS::FillRef background=object->GetBackground();
+
+            draw->SubClipRegion(object->GetOX()+background->leftBorder,
+                                object->GetOY()+background->topBorder,
+                                object->GetOWidth()-background->leftBorder-background->rightBorder,
+                                object->GetOHeight()-background->topBorder-background->bottomBorder);
+          }
+
+        }
+
+        object->VisitChildren(*this,true);
+
+        return true;
+      }
+    };
+
+    struct DrawVisitor : public Visitor
+    {
+      OS::DrawInfo *draw;
+      int          x,y;
+      size_t       w,h;
+
+      bool Visit(Object* object)
+      {
+        int    cx,cy;
+        size_t cw,ch;
+
+        cx=x;
+        cy=y;
+        cw=w;
+        ch=h;
+
+        // Normally everything should be layouted in the layouting phase, however
+        // not all objetc implement VisitChildren or call Layout manually for their children.
+        if (!object->IsLayouted()) {
+          object->Layout();
+        }
+
+        if (object->OIntersect(cx,cy,cw,ch)) {
+          // TODO: Now, even if the object has a background it might not be necessary to draw it,
+          // because the to be drawn area is complete covered by one or more children (recursivly)
+          // that all have their own background and no transparent border.
+          if (object->HasBackground()) {
+            ClipVisitor visitor;
+
+            visitor.draw=draw;
+            visitor.cliped=false;
+            visitor.x=cx;
+            visitor.y=cy;
+            visitor.w=cw;
+            visitor.h=ch;
+
+            object->VisitChildren(visitor,true);
+            if (visitor.cliped) {
+              draw->PushClipEnd();
+            }
+
+            object->PrepareForBackground(draw);
+
+            object->background->Draw(draw,
+                                     object->GetOX(),object->GetOY(),
+                                     object->GetOWidth(),object->GetOHeight(),
+                                     cx,cy,cw,ch);
+
+            draw->activated=false;
+            draw->selected=false;
+            draw->disabled=false;
+            draw->focused=false;
+
+            if (visitor.cliped) {
+              draw->PopClip();
+            }
+          }
+
+          object->PreDrawChilds(draw);
+          object->VisitChildren(*this,true);
+          object->PostDrawChilds(draw);
+        }
+        return true;
+      }
+    };
+
+    if (!background.Valid() || (background.Valid() && background->transparentBorder)) {
+      assert(parent!=NULL);
+
+      parent->DrawBackground(draw,x,y,w,h);
       return;
     }
 
-    Object *object=this;
+    assert(background.Valid());
 
-    while (object!=NULL && !object->background.Valid()) {
-      object=object->parent;
-    }
+    DrawVisitor visitor;
 
-    if (object!=NULL && object->background.Valid()) {
-      assert(x>=object->oX);
+    visitor.draw=draw;
+    visitor.x=x;
+    visitor.y=y;
+    visitor.w=w;
+    visitor.h=h;
 
-      if (object->background->transparentBorder) {
-        object->DrawParentBackgroundWithDrawInfo(draw,x,y,w,h);
-      }
-
-      object->background->Draw(draw,
-                               object->GetOX(),object->GetOY(),
-                               object->GetOWidth(),object->GetOHeight(),
-                               x,y,w,h);
-    }
-    else {
-      OS::display->GetFill(OS::Display::backgroundFillIndex)->Draw(draw,
-                                                                   GetOX(),GetOY(),
-                                                                   GetOWidth(),
-                                                                   GetOHeight(),
-                                                                   x,y,w,h);
-    }
+    visitor.Visit(this);
   }
 
-  /**
-    This is just a convenience function. Draws the given rectangle using the
-    current background of the parent object.
-  */
-  void Object::DrawParentBackgroundWithDrawInfo(OS::DrawInfo* draw,
-                                                int x, int y, size_t w, size_t h)
+  void Object::PreDrawChilds(OS::DrawInfo* draw)
   {
-    bool selected;
+    // no code
+  }
 
-    selected=draw->selected;
-    draw->selected=false;
+  void Object::PostDrawChilds(OS::DrawInfo* draw)
+  {
+    // no code
+  }
 
-    if (parent!=NULL) {
-      if (!parent->OIntersect(x,y,w,h)) {
-        draw->selected=selected;
-        return;
-      }
+  void Object::PreDrawChild(OS::DrawInfo* draw, Object* /*child*/)
+  {
+    // no code
+  }
 
-      parent->DrawBackgroundWithDrawInfo(draw,x,y,w,h);
-    }
-    else {
-      if (!OIntersect(x,y,w,h)) {
-        draw->selected=selected;
-        return;
-      }
-
-      OS::display->GetFill(OS::Display::backgroundFillIndex)->Draw(draw,
-                                                                   oX,oY,
-                                                                   oWidth,oHeight,
-                                                                   x,y,w,h);
-    }
-
-    draw->selected=selected;
+  void Object::PostDrawChild(OS::DrawInfo* draw, Object* /*child*/)
+  {
+    // no code
   }
 
   /**
@@ -1648,19 +1748,6 @@ namespace Lum {
   */
   void Object::Draw(int x, int y, size_t w, size_t h)
   {
-    class LayoutVisitor : public Visitor
-    {
-    public:
-      bool Visit(Object* object)
-      {
-        if (!object->IsLayouted()) {
-          object->Layout();
-          object->VisitChildren(*this,true);
-        }
-        return true;
-      }
-    };
-
     visible=true;
     firstDraw=false;
 
@@ -1670,12 +1757,114 @@ namespace Lum {
 
     /* --- */
 
-    LayoutVisitor layoutVisitor;
-
-    if (!layouted) {
-      this->Layout();
-      VisitChildren(layoutVisitor,true);
+    // Normally everything should be layouted in the layouting phase, however
+    // not all objetc implement VisitChildren or call Layout manually for their children.
+    if (!IsLayouted()) {
+      Layout();
     }
+
+    DrawChildren(GetDrawInfo(),x,y,w,h);
+  }
+
+  void Object::DrawChildren(OS::DrawInfo* draw, int x, int y, size_t w, size_t h)
+  {
+    /*
+    class AreaVisitor : public Visitor
+    {
+    public:
+      size_t area;
+      int    x,y;
+      size_t width,height;
+
+      bool Visit(Object* object)
+      {
+        int    cx,cy;
+        size_t cw,ch;
+
+        cx=x;
+        cy=y;
+        cw=width;
+        ch=height;
+
+        if (object->OIntersect(cx,cy,cw,ch)) {
+          area+=cw*ch;
+        }
+
+        return true;
+      }
+    };
+
+    class SubRegionVisitor : public Visitor
+    {
+    public:
+      OS::DrawInfo *draw;
+
+      bool Visit(Object* object)
+      {
+        OS::Fill* background=object->GetBackground();
+        if (background!=NULL) {
+          if (!background->transparentBorder) {
+            draw->SubClipRegion(object->GetOX(),object->GetOY(),
+                                object->GetOWidth(),object->GetOHeight());
+          }
+          else {
+            draw->SubClipRegion(object->GetOX()+background->leftBorder,
+                                object->GetOY()+background->topBorder,
+                                object->GetOWidth()-background->leftBorder-background->rightBorder,
+                                object->GetOHeight()-background->topBorder-background->bottomBorder);
+          }
+        }
+        return true;
+      }
+    };*/
+
+    class DrawVisitor : public Visitor
+    {
+    public:
+      OS::DrawInfo *draw;
+      Object       *group;
+      int          x,y;
+      size_t       width,height;
+
+      bool Visit(Object* object)
+      {
+        if (object->GetOX()>=x+(int)width) {
+          // Object is right of area
+          return true;
+        }
+        if (object->GetOX()+(int)object->GetOWidth()<=x) {
+          // object is left of area
+          return true;
+        }
+        if (object->GetOY()>=y+(int)height) {
+          // object is below area
+          return true;
+        }
+        if (object->GetOY()+(int)object->GetOHeight()<=y) {
+          // object is above of area
+          return true;
+        }
+
+        group->PreDrawChild(draw,object);
+        object->Draw(x,y,width,height);
+        group->PostDrawChild(draw,object);
+
+        return true;
+      }
+    };
+
+    DrawVisitor drawVisitor;
+
+    drawVisitor.draw=draw;
+    drawVisitor.group=this;
+    drawVisitor.x=x;
+    drawVisitor.y=y;
+    drawVisitor.width=w;
+    drawVisitor.height=h;
+
+    PreDrawChilds(draw);
+    VisitChildren(drawVisitor,true);
+    PostDrawChilds(draw);
   }
 
   /**
@@ -1692,7 +1881,7 @@ namespace Lum {
   void Object::Redraw(int x, int y, size_t w, size_t h)
   {
     if (visible) {
-      GetWindow()->TriggerRedraw(x,y,int(w),int(h));
+      GetWindow()->TriggerRedraw(x,y,w,h);
     }
   }
 
@@ -1718,12 +1907,32 @@ namespace Lum {
   */
   void Object::Hide()
   {
-    visible=false;
+    class HideVisitor : public Visitor
+    {
+    public:
+      OS::DrawInfo *draw;
 
-    // Tell the window, that this object hides
-    // Dialog may delgate focus or other things
-    if (GetWindow()!=NULL && dynamic_cast<Window*>(GetWindow()->GetMaster())!=NULL) {
-      dynamic_cast<Window*>(GetWindow()->GetMaster())->ObjectHides(this);
+      bool Visit(Object* object)
+      {
+        object->Hide();
+        return true;
+      }
+    };
+
+    if (visible) {
+      HideVisitor visitor;
+
+      visitor.draw=GetDrawInfo();
+
+      VisitChildren(visitor,true);
+
+      visible=false;
+
+      // Tell the window, that this object hides
+      // Dialog may delgate focus or other things
+      if (GetWindow()!=NULL && dynamic_cast<Window*>(GetWindow()->GetMaster())!=NULL) {
+        dynamic_cast<Window*>(GetWindow()->GetMaster())->ObjectHides(this);
+      }
     }
   }
 
@@ -1756,6 +1965,11 @@ namespace Lum {
     redrawOnMouseActive=redraw;
   }
 
+  void Object::SetClipFromParent(bool clip)
+  {
+    clipFromParent=clip;
+  }
+
   /**
     Called when the mouse enters the object
   */
@@ -1907,10 +2121,14 @@ namespace Lum {
   }
 
   Scrollable::Scrollable()
-  : hAdjustment(new Model::Adjustment()), vAdjustment(new Model::Adjustment())
+  : hAdjustment(new Model::Adjustment()),
+    vAdjustment(new Model::Adjustment()),
+    updateDimensionsAction(new Model::Action())
   {
     Observe(hAdjustment->GetTopModel());
     Observe(vAdjustment->GetTopModel());
+
+    Observe(updateDimensionsAction);
   }
 
   Scrollable::~Scrollable()
@@ -1918,11 +2136,6 @@ namespace Lum {
     // no code
   }
 
-  void Scrollable::CalculateDimension()
-  {
-    // no code
-  }
-
   void Scrollable::SetHAdjustment(Model::Adjustment* adjustment)
   {
     assert(adjustment!=NULL && adjustment->GetTopModel()!=NULL);
@@ -1955,164 +2168,28 @@ namespace Lum {
     return vAdjustment;
   }
 
-  void Group::Draw(int x, int y, size_t w, size_t h)
+  void Scrollable::Layout()
   {
-    class AreaVisitor : public Visitor
-    {
-    public:
-      size_t area;
-      int    x,y;
-      size_t width,height;
+    UpdateDimensions();
 
-      bool Visit(Object* object)
-      {
-        int    cx,cy;
-        size_t cw,ch;
+    Control::Layout();
+  }
 
-        cx=x;
-        cy=y;
-        cw=width;
-        ch=height;
-
-        if (object->OIntersect(cx,cy,cw,ch)) {
-          area+=cw*ch;
-        }
-
-        return true;
-      }
-    };
-
-    class SubRegionVisitor : public Visitor
-    {
-    public:
-      OS::DrawInfo *draw;
-
-      bool Visit(Object* object)
-      {
-        OS::Fill* background=object->GetBackground();
-        if (background!=NULL) {
-          if (!background->transparentBorder) {
-            draw->SubClipRegion(object->GetOX(),object->GetOY(),
-                                object->GetOWidth(),object->GetOHeight());
-          }
-          else {
-            draw->SubClipRegion(object->GetOX()+background->leftBorder,
-                                object->GetOY()+background->topBorder,
-                                object->GetOWidth()-background->leftBorder-background->rightBorder,
-                                object->GetOHeight()-background->topBorder-background->bottomBorder);
-          }
-        }
-        return true;
-      }
-    };
-
-    class DrawVisitor : public Visitor
-    {
-    public:
-      Group  *group;
-      int    x,y;
-      size_t width,height;
-
-      bool Visit(Object* object)
-      {
-        if (object->GetOX()>=x+(int)width) {
-          // Object is right of area
-          return true;
-        }
-        if (object->GetOX()+(int)object->GetOWidth()<=x) {
-          // object is left of area
-          return true;
-        }
-        if (object->GetOY()>=y+(int)height) {
-          // object is below area
-          return true;
-        }
-        if (object->GetOY()+(int)object->GetOHeight()<=y) {
-          // object is above of area
-          return true;
-        }
-
-        group->PreDrawChild(object);
-        object->Draw(x,y,width,height);
-        group->PostDrawChild(object);
-
-        return true;
-      }
-    };
-
-    Object::Draw(x,y,w,h);
+  void Scrollable::Draw(int x, int y, size_t w, size_t h)
+  {
+    Control::Draw(x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
       return;
     }
 
-    /* --- */
-
-    AreaVisitor areaVisitor;
-    DrawVisitor drawVisitor;
-
-    areaVisitor.area=0;
-    areaVisitor.x=x;
-    areaVisitor.y=y;
-    areaVisitor.width=w;
-    areaVisitor.height=h;
-    VisitChildren(areaVisitor,true);
-
-    if (areaVisitor.area<w*h) {
-      OS::DrawInfo *draw=GetDrawInfo();
-
-      SubRegionVisitor subVisitor;
-
-      subVisitor.draw=draw;
-
-      draw->PushClipBegin(x,y,w,h);
-      VisitChildren(subVisitor,true);
-      draw->PushClipEnd();
-
-      DrawBackground(x,y,w,h);
-
-      draw->PopClip();
+    OS::display->QueueActionForEventLoop(updateDimensionsAction);
+  }
+  void Scrollable::Resync(Base::Model* model, const Base::ResyncMsg& msg)
+  {
+    if (model==updateDimensionsAction && updateDimensionsAction->IsFinished()) {
+      UpdateDimensions();
     }
-
-    drawVisitor.group=this;
-    drawVisitor.x=x;
-    drawVisitor.y=y;
-    drawVisitor.width=w;
-    drawVisitor.height=h;
-
-    VisitChildren(drawVisitor,true);
-  }
-
-  void Group::Hide()
-  {
-    class HideVisitor : public Visitor
-    {
-    public:
-      OS::DrawInfo *draw;
-
-      bool Visit(Object* object)
-      {
-        object->Hide();
-        return true;
-      }
-    };
-
-    HideVisitor visitor;
-
-    if (visible) {
-      VisitChildren(visitor,true);
-      Object::Hide();
-    }
-  }
-
-  void Group::PreDrawChild(Object* /*child*/)
-  {
-    // no code
-  }
-
-  void Group::PostDrawChild(Object* /*child*/)
-  {
-    // no code
   }
 
   List::~List()
@@ -2150,10 +2227,12 @@ namespace Lum {
     Note that some group-objects have a more special functions for adding
     members. However Add should always be supported
   */
-  void List::Add(Object* object)
+  List* List::Add(Object* object)
   {
     object->SetParent(this);
     list.push_back(object);
+
+    return this;
   }
 
   Component::Component()
@@ -2169,12 +2248,11 @@ namespace Lum {
 
   bool Component::VisitChildren(Visitor &visitor, bool onlyVisible)
   {
-    if (container!=NULL) {
-      return container->VisitChildren(visitor,onlyVisible);
-    }
-    else {
-      return true;
+    if (container!=NULL && !visitor.Visit(container)) {
+      return false;
     }
+
+    return true;
   }
 
   bool Component::HasBaseline() const
@@ -2216,31 +2294,11 @@ namespace Lum {
     assert(container!=NULL);
 
     container->MoveResize(x,y,width,height);
+    container->Layout();
 
     Object::Layout();
   }
 
-  void Component::Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    container->Draw(x,y,w,h);
-  }
-
-  void Component::Hide()
-  {
-    if (visible) {
-      container->Hide();
-      Object::Hide();
-    }
-  }
-
   ControlComponent::ControlComponent()
   : container(NULL)
   {
@@ -2254,12 +2312,11 @@ namespace Lum {
 
   bool ControlComponent::VisitChildren(Visitor &visitor, bool onlyVisible)
   {
-    if (container!=NULL) {
-      return container->VisitChildren(visitor,onlyVisible);
-    }
-    else {
-      return true;
+    if (container!=NULL && !visitor.Visit(container)) {
+      return false;
     }
+
+    return true;
   }
 
   bool ControlComponent::HasBaseline() const
@@ -2299,29 +2356,9 @@ namespace Lum {
   void ControlComponent::Layout()
   {
     container->MoveResize(x,y,width,height);
+    container->Layout();
 
     Control::Layout();
   }
-
-  void ControlComponent::Draw(int x, int y, size_t w, size_t h)
-  {
-    Control::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    container->Draw(x,y,w,h);
-  }
-
-  void ControlComponent::Hide()
-  {
-    if (visible) {
-      container->Hide();
-      Control::Hide();
-    }
-  }
 }
 
