@@ -24,8 +24,28 @@
 namespace Lum {
 
   Radio::Radio()
-   : ImageState(OS::Theme::objRadio)
+   : ImageState(OS::display->GetImage(OS::Display::radioImageIndex))
   {
     // no code
   }
+
+  Radio* Radio::Create()
+  {
+    Radio *r;
+
+    r=new Radio();
+
+    return r;
+  }
+
+  Radio* Radio::Create(Base::Model* model, size_t index)
+  {
+    Radio *r;
+
+    r=new Radio();
+    r->SetIndex(index);
+    r->SetModel(model);
+
+    return r;
+  }
 }
