@@ -112,7 +112,9 @@ namespace Lum {
       return true;
     }
 
-    if (!active) {
+    if (!active || !visible || GetModel()==NULL || !GetModel()->IsEnabled() ||
+        (intModel.Valid() && intModel->IsNull()) ||
+        (boolModel.Valid() && boolModel->IsNull())) {
       return false;
     }
 
@@ -173,31 +175,30 @@ namespace Lum {
     Control::Resync(model,msg);
   }
 
-  ImageState::ImageState(OS::Theme::ObjectType objectType, bool active)
+  ImageState::ImageState(OS::Image* image, bool active)
   : State(active),
-    object(objectType)
+    image(image)
   {
+    assert(image!=NULL);
     // no code
   }
 
   void ImageState::CalcSize()
   {
-    OS::display->GetTheme()->GetObjectSize(object,minWidth,minHeight);
-    width=minWidth;
-    height=minHeight;
+    assert(image.Valid());
+
+    width=image->GetWidth();
+    height=image->GetHeight();
 
     State::CalcSize();
   }
 
   void ImageState::DrawContent(int x, int y, size_t w, size_t h)
   {
-    if (OS::display->GetTheme()->NeedsObjectParentDraw(object)) {
-      DrawParentBackground(x,y,width,height);
-    }
+    assert(image.Valid());
 
-    OS::display->GetTheme()->DrawObject(GetDrawInfo(),object,
-                                        this->x,this->y,this->width,this->height,
-                                        x,y,w,h);
+    image->Draw(GetDrawInfo(),
+                this->x,this->y,this->width,this->height);
   }
 
 }
