@@ -26,7 +26,6 @@
 #include <Lum/Array.h>
 #include <Lum/Button.h>
 #include <Lum/Menu.h>
-#include <Lum/Space.h>
 
 namespace Lum {
 
@@ -193,7 +192,7 @@ namespace Lum {
   {
     assert(inited);
 
-    return GetTopBorder()+OS::display->GetTheme()->GetEntryVertTextOffset()+(height-font->height) / 2 + font->ascent;
+    return GetTopBorder()+(height-font->height) / 2 + font->ascent;
   }
 
   bool String::SetModel(Base::Model* model)
@@ -601,24 +600,41 @@ namespace Lum {
     }
   }
 
-  bool String::Drop(const Base::DnDType& type,
-                    Base::DnDData& data,
-                    Base::DnDObject::DnDAction action)
+  bool String::CanExecuteAction(const Base::Datatype& type,
+                                Base::DataExchangeObject::ExchangeAction action) const
   {
-    std::wstring text;
-
-    if (!type.Matches(Base::DnDType::text)) {
-      return false;
-    }
-
-    text=data.GetText();
-
-    if (text.length()==0) {
+    if (!type.Matches(Lum::Base::Datatype::text)) {
       return false;
     }
 
     switch (action) {
-    case dndInsert:
+    case actionMove:
+      return model.Valid() && Selected();
+      break;
+    case actionCopy:
+      return model.Valid() && Selected();
+      break;
+    case actionInsert:
+      return model.Valid();
+      break;
+    default:
+      return false;
+    }
+  }
+
+
+  bool String::ExecuteAction(const Base::Datatype& type,
+                             Base::Data& data,
+                             Base::DataExchangeObject::ExchangeAction action)
+  {
+    std::wstring text;
+
+    if (!CanExecuteAction(type,action)) {
+      return false;
+    }
+
+    switch (action) {
+    case actionInsert:
       if (model->Length()+text.length()>textWidth) {
         return false;
       }
@@ -626,20 +642,28 @@ namespace Lum {
       if (Selected()) {
         DeleteSelection();
       }
+
+      text=data.GetText();
+
       cursor+=text.length();
       model->Insert(cursor-text.length(),text);
-      break;
-    case dndCopy:
-    case dndMove:
-      if (text.length()>textWidth) {
-        return false;
-      }
-
-      model->Set(text);
-      break;
+      return true;
+    case actionCopy:
+    case actionMove:
+      data.SetText(model->Get().substr(markA,markB-markA));
+      return true;
     default:
       return false;
     }
+  }
+
+  bool String::PostprocessAction(Base::DataExchangeObject::ExchangeAction action)
+  {
+    if (!model.Valid() || action!=actionMove || !Selected()) {
+      return false;
+    }
+
+    DeleteSelection();
 
     return true;
   }
@@ -828,6 +852,12 @@ namespace Lum {
     OS::display->RemoveTimer(blinkAction);
   }
 
+  void String::PrepareForBackground(OS::DrawInfo* draw)
+  {
+    draw->focused=HasFocus();
+    draw->disabled=(!model.Valid() || !model->IsEnabled());
+  }
+
   void String::Draw(int x, int y, size_t w, size_t h)
   {
     OS::DrawInfo *draw=GetDrawInfo();
@@ -849,20 +879,11 @@ namespace Lum {
 
     yOff=this->y+(height-font->height) / 2;
 
-    /* Draw background */
-    if (!model.Valid() || !model->IsEnabled()) {
-      draw->disabled=true;
-    }
-
-    DrawBackground(x,y,w,h);
-
-    draw->disabled=false;
-
     /* Draw optional baseline */
     if (baseline) {
-      draw->PushForeground(OS::Display::tableTextColor);
-      draw->DrawLine(this->x+OS::display->GetTheme()->GetEntryHorizTextOffset(),yOff+font->ascent-1,
-                     this->x+width-1-OS::display->GetTheme()->GetEntryHorizTextOffset(),yOff+font->ascent-1);
+      draw->PushForeground(OS::Display::editTextColor);
+      draw->DrawLine(this->x,yOff+font->ascent-1,
+                     this->x+width-1,yOff+font->ascent-1);
       draw->PopForeground();
     }
 
@@ -876,21 +897,21 @@ namespace Lum {
     }
 
     /* x = left starting position of string*/
-    xPos=this->x+OS::display->GetTheme()->GetEntryHorizTextOffset();
-    fieldWidth=width-2*OS::display->GetTheme()->GetEntryHorizTextOffset();
+    xPos=this->x;
+    fieldWidth=width;
 
     /* Calculate starting pos depending on alignment */
     if (textAlign==left) {
       textPos=xPos;
     }
     else if (textAlign==right) {
-      textPos=xPos+fieldWidth-font->StringWidth(string,0);
+      textPos=xPos+fieldWidth-font->StringWidth(string,OS::Font::normal);
     }
     else { /* center */
-      textPos=xPos+(fieldWidth-font->StringWidth(string,0)) / 2;
+      textPos=xPos+(fieldWidth-font->StringWidth(string,OS::Font::normal)) / 2;
     }
 
-    /* Calculate bound of first letter of help^ */
+    /* Calculate bound of first letter */
     font->StringExtent(string.substr(0,1),extent2);
 
     /*
@@ -909,13 +930,7 @@ namespace Lum {
       font->StringExtent(help,extent1);
 
       /* Calculate cursor position */
-      cursorPos+=extent1.width-extent1.right;
-
-      /* correct cursor position by first character of 2nd string */
-      if (cursor<string.length()) {
-        font->StringExtent(string.substr(cursor,1),extent2);
-        cursorPos+=extent2.left;
-      }
+      cursorPos+=extent1.width;
     }
 
     /* Make cursor visible in gadget by correcting starting offset */
@@ -934,7 +949,8 @@ namespace Lum {
 
     draw->PushFont(font,OS::Font::normal);
     if (Selected()) { /* there exists a selection */
-      int markAPos,markBPos;
+      int         markAPos,markBPos;
+      OS::FillRef back=OS::display->GetFill(OS::Display::entryBackgroundFillIndex);
 
       //
       // Text before the selection
@@ -959,7 +975,7 @@ namespace Lum {
         }
 
         draw->PushClip(textPos,this->y,markAPos-textPos,height);
-        draw->PushForeground(OS::Display::tableTextColor);
+        draw->PushForeground(OS::Display::editTextColor);
         draw->DrawString(textPos,yOff+font->ascent,string);
         draw->PopForeground();
         draw->PopClip();
@@ -984,7 +1000,7 @@ namespace Lum {
         markBPos+=extent2.left;
 
         draw->PushClip(markBPos,yOff,width,height);
-        draw->PushForeground(OS::Display::tableTextColor);
+        draw->PushForeground(OS::Display::editTextColor);
         draw->DrawString(textPos,yOff+font->ascent,string);
         draw->PopForeground();
         draw->PopClip();
@@ -995,17 +1011,19 @@ namespace Lum {
       //
 
       draw->selected=true;
-      DrawBackground(markAPos,this->y,markBPos-markAPos+1,height);
+      back->Draw(draw,
+                 oX,oY,oWidth,oHeight,
+                 markAPos,yOff,markBPos-markAPos+1,font->height);
       draw->selected=false;
 
       draw->PushClip(markAPos,yOff,markBPos-markAPos+1,height);
-      draw->PushForeground(OS::Display::fillTextColor);
+      draw->PushForeground(OS::Display::editTextFillColor);
       draw->DrawString(textPos,yOff+font->ascent,string);
       draw->PopForeground();
       draw->PopClip();
     }
     else {
-      draw->PushForeground(OS::Display::tableTextColor);
+      draw->PushForeground(OS::Display::editTextColor);
       draw->DrawString(textPos,yOff+font->ascent,string);
       draw->PopForeground();
     }
@@ -1024,7 +1042,7 @@ namespace Lum {
         draw->PopFont();
       }
       else {
-        draw->PushForeground(OS::Display::tableTextColor);
+        draw->PushForeground(OS::Display::editTextColor);
         draw->FillRectangle(cursorPos,yOff,1,font->height);
         draw->PopForeground();
       }
@@ -1282,6 +1300,40 @@ namespace Lum {
     Control::Resync(model,msg);
   }
 
+  String* String::Create(bool horizontalFlex, bool verticalFlex)
+  {
+    String *s;
+
+    s=new String();
+    s->SetFlex(horizontalFlex,verticalFlex);
+
+    return s;
+  }
+
+  String* String::Create(Base::Model* model, bool horizontalFlex, bool verticalFlex)
+  {
+    String *s;
+
+    s=new String();
+    s->SetFlex(horizontalFlex,verticalFlex);
+    s->SetModel(model);
+
+    return s;
+  }
+
+  String* String::Create(Base::Model* model, size_t widthInChars,
+                         bool horizontalFlex, bool verticalFlex)
+  {
+    String *s;
+
+    s=new String();
+    s->SetFlex(horizontalFlex,verticalFlex);
+    s->SetMinWidth(Lum::Base::Size::stdCharWidth,widthInChars);
+    s->SetModel(model);
+
+    return s;
+  }
+
   String::InputHelper::InputHelper()
    : backAction(new Model::Action())
   {
@@ -1302,9 +1354,7 @@ namespace Lum {
     Button           *button;
     Model::ActionRef action=new Model::Action();
 
-    button=new Button(label);
-    button->SetFlex(true,true);
-    button->SetModel(action);
+    button=Button::Create(label,action,true,true);
 
     RegisterKey(action,text);
 
@@ -1313,13 +1363,7 @@ namespace Lum {
 
   Button* String::InputHelper::CreateTextButton(const std::wstring& label, Model::Action* action)
   {
-    Button           *button;
-
-    button=new Button(label);
-    button->SetFlex(true,true);
-    button->SetModel(action);
-
-    return button;
+    return Button::Create(label,action,true,true);
   }
 
   bool String::InputHelper::Open(String *string)
@@ -1386,7 +1430,7 @@ namespace Lum {
       array->Add(CreateTextButton(L"5",L"5"));
       array->Add(CreateTextButton(L"6",L"6"));
       array->Add(CreateTextButton(L"7",L"7"));
-      array->Add(new VSpace());
+      array->AddEmpty();
       array->Add(CreateTextButton(L"<-",backAction));
     }
     else if (type==typePosDec) {
@@ -1404,7 +1448,7 @@ namespace Lum {
       array->Add(CreateTextButton(L"3",L"3"));
 
       array->Add(CreateTextButton(L"0",L"0"));
-      array->Add(new VSpace());
+      array->AddEmpty();
       array->Add(CreateTextButton(L"<-",backAction));
     }
     else if (type==typeDec) {
@@ -1443,7 +1487,7 @@ namespace Lum {
       array->Add(CreateTextButton(L"D",L"D"));
       array->Add(CreateTextButton(L"E",L"E"));
       array->Add(CreateTextButton(L"F",L"F"));
-      array->Add(new VSpace());
+      array->AddEmpty();
       array->Add(CreateTextButton(L"<-",backAction));
     }
     else if (type==typePosFloat) {
@@ -1483,7 +1527,7 @@ namespace Lum {
 
       array->Add(CreateTextButton(L"0",L"0"));
       array->Add(CreateTextButton(Lum::Base::GetDecimalPointWString(),Lum::Base::GetDecimalPointWString()));
-      array->Add(new VSpace());
+      array->AddEmpty();
       array->Add(CreateTextButton(L"<-",backAction));
     }
 
