@@ -59,10 +59,8 @@ namespace Lum {
   }
 
   TableView::TableView()
-  : selectionAction(new Model::Action),
-    doubleClickAction(new Model::Action),
+  : doubleClickAction(new Model::Action),
     mouseSelectionAction(new Model::Action),
-    updateScrollerAction(new Model::Action),
     autoFitColumns(false),
     autoHSize(false),
     autoVSize(false),
@@ -81,14 +79,15 @@ namespace Lum {
     formatProvider(new FormatProvider()),
     font(OS::display->GetFont())
   {
-    SetBackground(OS::display->GetFill(OS::Display::tableBackgroundFillIndex));
+    SetClipFromParent(true);
+
+    SetBackground(new OS::EmptyFill());
 
     SetCanFocus(true);
     RequestFocus();
 
-    SetController(::Lum::controller);
+    SetController(Lum::controller);
 
-    Observe(updateScrollerAction);
     Observe(scrollTimerAction);
     Observe(mouseTimerAction);
   }
@@ -255,11 +254,6 @@ namespace Lum {
     }
   }
 
-  void TableView::SetSelectionAction(Model::Action* action)
-  {
-    selectionAction=action;
-  }
-
   void TableView::SetMouseSelectionAction(Model::Action* action)
   {
     mouseSelectionAction=action;
@@ -297,11 +291,6 @@ namespace Lum {
     return selection.Get();
   }
 
-  Model::Action* TableView::GetSelectionAction() const
-  {
-    return selectionAction.Get();
-  }
-
   Model::Action* TableView::GetDoubleClickAction() const
   {
     return doubleClickAction.Get();
@@ -459,7 +448,7 @@ namespace Lum {
     return true;
   }
 
-  void TableView::UpdateScroller()
+  void TableView::UpdateDimensions()
   {
     if (model.Valid() && !model->IsNull()) {
       hAdjustment->SetStepSize(font->StringWidth(L"m"));
@@ -487,6 +476,20 @@ namespace Lum {
     }
   }
 
+  void TableView::FillWithBackground(OS::DrawInfo* draw,
+                                     int xPos, int yPos, size_t width, size_t height)
+  {
+    if (!OIntersect(xPos,yPos,width,height)) {
+      return;
+    }
+
+    OS::FillRef fill=OS::display->GetFill(OS::Display::tableBackgroundFillIndex);
+
+    fill->Draw(draw,
+               x,y,this->width,this->height,
+               xPos,yPos,width,height);
+  }
+
   void TableView::DrawCell(OS::DrawInfo* draw,
                            int xPos,
                            int yPos,
@@ -568,6 +571,7 @@ namespace Lum {
 
       object->Resize(width,height);
       object->Move(xPos,yPos+((int)height-(int)object->GetOHeight()) / 2);
+      object->Layout();
 
       if (drawBackground) {
         back->Draw(draw,xPos,yPos,width,height,xPos,yPos,width,height);
@@ -610,7 +614,7 @@ namespace Lum {
   void TableView::DrawLine(OS::DrawInfo* draw, int y, size_t line)
   {
     if (line>model->GetRows()) {
-      DrawBackground(this->x,y,width,rowHeight);
+      FillWithBackground(draw,this->x,y,width,rowHeight);
       return;
     }
 
@@ -770,7 +774,7 @@ namespace Lum {
 
   void TableView::Layout()
   {
-    // If we are still scorlling during a resize event, stop scrolling
+    // If we are still scrolling during a resize event, stop scrolling
     scrollRefresh=false;
 
     CalculateRowHeight();
@@ -779,8 +783,6 @@ namespace Lum {
       FitColumns();
     }
 
-    UpdateScroller();
-
     Scrollable::Layout();
   }
 
@@ -796,80 +798,78 @@ namespace Lum {
 
     CalculateRowHeight();
 
-    if (model.Valid() && model->GetRows()>0) {
-      RecalcTableWidth();
+    Lum::OS::DrawInfo *draw=GetDrawInfo();
 
-      OS::display->QueueActionForEventLoop(updateScrollerAction);
+    if (!model.Valid() || model->GetRows()==0) {
+      FillWithBackground(draw,x,y,w,h);
+      return;
+    }
 
-      Lum::OS::DrawInfo *draw=GetDrawInfo();
+    RecalcTableWidth();
+
+    scrolling=(currentPos!=vAdjustment->GetTop());
+
+    if (OS::display->GetType()==OS::Display::typeTextual) {
+      currentPos=vAdjustment->GetTop();
+      DrawFull(draw);
+      return;
+    }
+
+    if (scrolling && scrollRefresh) {
+      size_t recentPos=currentPos;
+      size_t diff;
+
+      if (currentPos<vAdjustment->GetTop()) {
+        size_t step;
+
+        step=(vAdjustment->GetTop()-currentPos)/STEP;
+        //step=(int)(sqrt(vAdjustment->GetTop()-currentPos)+0.5);
+
+        if (step==0) {
+          step=1;
+        }
+
+        currentPos+=step;
+      }
+      else {
+        size_t step;
+
+        step=(currentPos-vAdjustment->GetTop())/STEP;
+        //step=(int)(sqrt(currentPos-vAdjustment->GetTop())+0.5);
+
+        if (step==0) {
+          step=1;
+        }
+
+        currentPos-=step;
+      }
+
+      diff=std::max(recentPos,currentPos)-std::min(recentPos,currentPos);
+
+      if (diff>=height) {
+        DrawFull(draw);
+      }
+      else if (currentPos>recentPos) {
+        draw->CopyArea(this->x,this->y+diff,width,height-diff,this->x,this->y);
+        DrawRect(draw,height-diff,diff);
+      }
+      else if (recentPos>currentPos) {
+        draw->CopyArea(this->x,this->y,width,height-diff,this->x,this->y+diff);
+        DrawRect(draw,0,diff);
+      }
 
       scrolling=(currentPos!=vAdjustment->GetTop());
 
-      if (OS::display->GetType()==OS::Display::typeTextual) {
-        currentPos=vAdjustment->GetTop();
-        DrawFull(draw);
-        return;
-      }
-
-      if (scrolling && scrollRefresh) {
-        size_t recentPos=currentPos;
-        size_t diff;
-
-        if (currentPos<vAdjustment->GetTop()) {
-          size_t step;
-
-          step=(vAdjustment->GetTop()-currentPos)/STEP;
-          //step=(int)(sqrt(vAdjustment->GetTop()-currentPos)+0.5);
-
-          if (step==0) {
-            step=1;
-          }
-
-          currentPos+=step;
-        }
-        else {
-          size_t step;
-
-          step=(currentPos-vAdjustment->GetTop())/STEP;
-          //step=(int)(sqrt(currentPos-vAdjustment->GetTop())+0.5);
-
-          if (step==0) {
-            step=1;
-          }
-
-          currentPos-=step;
-        }
-
-        diff=std::max(recentPos,currentPos)-std::min(recentPos,currentPos);
-
-        if (diff>=height) {
-          DrawFull(draw);
-        }
-        else if (currentPos>recentPos) {
-          draw->CopyArea(this->x,this->y+diff,width,height-diff,this->x,this->y);
-          DrawRect(draw,height-diff,diff);
-        }
-        else if (recentPos>currentPos) {
-          draw->CopyArea(this->x,this->y,width,height-diff,this->x,this->y+diff);
-          DrawRect(draw,0,diff);
-        }
-
-        scrolling=(currentPos!=vAdjustment->GetTop());
-
-        if (scrolling) {
-          Lum::OS::display->AddTimer(0,1000000/25,scrollTimerAction);
-        }
-        else {
-          scrollRefresh=false;
-        }
+      if (scrolling) {
+        Lum::OS::display->AddTimer(0,1000000/25,scrollTimerAction);
       }
       else {
         scrollRefresh=false;
-        DrawRect(draw,y-this->y,h);
       }
     }
     else {
-      DrawBackground(x,y,w,h);
+      scrollRefresh=false;
+      DrawRect(draw,y-this->y,h);
     }
   }
 
@@ -923,13 +923,6 @@ namespace Lum {
     }
   }
 
-  void TableView::OnSelection()
-  {
-    if (selectionAction.Valid()) {
-      selectionAction->Trigger();
-    }
-  }
-
   void TableView::OnMouseSelection()
   {
     if (mouseSelectionAction.Valid()) {
@@ -1235,7 +1228,6 @@ namespace Lum {
     const Model::Header::FitColumn      *fitColumn;
     const Model::Selection::RefreshCell *selRefreshCell;
     const Model::Selection::RefreshRow  *selRefreshRow;
-    const Model::Selection::Selected    *selected;
 
     if ((refreshCell=dynamic_cast<const Model::Table::RefreshCell*>(&msg))!=NULL) {
       RedrawLine(refreshCell->row);
@@ -1248,6 +1240,11 @@ namespace Lum {
       Redraw(); // TODO: Better calculate to be redrawn region...
     }
     else if ((deleteRow=dynamic_cast<const Model::Table::DeleteRow*>(&msg))!=NULL) {
+      if (selection.Valid() && selection->HasSelection()) {
+        for (size_t r=deleteRow->row; r<deleteRow->row+deleteRow->count; r++) {
+          selection->OnLineDeleted(r);
+        }
+      }
       scrollRefresh=false;
       Redraw(); // TODO: Better calculate to be redrawn region...
     }
@@ -1264,9 +1261,6 @@ namespace Lum {
     else if ((selRefreshRow=dynamic_cast<const Model::Selection::RefreshRow*>(&msg))!=NULL) {
       RedrawLine(selRefreshRow->row);
     }
-    else if ((selected=dynamic_cast<const Model::Selection::Selected*>(&msg))!=NULL) {
-      OnSelection();
-    }
     else { // unknown/empty message
       if (model==this->model) {
         if (selection.Valid() && selection->HasSelection()) {
@@ -1296,10 +1290,9 @@ namespace Lum {
         mouseTimer.SetToNow();
         Lum::OS::display->AddTimer(0,1000000/20,mouseTimerAction);
       }
-      else if (model==updateScrollerAction && updateScrollerAction->IsFinished()) {
-        UpdateScroller();
-      }
     }
+
+    Scrollable::Resync(model,msg);
   }
 
   TableView::Controller::Controller()
