@@ -105,8 +105,6 @@ namespace Lum {
 
     /* --- */
 
-    DrawBackground(x,y,w,h);
-
     if (model.Valid() && !model->IsNull()) {
       OS::DrawInfo        *draw=GetDrawInfo();
       Lum::OS::FontExtent extent;
@@ -154,4 +152,41 @@ namespace Lum {
       Control::Resync(model,msg);
     }
   }
+
+  TextValue* TextValue::Create(bool horizontalFlex, bool verticalFlex)
+  {
+    TextValue* t;
+
+    t=new TextValue();
+    t->SetFlex(horizontalFlex,verticalFlex);
+
+    return t;
+  }
+
+  TextValue* TextValue::Create(Base::Model* model,
+                               bool horizontalFlex, bool verticalFlex)
+  {
+    TextValue* t;
+
+    t=new TextValue();
+    t->SetFlex(horizontalFlex,verticalFlex);
+    t->SetModel(model);
+
+    return t;
+  }
+
+  TextValue* TextValue::Create(Base::Model* model,
+                               Alignment alignment,
+                               bool horizontalFlex, bool verticalFlex)
+  {
+    TextValue* t;
+
+    t=new TextValue();
+    t->SetFlex(horizontalFlex,verticalFlex);
+    t->SetAlignment(alignment);
+    t->SetModel(model);
+
+    return t;
+  }
+
 }
