@@ -78,6 +78,20 @@ namespace Lum {
     State::CalcSize();
   }
 
+  void Toggle::PrepareForBackground(OS::DrawInfo* draw)
+  {
+    if (boolModel.Valid() && boolModel->IsEnabled()) {
+      if (!boolModel->IsNull() && boolModel->Get()) {
+        draw->selected=true;
+      }
+    }
+    else {
+      draw->disabled=true;
+    }
+
+    draw->focused=HasFocus();
+  }
+
   void Toggle::Layout()
   {
     if (label!=NULL) {
@@ -92,10 +106,25 @@ namespace Lum {
 
   void Toggle::DrawContent(int x, int y, size_t w, size_t h)
   {
-    DrawBackground(x,y,w,h);
-
     if (label!=NULL) {
       label->Draw(x,y,w,h);
     }
   }
+
+  Toggle* Toggle::Create(const std::wstring& label,
+                         Base::Model* model,
+                         bool horizontalFlex, bool verticalFlex)
+  {
+    Toggle *t=new Toggle();
+
+    t->SetFlex(horizontalFlex,verticalFlex);
+    t->SetModel(model);
+    t->SetLabel(label);
+
+    return t;
+  }
 }
+
+
+
+
